<?php
namespace HCM\Geral\Entity;

use HCM\Util\Repositories;
use HCM\Util\TkEntity;

class GpemovimentacaoEntity extends TkEntity
{

    private $estrutura;
    private $vinculo;
    private $_nrOrg;
    private $mesCompetencia;
    private $nrtipoestrutlegal;


    public function __construct($entityManager, $hcmParameters){

        parent::__construct(null, $entityManager);
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->mesCompetencia = $hcmParameters->getDtCompetencia();
        $this->nrtipoestrutlegal = $hcmParameters->getTipoEstruturaLegalOrg();
    }

    public function retornaMovimentacaoRecentePorEstrutura(
        $nr_org, $nr_estrutura, $nr_vinculo, $dt_competencia
    ){
        $resultado = current(
             $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->retornaMovimentacaoRecentePorEstrutura(
                $nr_org, $nr_estrutura, $nr_vinculo, $dt_competencia
            )
        );
        return $resultado;
    }

    public function carregaMovimentacoes($nrOrg, $nrVinculo, $ultimoDiaCompetencia){
        $this->resetItems();
        $resultados = Gpemovimentacao::retornaMovimentacoes($nrOrg, $nrVinculo, $ultimoDiaCompetencia);
        foreach ($resultados as $item) {
            $this->addItem($item, $item->getNrmovimentacao());
        }
    }

    public function carregaMovimentacoesPorTipo($nrOrg, $nrVinculo, $tipo, $ultimoDiaCompetencia, $tipoEstrutura){
        $this->resetItems();
        $resultados = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->retornaMovimentacoesPorTipo($nrOrg, $nrVinculo, $tipo, $ultimoDiaCompetencia, $tipoEstrutura);
        foreach ($resultados as $item) {
            $this->addItem($item, $item->getNrmovimentacao());
        }
    }
    
    public function carregaMovimentacoesPorTipoHolerite($nrOrg, $nrVinculo, $tipo, $ultimoDiaCompetencia, $tipoEstrutura){
        $this->resetItems();
        $resultados = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->retornaMovimentacoesPorTipoHolerite($nrOrg, $nrVinculo, $tipo, $ultimoDiaCompetencia, $tipoEstrutura);
        foreach ($resultados as $item) {
            $this->addItem($item, $item->getNrmovimentacao());
        }
    }
    
    public function carregaMovimentacoesPorHierarquia($nrOrg, $nrVinculo, $nrtipoestrutura, $dtIni, $dtFim){
        $this->resetItems();
        $resultados = Gpemovimentacao::retornaMovimentacoesPorHierarquia($nrOrg, $nrVinculo, $nrtipoestrutura, $dtIni, $dtFim);
        foreach ($resultados as $item) {
            $this->addItem($item, $item->getNrmovimentacao());
        }
    }

    public function carregaMovimentacoesPeriodo($nrOrg, $nrVinculo, $dtInicio, $dtFim){
        $this->resetItems();
        $resultados = Gpemovimentacao::retornaMovimentacoesPeriodo($nrOrg, $nrVinculo, $dtInicio, $dtFim);
        foreach ($resultados as $item) {
            $this->addItem($item, $item->getNrmovimentacao());
        }
    }

    public function verificaMovPorTipoTransferencia($nrTpMovTransf, $dtInicioMovimento, $dtRetroativa){

        foreach($this->getItems() as $item) {
            if ($item->getNrtpmovtransfm() === $nrTpMovTransf) {
                if (empty($dtRetroativa) === false) {
                    if ($item->getDtmovimentretro() === $dtRetroativa && ($item->getDtinimoviment() === $dtInicioMovimento)) {
                        return true;
                    }
                } else {
                    if ($item->getDtinimoviment() === $dtInicioMovimento) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    public function analisaPeriodoMovimentacoes($nrVinculo, TkDateTime $dtInicio, TkDateTime $dtFim){

        $data_ini_aux = new TkDateTime();
        $data_fim_aux = new TkDateTime();

        $movimentacoes = $this->getItems();
        $diasMovimentacoes = array();
        foreach ($movimentacoes as $movimentacao) {
            if ($movimentacao->getNrvinculom() === $nrVinculo) {
                // Determina data inicio da movimentacao.
                $data_ini_aux->set($movimentacao->getDtinimoviment(), TkDateTime::FORMAT_DATE);

                // Determina data final da movimentacao.
                if ($movimentacao->getDtfimmoviment() !== null) {
                    $data_fim_aux->set($movimentacao->getDtfimmoviment(), TkDateTime::FORMAT_DATE);
                } else {
                    $data_fim_aux->set($dtFim);
                }

                if ((($data_ini_aux->isLater($dtInicio) === true)
                    ||($data_ini_aux->equals($dtInicio) === true))
                    &&(($data_ini_aux->isEarlier($dtFim) === true)
                    ||($data_ini_aux->equals($dtFim) === true))
                ) {
                    // Comecou e acabou durante.
                    $qtd_dias = $data_ini_aux->daysAfter($data_fim_aux);
                    $diasMovimentacoes[$movimentacao->getNrmovimentacao()] = $qtd_dias;
                } elseif (($data_ini_aux->isEarlier($dtInicio) === true)
                        &&(($data_fim_aux->isEarlier($dtFim) === true)
                        || ($data_fim_aux->equals($dtFim) === true))
                ) {
                    if (($data_fim_aux->isLater($dtInicio) === true)
                        ||($data_fim_aux->equals($dtInicio) === true)
                    ) {
                        // Comecou antes e acabou durante.
                        $qtd_dias = $data_ini_aux->daysAfter($data_fim_aux);
                        $diasMovimentacoes[$movimentacao->getNrmovimentacao()] = $qtd_dias;
                    }
                }
            }
        }
        return $diasMovimentacoes;
    }

    public function incluiMovimentacao(
        $nrOrg, $nrTipoTransfer, $nrVinculom, $nrEstruturam, $dtIniMoviment, $nrTipoEstrutura = null,
        $nrTpMovTransf = null, $dtRetroativa = null, $idAtualizaAcerto = null, $dtFimMovimento = null
    ){
        $movimentacao = new self();
        $movimentacao->setNrorg($nrOrg);
        $movimentacao->setNrtipotransfer($nrTipoTransfer);
        $movimentacao->setNrvinculom($nrVinculom);
        $movimentacao->setNrestruturam($nrEstruturam);
        $movimentacao->setDtinimoviment($dtIniMoviment);
        $movimentacao->setNrtipoestrutura($nrTipoEstrutura);
        $movimentacao->setNrtpmovtransfm($nrTpMovTransf);
        $movimentacao->setDtmovimentretro($dtRetroativa);
        $movimentacao->setIdatualizaacerto($idAtualizaAcerto);
        $movimentacao->setDtfimmoviment($dtFimMovimento);

        $movimentacao->persist();
        return $movimentacao;

    }

    public function retornaMovimentacaoAnterior($nrVinculo, $dtMovimentacao, $cdCagedAdmissao){
        $movimentacaoAnterior = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->retornaMovimentacaoAnterior(
            $this->mesCompetencia, $this->_nrOrg, $nrVinculo, $dtMovimentacao, $cdCagedAdmissao, $this->nrtipoestrutlegal
        );

        return $movimentacaoAnterior;
    }
    
    public function retornaMovimentacoesVinculo($nrOrg, $nrVinculo){
        $resultado = GpeMovimentacao::retornaMovimentacoesVinculo($nrOrg, $nrVinculo);
        return $resultado;
    }

    public function setMovimentacao($nrorg, $nrtipotransfer, $nrvinculom, $nrestruturam, $dtinimoviment,
                                       $nrtipoestrutura, $nrtpmovtransfm, $dtretroativa, $idatualizaacerto,
                                       $dtfimmovimento)
    {
        $this->setNrorg($nrorg);
        $this->setNrtipotransfer($nrtipotransfer);
        $this->setNrvinculom($nrvinculom);
        $this->setNrestruturam($nrestruturam);
        $this->setDtinimoviment($dtinimoviment);
        $this->setNrtipoestrutura($nrtipoestrutura);
        $this->setNrtpmovtransfm($nrtpmovtransfm);
        $this->setDtmovimentretro($dtretroativa);
        $this->setIdatualizaacerto($idatualizaacerto);
        $this->setDtfimmoviment($dtfimmovimento);
    }

    public function getVinculo() {
        if (!$this->vinculo) {
            $this->vinculo = Gpevinculom::getVinculo($this->getNrorg(), $this->getNrvinculom());
        }
        return $this->vinculo;
    }

    public function getEstrutura() {
        if (!$this->estrutura) {
            $this->estrutura = EstruturamEntity::getEntityByWhere(
                'NRORG = ? AND NRESTRUTURAM = ?', array($this->getNrorg(), $this->getNrestruturam())
            );
        }
        return $this->estrutura;
    }
}