<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S2300Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getItensParaEnvioS2300($params) {
        try {
            $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,
       '' as NRTABELA,
       'I' as IDOPERACAO,
       GREATEST(NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC),NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC)) as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN GPE_PESSOAH PH ON PH.NRORG = PE.NRORG
                     AND PH.NRPESSOA = PE.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                               WHERE H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= NVL(:P_DTRESCISAOVINC,SYSDATE))
   AND VM.NRTIPOVINCULOM IN (2,3,10) /*Estagiario, Socio, Autonomo*/
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 1
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
                "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
                "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
                "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
                "P_DTRESCISAOVINC" => isset($params['DTRESCISAOVINC']) ? $params['DTRESCISAOVINC'] : null,
            );
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDadosS2300($parametros) {
        try {
            $dataHistorico = \HCM\Util\DateUtil::getDataDeString($parametros['DTMESCOMPETENC'], \HCM\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA, true)->format("d/m/Y");
            
            /* Se for retificação do evento utiliza a data de envio do original para considerar as mesmas situações da época do primeiro envio */
            $dataEnvioOrig = null;
            if ($parametros['IDOPERACAO'] == 'R') {
                $dataEnvioOrig = $parametros['DTRESPOSTAORIG'];
            }
            
            $dados = $this->getDadosPrincipais($parametros['NRENVIO'], $parametros['NRCHAVE'], $parametros['NRESTRUTURAM'], $parametros['NRTPAMBIENTE'], $dataHistorico, $dataEnvioOrig);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosPrincipais($nrenvio, $nrchave, $nrestruturam, $nrtpambiente, $dtmescompetenc, $dataEnvioOrig) {
        $sql = <<<SQL
/*getDadosS2300*/
SELECT VM.NRVINCULOM CHAVE,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,
       --trabalhador
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') cpfTrab,
       PH.NMPESSOA nmTrab,
       PH.IDSEXOPESSOA sexo,
       RP.CDESOCIAL racaCor,
       EC.CDESOCIAL estCiv,
       GI.CDESOCIAL grauInstr,
       '' nmSoc,
       --nascimento
       TO_CHAR(PH.DTNASCPESSOA,'YYYY-MM-DD') dtNascto,
       PA.CDESOCIAL paisNascto,
       PB.CDESOCIAL paisNac,
       --endereco
       --brasil
       NVL(LG.CDESOCIAL,'O') tpLograd,
       EP.DSENDERECO dscLograd,
       EP.NRIMOVELENDERECO nrLograd,
       SUBSTR(EP.DSCOMPLEENDERECO, 0, 30) complemento,
       EP.NMBAIRROENDERECO bairro,
       REPLACE(REPLACE(REPLACE(EP.NRCEPENDERECO,' '),'.'),'-') cep,
       MP.CDMUNICIBGE brasil_codMunic,
       EP.SGESTADO brasil_uf,
       --exterior
       PC.CDESOCIAL paisResid,
       EE.DSENDERECO exterior_dscLograd,
       EE.NRIMOVELENDERECO exterior_nrLograd,
       SUBSTR(EE.DSCOMPLEENDERECO, 0, 30) exterior_complemento,
       EE.NMBAIRROENDERECO exterior_bairro,
       ME.NMMUNICIPIO nmCid,
       REPLACE(REPLACE(REPLACE(EE.NRCEPENDERECO,' '),'.'),'-') codPostal,
       --trabImig
       DECODE(PB.CDESOCIAL,'105','',DECODE(PH.IDPOSVISTOPERM,'S','2','1')) tmpResid,
       DECODE(PB.CDESOCIAL,'105','','7') condIng,
       --infoDeficiencia
       DECODE(PH.NRCONDFISPES,'2','S','N') defFisica,
       DECODE(PH.NRCONDFISPES,'4','S','N') defVisual,
       DECODE(PH.NRCONDFISPES,'3','S','N') defAuditiva,
       DECODE(PH.NRCONDFISPES,'5','S','N') defMental,
       DECODE(PH.NRCONDFISPES,'8','S','N') defIntelectual,
       DECODE(PH.NRCONDFISPES,'7','S','N') reabReadap,
       '' dsObservacao,
       --<<dependentes>>--
       --contato
       (SELECT REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(MAX(C.CDPREFIXCOMUPARC||C.CDCOMUNICAPARC),' '),'-'),'('),')'),'N/A')
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU = '01') fonePrinc, -- 1-Telefone principal
       (SELECT MAX(C.CDCOMUNICAPARC)
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU = '05') emailPrinc, -- 5-email
       --infoTSVInicio
       CASE WHEN VM.DTADMISSAOVINC < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) THEN 'S' ELSE 'N' END cadIni,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) matricula,
       CE.CDESOCIAL codCateg,
       TO_CHAR(VM.DTADMISSAOVINC,'YYYY-MM-DD') dtInicio,
       '' nrProcTrab,
       CASE WHEN CE.CDESOCIAL IN ('201','202','401','731','734','738') THEN (CASE WHEN VM.NRVINCULOEMPREG IN ('3','4','12','13') THEN '2' ELSE '1' END) ELSE '' END natAtividade,
       --infoComplementares
       --cargoFuncao
       CH.NMOCUPACAOH nmCargo,
       CH.NRCBO CBOCargo,
       FH.NMOCUPACAOH nmFuncao,
       FH.NRCBO CBOFuncao,
       --remuneracao
       CASE WHEN CE.CDESOCIAL IN ('701','711','712','721','722') THEN ROUND(SL.VRSALARIO, 2) ELSE NULL END vrSalFx,
       CASE WHEN CE.CDESOCIAL IN ('701','711','712','721','722') THEN DECODE(SL.IDTPSALARIO,'HORA','1','DIARIO','2','SEMANAL','3','QUINZENAL','4','MENSAL','5','TAREFA','6','7') ELSE '' END undSalFixo,
       CASE WHEN CE.CDESOCIAL IN ('701','711','712','721','722') THEN SUBSTR(SL.DSOBSERVACAO,1,90) ELSE '' END dscSalVar,
       --FGTS
       CASE WHEN CE.CDESOCIAL IN ('721') THEN TO_CHAR(VM.DTOPCAOFGTS,'YYYY-MM-DD') ELSE NULL END dtOpcFGTS,
       --infoDirigenteSindical
       DECODE(CE.CDESOCIAL,'401',CE.NRCATEGESOC,'') categOrig,
       DECODE(CE.CDESOCIAL,'401',REPLACE(REPLACE(REPLACE(ES.CDCNPJESTRUT,'.'),'/'),'-'),'') cnpjOrigem,
       DECODE(CE.CDESOCIAL,'401',TO_CHAR(VM.DTADMISSAOVINC,'YYYY-MM-DD'),'') dtAdmOrig,
       DECODE(CE.CDESOCIAL,'401',VM.CDMATRICULA,'') matricOrig,
       --infoTrabCedido
       DECODE(CE.CDESOCIAL,'410',CE.NRCATEGESOC,'') infoTrabCedido_categOrig,
       DECODE(CE.CDESOCIAL,'410','__cnpjOrigem__','') cnpjCednt,
       DECODE(CE.CDESOCIAL,'410',VM.CDMATRICULA,'') matricCed,
       DECODE(CE.CDESOCIAL,'410',TO_CHAR(VM.DTADMISSAOVINC,'YYYY-MM-DD'),'') dtAdmCed,
       DECODE(CE.CDESOCIAL,'410','1','') tpRegTrab,
       DECODE(CE.CDESOCIAL,'410',VH.NRTPREGIMEPREV,'') tpRegPrev,
       DECODE(CE.CDESOCIAL,'410','__infOnus__','') infOnus,
       --infoEstagiario
       DECODE(CE.CDESOCIAL,'901','N','') natEstagio,
       DECODE(CE.CDESOCIAL,'901',DECODE(GI.CDESOCIAL,'02','1','03','1','04','1','05','1','06','2','07','2','08','4','09','4','8'),'') nivEstagio,
       DECODE(CE.CDESOCIAL,'901','','') areaAtuacao,
       DECODE(CE.CDESOCIAL,'901','','') nrApol,
       DECODE(CE.CDESOCIAL,'901',ROUND(SL.VRSALARIO, 2),'') vlrBolsa,
       DECODE(CE.CDESOCIAL,'901',TO_CHAR(VH.DTFIMCONTRDETERMIN,'YYYY-MM-DD'),'') dtPrevTerm,
       --instEnsino
       DECODE(CE.CDESOCIAL,'901',REPLACE(REPLACE(REPLACE(IEH.CDCNPJESTRUT,'.'),'/'),'-'),'') cnpjInstEnsino,
       DECODE(CE.CDESOCIAL,'901',NVL(IEH.NMRAZSOCESTRUT,FO.DSINSTITUICAO),'') nmRazao,
       DECODE(CE.CDESOCIAL,'901',IEE.DSENDERECO,'') instEnsino_dscLograd,
       DECODE(CE.CDESOCIAL,'901',IEE.NRIMOVELENDERECO,'') instEnsino_nrLograd,
       DECODE(CE.CDESOCIAL,'901',IEE.NMBAIRROENDERECO,'') instEnsino_bairro,
       DECODE(CE.CDESOCIAL,'901',REPLACE(REPLACE(REPLACE(IEE.NRCEPENDERECO,' '),'.'),'-'),'') instEnsino_cep,
       DECODE(CE.CDESOCIAL,'901',MIE.CDMUNICIBGE,'') instEnsino_codMunic,
       DECODE(CE.CDESOCIAL,'901',IEE.SGESTADO,'') instEnsino_uf,
       --ageIntegracao
       --supervisorEstagio
       DECODE(CE.CDESOCIAL,'901',PNS.NRINSCRICAOPARC,'') cpfSupervisor,
       DECODE(CE.CDESOCIAL,'901',PNS.NMPRINCIPALPARC,'') nmSuperv,
       --afastamento
       TO_CHAR(
        CASE
            WHEN SF.DTINISITUFUNC IS NULL AND GO.DTINIGOZOFERIAS IS NULL THEN NULL  /*Situacao Normal*/
            WHEN GO.DTINIGOZOFERIAS < SF.DTINISITUFUNC OR SF.DTINISITUFUNC IS NULL THEN GO.DTINIGOZOFERIAS  /*Em ferias*/
            WHEN SF.DTINISITUFUNC < GO.DTINIGOZOFERIAS OR GO.DTINIGOZOFERIAS IS NULL THEN SF.DTINISITUFUNC  /*Afastado*/
            ELSE NULL   
        END,
       'YYYY-MM-DD') dtIniAfast,
       CASE
        WHEN SF.DTINISITUFUNC IS NULL AND GO.DTINIGOZOFERIAS IS NULL THEN ''  /*Situacao Normal*/
        WHEN GO.DTINIGOZOFERIAS < SF.DTINISITUFUNC OR SF.DTINISITUFUNC IS NULL THEN '15'  /*Em ferias*/
        WHEN SF.DTINISITUFUNC < GO.DTINIGOZOFERIAS OR GO.DTINIGOZOFERIAS IS NULL THEN MA.CDESOCIAL  /*Afastado*/
        ELSE ''
       END codMotAfast,
       --desligamento
       CASE WHEN VM.DTRESCISAOVINC < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) THEN TO_CHAR(VM.DTRESCISAOVINC,'YYYY-MM-DD') ELSE NULL END dtTerm,
       VM.NRVINCULOM, PE.NRORG, PE.NRPARCNEGOCIO
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  JOIN GPE_TIPOVINCULOH VT ON VT.NRTIPOVINCULOM = VM.NRTIPOVINCULOM
                          AND (VT.NRORG = VM.NRORG OR VT.NRORG = 0 OR VT.NRORG IS NULL)
                          AND VT.IDVINCULOEMPR = 'S'
                          AND VT.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_TIPOVINCULOH
                                                    WHERE GPE_TIPOVINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                      AND GPE_TIPOVINCULOH.NRTIPOVINCULOM = VT.NRTIPOVINCULOM
                                                      AND GPE_TIPOVINCULOH.NRORG = VT.NRORG)
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTHISTORICO
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTHISTORICO),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
                                                  
  /*Estrutura*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM
                     
  /*Local de Trabalho*/
  JOIN ESTRUTURAH ET ON ET.NRESTRUTURAM = VH.NRESTRUTGEREN
                    AND ET.NRORG = VH.NRORG
                    AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = ET.NRORG)
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = ET.NRPARCNEGOCIO
                     AND PN.NRORG = ET.NRORG
  /*Dados do Empregado - Dependentes*/
  JOIN GPE_PESSOA PE ON VM.NRPESSOA = PE.NRPESSOA
                    AND VM.NRORG    = PE.NRORG
  JOIN GPE_PESSOAH PH ON PE.NRPESSOA = PH.NRPESSOA
                     AND PE.NRORG    = PH.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTHISTORICO
                                                 AND GPE_PESSOAH.NRPESSOA = PH.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PH.NRORG)
  /*Dados do Empregado - Naturalidade*/
  LEFT JOIN PAIS PA ON PA.CDPAIS = PH.CDPAIS
  LEFT JOIN GPE_NACIONALIDADE NA ON PH.NRNACIONALID = NA.NRNACIONALIDADE
                                AND (PH.NRORG = NA.NRORG OR NA.NRORG = 0)
  LEFT JOIN PAIS PB ON PB.CDPAIS = NA.CDPAIS
  /*Endereço Principal*/
  LEFT JOIN ENDERECOPARC EP ON PE.NRPARCNEGOCIO = EP.NRPARCNEGOCIO
                           AND PE.NRORG = EP.NRORG
                           AND EP.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN MUNICIPIO MP ON MP.CDPAIS = EP.CDPAIS
                        AND MP.SGESTADO = EP.SGESTADO
                        AND MP.CDMUNICIPIO = EP.CDMUNICIPIO
  /*Endereço Exterior*/
  LEFT JOIN ENDERECOPARC EE ON PE.NRPARCNEGOCIO = EE.NRPARCNEGOCIO
                           AND PE.NRORG = EE.NRORG
                           AND EE.CDTIPOENDERECO = 'EXTERIOR'
  LEFT JOIN MUNICIPIO ME ON ME.CDPAIS = EE.CDPAIS
                        AND ME.SGESTADO = EE.SGESTADO
                        AND ME.CDMUNICIPIO = EE.CDMUNICIPIO
  LEFT JOIN PAIS PC ON PC.CDPAIS = EE.CDPAIS
  /*Cargo*/
  LEFT JOIN GPE_OCUPACAOH CH ON CH.NROCUPACAOM = VH.NRCARGO
                            AND CH.NRORG = VH.NRORG
                            AND CH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH
                                                      WHERE GPE_OCUPACAOH.DTMESCOMPETENC <= LAST_DAY(:P_DTMESCOMPETENC)
                                                        AND GPE_OCUPACAOH.NROCUPACAOM = CH.NROCUPACAOM
                                                        AND GPE_OCUPACAOH.NRORG = CH.NRORG)
  /*Função*/
  LEFT JOIN GPE_OCUPACAOH FH ON FH.NROCUPACAOM = VH.NRFUNCAO
                            AND FH.NRORG = VH.NRORG
                            AND FH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH
                                                      WHERE GPE_OCUPACAOH.DTMESCOMPETENC <= LAST_DAY(:P_DTMESCOMPETENC)
                                                        AND GPE_OCUPACAOH.NROCUPACAOM = FH.NROCUPACAOM
                                                        AND GPE_OCUPACAOH.NRORG = FH.NRORG)
  /*Salário*/
  LEFT JOIN GPE_ALTESALARIO SL ON SL.NRVINCULOM = VM.NRVINCULOM
                              AND SL.NRORG = VM.NRORG
                              AND SL.DTALTESALARIO = (SELECT MAX(DTALTESALARIO) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO <= LAST_DAY(:P_DTMESCOMPETENC)
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
                              AND SL.NROCORRENCIA =  (SELECT MAX(NROCORRENCIA) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO = SL.DTALTESALARIO
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
  /*Horário*/
  LEFT JOIN GPE_ESCALATRABH EH ON EH.NRESCALATRABM = VH.NRESCALATRABM
                              AND EH.NRORG = VH.NRORG
                              AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_ESCALATRABH
                                                        WHERE GPE_ESCALATRABH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND GPE_ESCALATRABH.NRESCALATRABM = EH.NRESCALATRABM
                                                          AND GPE_ESCALATRABH.NRORG = EH.NRORG)
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG)    
  LEFT JOIN PTO_TIPOJORNADA TJ ON TJ.NRTPJORNADA = EH.NRTPJORNADA
                              AND (TJ.NRORG = EH.NRORG OR TJ.NRORG = 0)
  /*Afastamento - Situacao Funcional*/
  LEFT JOIN (SELECT SF.NRORG, SF.NRVINCULOM, SF.DTINISITUFUNC, SF.DTFIMSITUFUNC, SF.NRMOTIVOAFASTA
               FROM GPE_ALTESITUFUNC SF
               JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                                     AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                                           WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                             AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                             AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
              WHERE SF.NRORG = :P_NRORG
                AND SF.NRVINCULOM = :P_NRCHAVE
                AND NVL(:P_DTENVIOORIG,SYSDATE) BETWEEN SF.DTINISITUFUNC AND NVL(SF.DTFIMSITUFUNC,NVL(:P_DTENVIOORIG,SYSDATE)) /*P_DTENVIOORIG para casos de retificacao*/
                AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
                     OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
                AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
             ) SF ON SF.NRVINCULOM = VH.NRVINCULOM
                 AND SF.NRORG = VH.NRORG
                 AND SF.DTINISITUFUNC < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR))
                 AND (SF.DTFIMSITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) OR SF.DTFIMSITUFUNC IS NULL)
  /*Motivo Afastamento*/
  LEFT JOIN GPE_MOTIVOAFASTA MA ON MA.NRMOTIVOAFASTA = SF.NRMOTIVOAFASTA
                               AND (MA.NRORG = SF.NRORG OR MA.NRORG = 0)
  /*Afastamento - Ferias*/
  LEFT JOIN (SELECT FE.NRORG, FE.NRVINCULOM, GO.DTINIGOZOFERIAS, GO.DTFIMGOZOFERIAS
               FROM FPA_GOZOFERIAS GO
               JOIN FPA_FERIAS FE ON FE.NRORG = GO.NRORG
                                 AND FE.NRFERIAS = GO.NRFERIAS
              WHERE GO.NRORG = :P_NRORG
                AND NVL(:P_DTENVIOORIG,SYSDATE) BETWEEN GO.DTINIGOZOFERIAS AND NVL(GO.DTFIMGOZOFERIAS,NVL(:P_DTENVIOORIG,SYSDATE)) /*P_DTENVIOORIG para casos de retificacao*/
             ) GO ON GO.NRORG = VH.NRORG
                 AND GO.NRVINCULOM = VH.NRVINCULOM
                 AND GO.DTINIGOZOFERIAS < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR))
                 AND (GO.DTFIMGOZOFERIAS >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) OR GO.DTFIMGOZOFERIAS IS NULL)
  /*Sindicato*/
  LEFT JOIN ESTRUTURAH ES ON ES.NRESTRUTURAM = VH.NRESTRUTSIND
                         AND ES.NRORG = VH.NRORG
                         AND ES.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                   WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                     AND ESTRUTURAH.NRESTRUTURAM = ES.NRESTRUTURAM
                                                     AND ESTRUTURAH.NRORG = ES.NRORG)
  /*RaçaCor*/
  LEFT JOIN GPE_RACAPESSOA RP ON RP.NRRACAPESSOA = PH.NRRACAPESSOA
                             AND (RP.NRORG = PH.NRORG OR RP.NRORG = 0)
  /*Estado Civil*/
  LEFT JOIN ESTADOCIVIL EC ON EC.CDESTACIVIL = PH.CDESTACIVIL
                          AND (EC.NRORG = PH.NRORG OR EC.NRORG = 0)
  /*Grau Instrução*/
  LEFT JOIN GPE_GRAUINSTR GI ON GI.NRGRAUINSTR = PH.NRGRAUINSTR
                            AND (GI.NRORG = PH.NRORG OR GI.NRORG = 0)
  /*Logradouro*/
  LEFT JOIN LOGRADOURO LG ON LG.CDLOGRADOURO = EP.CDLOGRADOURO
                         AND (LG.NRORG = EP.NRORG OR LG.NRORG = 0)
  /*Instituição Ensino*/
  LEFT JOIN DES_FORMACAO FO ON FO.NRORG = PE.NRORG
                           AND FO.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                           AND FO.DTINICIO = (SELECT MAX(DTINICIO) FROM DES_FORMACAO
                                               WHERE NRORG = PE.NRORG
                                                 AND NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                                                 AND DTINICIO <= LAST_DAY(:P_DTMESCOMPETENC))
  LEFT JOIN ESTRUTURAH IEH ON IEH.NRORG = FO.NRORG
                          AND IEH.NRESTRUTURAM = FO.NRESTRUTURAM
                          AND IEH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                     WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND NRESTRUTURAM = IEH.NRESTRUTURAM
                                                       AND NRORG = IEH.NRORG)
  LEFT JOIN ENDERECOPARC IEE ON IEE.NRPARCNEGOCIO = IEH.NRPARCNEGOCIO
                            AND IEE.NRORG = IEH.NRORG
                            AND IEE.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN MUNICIPIO MIE ON MIE.CDPAIS = IEE.CDPAIS
                         AND MIE.SGESTADO = IEE.SGESTADO
                         AND MIE.CDMUNICIPIO = IEE.CDMUNICIPIO
  LEFT JOIN PAIS PIE ON PIE.CDPAIS = IEE.CDPAIS
  /*Supervisor Estagio*/
  LEFT JOIN PARCNEGOCIO PNS ON PNS.NRPARCNEGOCIO = IEH.NRPARCRESPINF
                           AND PNS.NRORG = IEH.NRORG
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Parametro do eSocial*/
  LEFT JOIN ESO_PARAMETRO PARAM ON PARAM.NRORG = :P_NRORG
                            AND PARAM.NRESTRUTURAM = :P_NRESTRUTURAM
                            AND PARAM.DTINIVIGENCIA <= :P_DTMESCOMPETENC
                            AND (PARAM.DTFIMVIGENCIA IS NULL OR PARAM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND VM.NRTIPOVINCULOM IN (2,3,10) --Estagiario, Socio, Autonomo
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidNaoPerProducao(),
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidNaoPerProdRest(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidNaoPerNacioSimples(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_DTHISTORICO" => $dtmescompetenc,
            "P_DTENVIOORIG" => $dataEnvioOrig,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getDadosS2300dependente($parametros) {
        try {
            $dados = $this->getDadosdependente($parametros['NRORG'], $parametros['NRPARCNEGOCIO'], $parametros['NRVINCULOM']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosdependente($nrorg, $nrparcnegocio, $nrvinculom) {
        $sql = <<<SQL
/*getDadosS2300dependente*/
SELECT DISTINCT LPAD(T.CDESOCIAL,2,'0') tpDep,
       H.NMPESSOA nmDep,
       TO_CHAR(H.DTNASCPESSOA,'YYYY-MM-DD') dtNascto,
       REPLACE(REPLACE(H.NRCPFPESSOA, '.'), '-') cpfDep,
       NVL((SELECT 'S' FROM FPA_DEPVINCULO WHERE NRVINCULOM = :P_NRVINCULOM AND NRPESSOA = P.NRPESSOA AND NRORG = P.NRORG AND NRTIPODEPENDE = '2'),'N') depIRRF,
       NVL((SELECT 'S' FROM FPA_DEPVINCULO WHERE NRVINCULOM = :P_NRVINCULOM AND NRPESSOA = P.NRPESSOA AND NRORG = P.NRORG AND NRTIPODEPENDE = '1'),'N') depSF,
       DECODE(NVL(H.NRCONDFISPES,1),1,'N','S') incTrab
  FROM RELACIONAPARC R
  JOIN TIPORELACIONA T ON (T.NRORG = R.NRORG OR T.NRORG = 0)
                      AND T.NRTIPORELACIONA = R.NRTIPORELACIONA
  JOIN GPE_PESSOA P ON R.NRPARCNEGRELAC = P.NRPARCNEGOCIO
                   AND R.NRORG = P.NRORG
  JOIN GPE_PESSOAH H ON P.NRPESSOA = H.NRPESSOA
                    AND P.NRORG = H.NRORG
                    AND H.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                             WHERE H1.NRPESSOA = P.NRPESSOA
                                               AND H1.NRORG = P.NRORG
                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN FPA_DEPVINCULO D ON D.NRPESSOA = P.NRPESSOA
                       AND D.NRORG = P.NRORG
                       AND D.NRVINCULOM = :P_NRVINCULOM
 WHERE R.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
   AND R.NRORG = :P_NRORG
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRPARCNEGOCIO" => $nrparcnegocio,
            "P_NRVINCULOM" => $nrvinculom,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
        );
        return $this->connection->fetchAll($sql, $params);
    }

}