<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Factory\ESocialFactory;
use HCM\Geral\Repository\ESocialItensCadTabRepository;
use HCM\Geral\Repository\ESocialItensNaoPerRepository;
use HCM\Geral\Repository\ESocialItensPeriodRepository;
use HCM\Geral\Repository\ESocialRepository;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\FPA\Calculos\Geral\Inicializa;
use HCM\Util\Builder\Template;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\ModeloImportacao\BasicEntityESocial;
use HCM\Util\ModeloImportacao\DataBuilderESocial;
use HCM\Util\MsgProcesso;
use HCM\Util\Repositories;
use HCM\Util\Util;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Remote\FileUploader\Server;

class ESocialService {

    const TPAMB_PROD = 1;
    const TPAMB_PROD_REST_REAL = 2;
    const TPAMB_PROD_REST_FICT = 3;
    
    const URL_PRODREST = 'https://webservices.producaorestrita.esocial.gov.br';
    const URL_ENVIO_PRODUCAO = 'https://webservices.envio.esocial.gov.br';
    const URL_CONSULTA_PRODUCAO = 'https://webservices.consulta.esocial.gov.br';
    const LOCATION_ENVIO = '/servicos/empregador/enviarloteeventos/WsEnviarLoteEventos.svc';
    const LOCATION_CONSULTA = '/servicos/empregador/consultarloteeventos/WsConsultarLoteEventos.svc';
    const FUNCTION_ENVIO = 'EnviarLoteEventos';
    const FUNCTION_CONSULTA = 'ConsultarLoteEventos';
    
    const IDSTATUS_PENDENTE = 'P';
    const IDSTATUS_ERRO_ENVIO = 'E';
    const IDSTATUS_PROCESSAMENTO = 'M';
    const IDSTATUS_ERRO_PROCESSAMENTO = 'R';
    const IDSTATUS_CONCLUIDO = 'C';
    const IDSTATUS_CONCLUIDO_ADVERTENCIA = 'A';

    /* Códigos de retorno do envio do lote */
    const CD_RET_LOTE_201 = '201';     // Lote recebido com sucesso
    const CD_RET_LOTE_202 = '202';     // Lote recebido com advertências
    const CD_RET_LOTE_301 = '301';     // Erro servidor eSocial
    const CD_RET_LOTE_401 = '401';     // Lote incorreto - Erro preenchimento
    const CD_RET_LOTE_402 = '402';     // Lote incorreto - Schema inválido
    const CD_RET_LOTE_403 = '403';     // Lote incorreto - Versão do schema não permitida
    const CD_RET_LOTE_404 = '404';     // Lote incorreto - Erro certificado
    const CD_RET_LOTE_405 = '405';     // Lote incorreto - Lote nulo ou vazio
    /* Códigos de retorno da consulta de processamento do lote */
    const CD_RET_CONS_101 = '101';     // Lote aguardando processamento
    const CD_RET_CONS_201 = '201';     // Lote Processado com Sucesso.
    const CD_RET_CONS_202 = '202';     // Lote Processado com Advertências.
    const CD_RET_CONS_301 = '301';     // Erro Servidor eSocial
    const CD_RET_CONS_401 = '401';     // Lote Incorreto - Erro preenchimento.
    const CD_RET_CONS_402 = '402';     // Lote Incorreto - Schema Inválido.
    const CD_RET_CONS_403 = '403';     // Lote Incorreto - Versão do Schema não permitida.
    const CD_RET_CONS_404 = '404';     // Lote Incorreto - Erro Certificado.
    const CD_RET_CONS_405 = '405';     // Lote Incorreto - Lote nulo ou vazio.
    const CD_RET_CONS_501 = '501';     // Solicitação de Consulta Incorreta -Erro Preenchimento.
    const CD_RET_CONS_502 = '502';     // Solicitação de Consulta Incorreta - Schema Inválido.
    const CD_RET_CONS_503 = '503';     // Solicitação de Consulta Incorreta - Versão do Schema Não Permitida.
    const CD_RET_CONS_504 = '504';     // Solicitação de Consulta Incorreta - Erro Certificado.
    const CD_RET_CONS_505 = '505';     // Solicitação de Consulta Incorreta - Consulta nula ou vazia.
    /* Códigos de retorno de processamento do evento */
    const CD_RET_EVEN_201 = '201';     // Sucesso
    const CD_RET_EVEN_202 = '202';     // Sucesso com advertência
    const CD_RET_EVEN_301 = '301';     // Erro Servidor
    const CD_RET_EVEN_401 = '401';     // Erro no conteúdo do evento
    const CD_RET_EVEN_402 = '402';     // Schema inválido
    const CD_RET_EVEN_403 = '403';     // Leiaute inválido
    const CD_RET_EVEN_404 = '404';     // Erro do certificado digital da assinatura do evento
    const CD_RET_EVEN_405 = '405';     // Erro na assinatura evento
    const CD_RET_EVEN_406 = '406';     // Evento não pertence ao grupo especificado no lote de eventos
    const CD_RET_EVEN_411 = '411';     // Assinante inválido

    const ERRO = 'Erro';
    const ADVERTENCIA = 'Advertência';

    private $_tempEsocialPath;

    private $_xmlTemplatesPath;
    private $_xsdTemplatesPath;
    private $_certificatesPath;
    private $_tempCertificatesPath;

    private $_tpInscTransmissor;
    private $_nrInscTransmissor;
    private $_tpInscEmpregador;
    private $_nrInscEmpregador;
    
    private $_certFile;
    private $_certPass;
    private $_certKeyFile;
    
    private $_wsLocation;
    private $_wsUri;
    private $_wsWsdl;
    private $_wsFuncion;
    
    private $_wsConsultaAction;
    private $_wsConsultaUrl;
    private $_wsConsultaWsdl;

    private $_xmlFilesPath;

    private $_arrXmlCaracter;
    private $_arrXmlEscape;

    private $_tpAmb;
    private $_procEmi;
    private $_verProc;

    private $_tipoOcorRetorno;
    
    private $_tagsTrad;
    private $_tagsOpc;
    private $_tagNivel;
    
    private $_vinculom;
    
    public $arrayMensagens = array(
    "HCM09000_ESO_CALLED_EVENTO_ERROR" => "Erro ao montar dados de envio para o evento informado",
    "HCM09000_ESO_CARGAINI_EXISTS_ERROR" => "Processo de carga inicial já realizado para a estrutura",
    "HCM09000_ESO_CERTIFICADO_NOT_FOUND_ERROR" => "Certificado do transmissor não encontrado. Favor carregar o certificado na tela 'Parametros do eSocial'.",
    "HCM09000_ESO_CERTIFICADO_NOT_OPEN_ERROR" => "Não foi possível ler o certificado do transmissor. Favor tentar novamente.",
    "HCM09000_ESO_CERTIFICADO_VALIDADE_ERROR" => "Certificado do transmissor com validade vencida. Favor carregar um novo certificado na tela 'Parametros do eSocial'.",
    "HCM09000_ESO_ENVIOS_NOT_FOUND_ERROR" => "Não foram encontrados itens para filtro informado",
    "HCM09000_ESO_EXCLUSAO_ERROR" => "Não é possível excluir eventos que já foram enviados",
    "HCM09000_ESO_FILE_LOTE_NOT_FOUND_ERROR" => "Arquivo de lote não encontrado para envio",
    "HCM09000_ESO_GENERATE_CERT_KEY_ERROR" => "Certificado inválido ou senha inválida",
    "HCM09000_ESO_IDEVENTO_INVALID_ERROR" => "Evento com ID inválido. Favor verificar a inscrição do empregador e seu tipo (CNPJ ou CPF), e reenviar o evento",
    "HCM09000_ESO_INSCR_TRANSM_NOT_FOUND_ERROR" => "O responsável pela transmissão informado, não possui inscrição",
    "HCM09000_ESO_EVENTO_MATRIZ_ERROR" => "O evento só pode ser enviado por uma estrutura que possua um CNPJ Matriz",
    "HCM09000_ESO_LOTE_QTDE_ERROR" => "O lote pode ser composto, no máximo, por 50 eventos",
    "HCM09000_ESO_LOTE_STATUS_ERROR" => "O lote só pode ser transmitido com eventos pendentes",
    "HCM09000_ESO_LOTE_VAZIO_ERROR" => "Não foram encontrados dados para envio do lote",
    "HCM09000_ESO_PARAMETRO_NOT_FOUND_ERROR" => "Parãmetro do eSocial não encontrado para a estrutura",
    "HCM09000_ESO_PRODREST_OPER_ERROR" => "Operação válida somente para o ambiente de Produção Restrita",
    "HCM09000_ESO_SENHA_CERTIFICADO_NOT_FOUND_ERROR" => "Senha do certificado do transmissor não encontrada. Favor informar a senha na tela 'Parametros do eSocial'.",
    "HCM09000_ESO_TPINSCEMPREG_ERROR" => "Tipo de inscrição do empregador inválido. O empregador deve ser pessoa júridica ou pessoa física",
    "HCM09000_ESO_TPINSCTRANSM_ERROR" => "Tipo de inscrição do responsável pela transmissão inválido. O responsável deve ser pessoa júridica ou pessoa física",
    "HCM09000_ESO_UNIQUE_ENVIO_ERROR" => "Já existe envio para evento com mesmos parâmetros");

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, FPA $fpa, ESocialFactory $eSocialFactory, ESocialRepository $eSocialRepository,
                        ESocialItensCadTabRepository $eSocialItensCadTabRepository, ESocialItensNaoPerRepository $eSocialItensNaoPerRepository, ESocialItensPeriodRepository $eSocialItensPeriodRepository,
                        /*Server*/ $zhFileUploader) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->fpa = $fpa;
        $this->eSocialFactory = $eSocialFactory;
        $this->eSocialRepository = $eSocialRepository;
        $this->eSocialItensCadTabRepository = $eSocialItensCadTabRepository;
        $this->eSocialItensNaoPerRepository = $eSocialItensNaoPerRepository;
        $this->eSocialItensPeriodRepository = $eSocialItensPeriodRepository;
        $this->zhFileUploader = $zhFileUploader;

        $this->_tempEsocialPath = $this->hcmParameters->getTempDirectory() . DIRECTORY_SEPARATOR . "esocial";

        $this->_xmlTemplatesPath = ESOCIAL_FILES_PATH . DIRECTORY_SEPARATOR . "templates" . DIRECTORY_SEPARATOR . "xml";
        $this->_xsdTemplatesPath = ESOCIAL_FILES_PATH . DIRECTORY_SEPARATOR . "templates" . DIRECTORY_SEPARATOR . "xsd";
        $this->_certificatesPath = $this->hcmParameters->getFileServerDirectory();
        $this->_tempCertificatesPath = $this->_tempEsocialPath . DIRECTORY_SEPARATOR . "certificates" . DIRECTORY_SEPARATOR . $this->hcmParameters->getNrorgTrabalho();

        $this->_xmlFilesPath = $this->_tempEsocialPath . DIRECTORY_SEPARATOR . "xmlFiles" . DIRECTORY_SEPARATOR . $this->hcmParameters->getNrorgTrabalho();

        $this->_arrXmlCaracter = array('>', '<', '&', '"', "'");
        $this->_arrXmlEscape = array('&gt;', '&lt;', '&amp;', '&quot;', '&#39');

        $this->_procEmi = 1;
        $this->_verProc = $this->hcmParameters->getProjectVersion();

        $this->_tipoOcorRetorno = array(
            '1' => self::ERRO,
            '2' => self::ADVERTENCIA,
        );
            
        $this->_tagsTrad = array();
        $this->_tagsOpc = array();
        $tagsTrad = $this->eSocialRepository->getTagName($this->hcmParameters->getNrorgPadrao() ? $this->hcmParameters->getNrorgPadrao() : 0);
        foreach ($tagsTrad as $tag) {
            $this->_tagsTrad[$tag['DSTAGNAME']] = $tag['DSTAGTRADUCAO'] != '' ?  $tag['DSTAGTRADUCAO'] : $tag['DSTAGNAME'];
            $cdOpcoes = $tag['DSCDOPCTAG'] ? explode(';', $tag['DSCDOPCTAG']) : null;
            $nmOpcoes = $tag['DSNMOPCTAG'] ? explode(';', $tag['DSNMOPCTAG']) : null;
            if($cdOpcoes){
                foreach($cdOpcoes as $index => $cdOpcao){
                    $this->_tagsOpc[$tag['DSTAGNAME']][$cdOpcao] = $nmOpcoes[$index];
                }
            }
        }
        
        // Variável temporária para gravar e ler os arquivos no servidor AWS para evitar tráfego com o FileServer.
        $this->filesDir = __DIR__ . "/../../../../../../../files/";
        $this->filesDirESocial = "/eSocial/";
        
        // Variável temporária para salvar log de execução do eSocial
        $this->logId = $this->hcmParameters->getNrorgTrabalho() . '.' . $this->hcmParameters->getCdoperador() . '.' . DateUtil::getDataAtual()->format('YmdHis');
        $this->logContent = "========================================= Start log at " . DateUtil::getDataAtual()->format('Y/m/d H:i:s') . " [" . $this->hcmParameters->getNrorgTrabalho() . "]\n";
        $this->gravaLog = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->
            buscavalor($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getDtCompetencia(), 'ESOCIAL_GRAVA_LOG');
    }

    /**
     * Retorna todas as versões do eSocial disponíveis na tabela ESO_EVENTO
     * @param $idativo
     * @return array
     */
    public function getVersoesESocial($idativo) {
        return $this->eSocialRepository->getVersoesESocial($idativo);
    }
    
    /**
     * Atualiza o idativo do envio selecionado
     * @param $nrorg
     * @param $nrestruturam
     * @param $nrenvio
     * @return 
     */
    public function inativaEvento($nrorg, $nrestruturam, $nrenvio){
        return $this->eSocialRepository->inativaEvento($nrorg, $nrestruturam, $nrenvio);
    }
        
    /**
     * Retorna todas as estruturas legais da organização com os parametros do esocial
     * @param $nrorg
     * @param $dtmescompetenc
     * @param $nrtipoestrutlegal
     * @return array
     */
    public function getEstruturasESocial($nrorg, $dtmescompetenc, $nrtipoestrutlegal, $idGrupo, $conditions) {
        if(is_array($idGrupo) && $idGrupo[0] == 'P'){
            $idGrupo = 'P';
        }
        return $this->eSocialRepository->getEstruturasESocial($nrorg, $dtmescompetenc, $nrtipoestrutlegal, $idGrupo, $conditions);
    }
    
    /**
     * Retorna o evento do eSocial
     * @param $nmevento
     * @param $dsversao
     * @return object
     */
    public function getEvento($nrevento) {
        try {
            return $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nrevento' => $nrevento));
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Retorna o evento do eSocial
     * @param $nmevento
     * @param $dsversao
     * @return object
     */
    public function getEventoPorNomeVersao($nmevento, $dsversao) {
        try {
            return $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nmevento' => $nmevento, 'dsversao' => $dsversao));
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Retorna o evento de origem do evento de exclusão do eSocial
     * @param $nrorg
     * @param $nrenvio
     * @return object
     */
    public function getEnvioCadastro($nrorg, $nrenvio) {
        try {
            return $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findOneBy(array('nrorg' => $nrorg, 'nrenvio' => $nrenvio));
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Retorna o evento do eSocial
     * @param $nrorg
     * @param $nrenvio
     * @return object
     */
    public function getEnvioCadastroOrigemExcl($nrorg, $nrenvio) {
        try {
            return $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findOneBy(array('nrorg' => $nrorg, 'nrenvioexcl' => $nrenvio));
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function geraMatriculaESocial($nrorg, $nrestruturam){
        $dtmescompetenc = $this->hcmParameters->getDtcompetencia();
        return $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->retonaUltimaMatriculaEsoIncrementada($nrorg, $nrestruturam, $dtmescompetenc);
    }
    
    /**
     * Realiza validações antes de inserir um novo registro
     * @param $nrevento
     * @param $nrchave
     * @param $nrtabela
     * @param $idoperacao
     * @param $dtmescompetenc
     * @throws \Exception
     */
    public function beforeInsertEnvioCadastros($nrestruturam, $nrtpambiente, $nrevento, $nrchave, $nrtabela, $idoperacao, $dtmescompetenc, $jenkins = false) {
        try {
            $dtCompetencia = DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            $evento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nrorg' => $this->hcmParameters->getNrorgPadrao(), 'nrevento' => $nrevento));
            
            if ($evento->getIdgrupo() == 'P') {
                $eventoFechamento = $this->validaFechamentoEstrutura($this->hcmParameters->getNrorgTrabalho(), $nrestruturam, $dtCompetencia, $nrtabela);
                $eventoReabertura = $this->validaReaberturaEstrutura($this->hcmParameters->getNrorgTrabalho(), $nrestruturam, $dtCompetencia, $nrtabela);
                if ($evento->getNmevento() == 'S-1299' && $eventoFechamento && !$eventoReabertura) {
                    throw new \Exception(Messages::evaluateMessage(Messages::HCM09000_ESO_ESTRUTURA_FECHAMENTO_EXISTS_ERROR, $nrestruturam));
                }
                if ($evento->getNmevento() == 'S-1298' && !$eventoFechamento) {
                    throw new \Exception('Não foi encontrado o evento de fechamento para a estrutura ' . $nrestruturam);
                }
            }
            
            if ($evento->getNmevento() != 'S-1298' && $evento->getNmevento() != 'S-1299') {
                //@todo Validar se já existe o evento com a mesma chave, mesma operação e mesma competência (*analisar)
                //$envios = $this->eSocialRepository->getUniqueEnvioCadastros($nrestruturam, $nrtpambiente, $nmevento, $nrchave, $idoperacao, $dtmescompetenc);
                $envios = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosCadastrosEvento(
                    $this->hcmParameters->getNrorgTrabalho(), $evento->getNmevento(), $nrestruturam, $nrtpambiente, 'S', $nrchave, ($nrtabela==""?null:$nrtabela), [$idoperacao, 'I', 'R']
                );
                /*$envios = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findBy(
                    array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrestruturam' => $nrestruturam, 'nrtpambiente' => $nrtpambiente, 'nrevento' => $nrevento, 'nrchave' => $nrchave, 'nrtabela' => ($nrtabela==""?null:$nrtabela), 'idoperacao' => [$idoperacao, 'I', 'R'], 'idativo' => 'S', 'nrenvioexcl' => null),
                    array('dtmescompetenc' => 'ASC')
                );*/
                if (!empty($envios)) {
                    foreach ($envios as $envio) {
                        if ($evento->getIdgrupo() == 'P') {
                            // Para periódicos valida se já existe o evento com a mesma chave, mesma operação e mesma competência
                            // Se não for mesma competência pode inserir
                            if ($envio->getDtmescompetenc() == $dtCompetencia) {
                                if($jenkins){
                                    return true;
                                }else{
                                    throw new \Exception(Messages::HCM09000_ESO_UNIQUE_ENVIO_ERROR,Messages::ALERT_MESSAGE_CODE);
                                }
                            }
                        } else {
                            // Valida se já existe o evento com a mesma chave, mesma operação e mesma competência ou competência maior
                            if ($envio->getDtmescompetenc() >= $dtCompetencia) {
                                if($jenkins){
                                    return true;
                                }else{
                                    throw new \Exception(Messages::HCM09000_ESO_UNIQUE_ENVIO_ERROR,Messages::ALERT_MESSAGE_CODE);
                                }
                            } else {
                                // Se não existir verifica se existe outro com competencia anterior e exlui para não gerar dois envios para o mesmo histórico
                                if ($envio->getDtmescompetenc() < $dtCompetencia && $evento->getNmevento() != 'S-2230') {
                                    // Se status pendente, com erro de envio ou com erro de processamento inativa o envio para não perder o registro de tentativa de envio protocolo/recibo
                                    if ($envio->getIdstatus() == 'P' || $envio->getIdstatus() == 'E' || $envio->getIdstatus() == 'R') {
                                        $envio->setIdativo('N');
                                        $envio->setDtultatu(DateUtil::getDataAtual());
                                        $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                                        $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                                        $this->entityManager->persist($envio);
                                        $this->entityManager->flush();
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function beforeDeleteEnvioCadastros($row) {
        try {
            if ($row['NMEVENTO'] == 'S-3000' || $row['IDOPERACAO'] == 'E' || $row['IDOPERACAO'] == 'R') {
                $enviosOrig = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findBy(array('nrorg' => $row['NRORG'], 'nrenvioexcl' => $row['NRENVIO']));

                foreach ($enviosOrig as $envioOrig) {
                    $envioOrig->setNrenvioexcl();
                    $this->entityManager->persist($envioOrig);
                    $this->entityManager->flush();
                }
                
                /*Verifica se tem histórico de envio*/
                /*$historicos = $this->entityManager->getRepository(Repositories::ESO_HISTORICOENVIO)->findBy(array('nrorg' => $row['NRORG'], 'nrenvio' => $row['NRENVIO']));
                foreach ($historicos as $historico) {
                    $this->entityManager->remove($historico);
                    $this->entityManager->flush();
                }*/
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function deleteDadosEnvio($row){
        if($row['NREVENTO'] = 'S-1200' || $row['NREVENTO'] = 'S-1210'){
            $dadosEnvio = $this->entityManager->getRepository(Repositories::ESO_DADOSENVIO)->findBy(array('nrorg' => $row['NRORG'], 'nrenvio' => $row['NRENVIO']));
            if($dadosEnvio){
                try{
                    $this->entityManager->beginTransaction();
                    foreach ($dadosEnvio as $itemEnvio) {
                        $this->entityManager->remove($itemEnvio);
                        $this->entityManager->flush();
                    }
                    $this->entityManager->commit();
                }catch(\Exception $e){
                    $this->entityManager->rollback();
                    throw $e;
                }
            }
        }
    }
    
     public function deleteEventos($row){
        if($row['NMEVENTO'] = 'S-2300' || $row['NMEVENTO'] = 'S-2200'){
            $dadosEnvio = $this->entityManager->getRepository(Repositories::ESO_DADOSENVIO)->findBy(array('nrorg' => $row['NRORG'], 'nrenvio' => $row['NRENVIO']));
            if($dadosEnvio){
                try{
                    $this->entityManager->beginTransaction();
                    foreach ($dadosEnvio as $itemEnvio) {
                        $this->entityManager->remove($itemEnvio);
                        $this->entityManager->flush();
                    }
                    $this->entityManager->commit();
                }catch(\Exception $e){
                    $this->entityManager->rollback();
                    throw $e;
                }
            }
        }
    }
    
    public function setEventoExclusaoEventoOrigem($nrorg, $nrenvioexcl, $nrenvio, $idgrupo, $nrevento) {
        try {
            $envio = $this->getEnvioCadastro($nrorg, $nrenvio);
            
            if ($idgrupo == 'T') {
                $envios = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findBy(
                    array('nrorg' => $nrorg, 'nrtpambiente' => $envio->getNrtpambiente(), 'idativo' => 'S', 'idstatus' => ['C','A'], 'nrestruturam' => $envio->getNrestruturam(), 'nrtabela' => $envio->getNrtabela(), 'nrchave' => $envio->getNrchave(), 'nrevento' => $nrevento)
                );
                foreach ($envios as $envioCad) {
                    $envioCad->setNrenvioexcl($nrenvioexcl);
                    $this->entityManager->persist($envioCad);
                }
            }
            
            $envio->setNrenvioexcl($nrenvioexcl);
            $this->entityManager->persist($envio);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * Realiza validações antes de enviar um novo evento
     */
    public function beforeEnvioEvento($nrorg, $nrestruturam, $nrtpambiente, $paramESocial, $nrevento, $nrchave, $nrtabela, $idoperacao, $dtmescompetenc, $jenkins = false) {
        try {
            $dtCompetencia = DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
            $evento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nrorg' => $this->hcmParameters->getNrorgPadrao(), 'nrevento' => $nrevento));
            
            if ($evento->getNmevento() == 'S-1200') {
                //$vinculos = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findBy(array('nrorg' => $nrorg, 'nrpessoa' => $nrchave, 'nrtipovinculom' => [1,2,3,10]));
                $vinculos = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculosRescisaoComplementar($nrorg, $nrchave, $dtCompetencia, 1);
                foreach ($vinculos as $vinculo) {
                    if ($vinculo->getDtrescisaovinc()) {
                        /*Verifica se a data de rescisão do vínculo é anterior à competência*/
                        if (DateUtil::comparaDatas($vinculo->getDtrescisaovinc(), $dtCompetencia)) {
                            
                            //Busca o evento de admissão para a versão do parâmetro
                            $eventoAdmissao = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(
                                array('nrorg' => $this->hcmParameters->getNrorgPadrao(), 'dsversao' => $paramESocial['DSVERSAO'], 'nmevento' => ($vinculo->getNrtipovinculom() == 1 ? 'S-2200' : 'S-2300'))
                            );
                            /*Busca histórico do vínculo com maior competência já que a rescisão é anterior à atual*/
                            $vinculoh = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->findOneBy(
                                array('nrorg' => $nrorg, 'nrvinculom' => $vinculo->getNrvinculom()),
                                array('dtmescompetenc' => 'DESC')
                            );
                            
                            /*Busca o histórico da  estrututa do vínculo*/
                            $estruturah = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaHistorico(
                                $nrorg, $vinculoh->getNrestrutlegal(), $dtCompetencia
                            );
                            /*Verifica se existe evento de admissao para a estrutura legal conforme o tipo de vínculo*/
                            $envioAdmissao = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosPorNomeEventoChaveCnpj(
                                $nrorg, $eventoAdmissao->getNmevento(), $vinculo->getNrvinculom(), $nrtpambiente, $estruturah->getCdcnpjestrut(), $dtCompetencia->format('d/m/Y'), 'S' 
                            );
                            
                            //Caso não exista envio de admissão é criado um novo
                            if (!$envioAdmissao) {
                                //Insere o evento de admissão para o vínculo
                                $envioAdmissao = $this->novoEsoEnvioCadastro(
                                    null, $eventoAdmissao->getNrevento(), $vinculoh->getNrestrutlegal(), $vinculo->getNrvinculom(), null, $vinculoh->getDtmescompetenc(), 'I', $nrtpambiente, null, $vinculo->getNrvinculom()
                                );
                                $envioAdmissao = $envioAdmissao->toArray();
                            }
                            
                            if (!empty($envioAdmissao)) {
                                $msgEnvioAdmissao = '';
                                
                                if ($envioAdmissao['IDSTATUS'] != 'C' && $envioAdmissao['IDSTATUS'] != 'A') {
                                    
                                    //Salva a inscrição do empregador original e utiliza do evento de admissão
                                    $paramAdmissao = $this->getParamESocial($vinculoh->getNrestrutlegal());
                                    $nrInscEmpregadorOrig = $this->_nrInscEmpregador;
                                    $this->_nrInscEmpregador = $paramAdmissao['CDCNPJESTRUT'];
                                    
                                    $msgEnvioAdmissao = 'Evento de admissão vínculo ' . $envioAdmissao['NRCHAVE'] . ' criado.';
                                    
                                    //Se o envio de admissão estiver pendente ou com erro é realizado nova tentantiva de transmissão
                                    if ($envioAdmissao['IDSTATUS'] == 'P' || $envioAdmissao['IDSTATUS'] == 'E' || $envioAdmissao['IDSTATUS'] == 'R') {
                                        $row = $envioAdmissao;
                                        $row['NMEVENTO'] = $eventoAdmissao->getNmevento();
                                        $row['DSVERSAO'] = $eventoAdmissao->getDsversao();
                                        $row['IDGRUPO'] = $eventoAdmissao->getIdgrupo();
                                        
                                        $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $row['NRENVIO']);
                                    
                                        //Transmite e consulta o evento de admissão
                                        $xmlEvento = $this->getXmlEvento(str_replace('.','-',$paramESocial['DSVERSAO']), $row);
                                        if ($xmlEvento) {
                                            $eventosLote[] = $xmlEvento;
                                        }
                                        
                                        if (empty($msgProcesso->getMessages()) && empty($eventosLote)) {
                                            MsgProcesso::getInstance()->addError('Erro ao gerar evento de admissão para o vínculo ' . $row['NRCHAVE']);
                                        } else if (empty($msgProcesso->getMessages())) {
                                            $xmlLoteEventos = $this->getXmlLote($row['IDGRUPO'], $paramESocial, str_replace('.','-',$paramESocial['DSVERSAO']), $eventosLote);
            
                                            $retorno = $this->enviarLote($eventosLote, $xmlLoteEventos);
                                            $msgEnvioAdmissao .= ' [Envio: ' . $retorno['COD'] . '-' . $retorno['MSG'] . ']';
                                            
                                            if (!is_null($retorno['REC'])) {
                                                $envioAdmissao['IDSTATUS'] = self::IDSTATUS_PROCESSAMENTO;
                                                $envioAdmissao['NRRECIBO'] = $retorno['REC'];
                                            }
                                        }
                                    }
                                    
                                    if ($envioAdmissao['IDSTATUS'] == self::IDSTATUS_PROCESSAMENTO) {
                                        $xmlConsulta = $this->getXmlConsulta($envioAdmissao['NRRECIBO'], str_replace('.','-',$paramESocial['DSVERSAO']));
                                        $retornoConsulta = $this->consultarEvento($envioAdmissao['NRRECIBO'], $xmlConsulta);
                                        $codRetornoConsulta = $retornoConsulta['COD'];
                                        $msgRetornoConsulta = $retornoConsulta['MSG'];
                                        $tmpEsperaConsulta = $retornoConsulta['TMP'];
                                        $msgEnvioAdmissao .= ' [Consulta: ' . $retornoConsulta['COD'] . '-' . $retornoConsulta['MSG'] . ']';
                                    }
                                    
                                    MsgProcesso::getInstance()->addMessage($msgEnvioAdmissao);
                                    
                                    //Retorna a inscrição do empregador original
                                    $this->_nrInscEmpregador = $nrInscEmpregadorOrig;
                                }
                            }
                                
                        }
                    }
                }
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna itens que ainda não foram enviados ao eSocial de acordo com o evento informado
     * @param $nmevento
     * @param $params array
     * @return array
     * @throws \Exception
     */
    public function getItensParaEnvioEvento($nmevento, $idgrupo, $params = array(), $jenkins = false) {
        try {
            $repository = null;
            $method = 'getItensParaEnvio' . str_replace('-','',$nmevento);
            
            switch ($idgrupo) {
                case 'I':
                case 'T':
                    $repository = $this->eSocialItensCadTabRepository;
                    break;
                case 'N':
                    $repository = $this->eSocialItensNaoPerRepository;
                    break;
                case 'P':
                    $repository = $this->eSocialItensPeriodRepository;
                    break;
                default:
                    $repository = $this->eSocialRepository;
                    break;
            }
            if (method_exists($repository, $method)) {
                return $repository->$method($params);
            } else {
                if(!$jenkins){
                    throw new \Exception(Messages::HCM09000_ESO_CALLED_EVENTO_ERROR,Messages::ERROR_MESSAGE_CODE);
                }
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrestruturam
     * @return array
     * @throws \Exception
     */
    public function getParamESocial($nrestruturam) {
        try {
            //@todo validar tipo de inscrição CNPJ/CPF e inscrição com caracteres inválidos
            $parametro = $this->eSocialRepository->getParamESocial($nrestruturam);
            if ($parametro['TPINSCRICAOEMP'] != '1' && $parametro['TPINSCRICAOEMP'] != '2') {
                throw new \Exception(Messages::HCM09000_ESO_TPINSCEMPREG_ERROR,Messages::ERROR_MESSAGE_CODE);
            }
            if ($parametro['TPINSCRICAOPARC'] != '1' && $parametro['TPINSCRICAOPARC'] != '2') {
                throw new \Exception(Messages::HCM09000_ESO_TPINSCTRANSM_ERROR,Messages::ERROR_MESSAGE_CODE);
            }
            return $parametro;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function atualizaDataCargaInicial($nrparametro) {
        try {
            $parametro = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->findOneBy(array(
                'nrparametro' => $nrparametro, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()
            ));
            $parametro->setDttransmissao(DateUtil::getDataAtual());
            $parametro->setDtultatu(DateUtil::getDataAtual());
            $parametro->setCdoperultatu($this->hcmParameters->getCdoperador());
            $parametro->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Método temporário para salvar arquivos no servidor AWS para evitar tráfego com o FileServer.
     * Retorna o xml de envio, retorno de envio e retorno de consulta dos eventos
     * @param $nrorg
     * @param $xmlFile
     * @param $idEvento
     * @return array
     */
    public function getXmlOnFilesPath($nrorg, $xmlFile, $idEvento, $dsDirXml) {
        try {
            
            // SOLUÇÃO NOVO FILESERVER	 
            // $fileServePath = $this->hcmParameters->getFileServerDirectory() . $dsDirXml;	 
            // $file = Util::buscaArquivoFileServer($fileServePath, 'xml');	 
            // // $file é o conteúdo do arquivo buscado
            
            $file = $this->hcmParameters->getFileServerDirectory() . $dsDirXml;
            error_reporting(E_ALL ^ E_WARNING);
            if (fopen($file, 'r')) { //Busca o arquivo no FileServer
                $xml = file_get_contents($file);
            } else {
                $nmFile = explode('/',$dsDirXml);
                $nmFile = end($nmFile);
                $file = $this->filesDir . $nmFile . ".xml";
                if (fopen($file, 'r')) { //Busca o arquivo na raiz da pasta files (Ex: ../files/arquivo.xml)
                    $xml = file_get_contents($file);
                } else {
                    $file = $this->filesDir . $dsDirXml . ".xml";
                    if (fopen($file, 'r')) { //Busca o arquivo na estrutura de pastas na pasta files (Ex: ../files/ProducaoRestrita/Cadastros/15547701000112/Envio/arquivo.xml)
                        $xml = file_get_contents($file);
                    } else {
                        $file = $this->filesDir . $this->hcmParameters->getNrorgTrabalho() . "/" . $this->filesDirESocial . $dsDirXml . ".xml";
                        if (fopen($file, 'r')) { //Busca o arquivo na estrutura de pastas da organização na pasta files  (Ex: ../files/20/eSocial/ProducaoRestrita/Cadastros/15547701000112/Envio/arquivo.xml)
                            $xml = file_get_contents($file);
                        } else {
                            throw new \Exception(Messages::FILE_NOT_FOUND);
                        }
                    }
                }
            }
            error_reporting(E_ALL ^ E_NOTICE);
            return $xml;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o xml de envio, retorno de envio e retorno de consulta dos eventos
     * @param $nrorg
     * @param $xmlFile
     * @param $idEvento
     * @return array
     */
    public function getXmlFiles($nrorg, $xmlFile, $idEvento, $dsDirXml, $dsDirXmlFilesPath=null) {
        try {
            
            // SOLUÇÃO NOVO FILESERVER	 
            // Subsitituir a função por:	 
            // $fileServePath = $this->hcmParameters->getFileServerDirectory() . '/files/'. $dsDirXml;	 
            // $file = Util::buscaArquivoFileServer($fileServePath . '.xml', 'xml', false);	 
            // // $file é o conteúdo do arquivo buscado	 
            // return $file;

            $file = $this->hcmParameters->getFileServerDirectory() . $dsDirXml;
            error_reporting(E_ALL ^ E_WARNING);
            if (fopen($file, 'r')) {
                $xml = file_get_contents($file);
            } else {
                if (!is_null($dsDirXmlFilesPath)) {
                    $xml = $this->getXmlOnFilesPath($nrorg, $xmlFile, $idEvento, $dsDirXmlFilesPath);
                } else {
                    throw new \Exception(Messages::FILE_NOT_FOUND);
                }
            }
            error_reporting(E_ALL ^ E_NOTICE);
            return $xml;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function setXmlFileToDataSet($xmlString) {
        try {
            $json = $this->parseXmlToJson($xmlString);
            $dataSet = $this->translateESocialJson($json);
            return $dataSet;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function parseXmlToJson($xmlFileContents) {
        try {
    		$xmlFileContents = str_replace(array("\n", "\r", "\t"), '', $xmlFileContents);
    		$xmlFileContents = trim(str_replace('"', "'", $xmlFileContents));
    		$simpleXml = simplexml_load_string($xmlFileContents);
    		$json = json_encode($simpleXml);
    		return $json;
        } catch(\Exception $e) {
            throw $e;
        }
	}
	
	public function translateESocialJson($json) {
        try {
            $rows = array();
    		$this->translateESocialTag(json_decode($json,true), null, $rows);
    		return $rows;
        } catch(\Exception $e) {
            throw $e;
        }
	}
	
	public function translateESocialTag($tag, $parent, &$rows) {
        try {
            $this->_tagNivel ++;
	        $id = count($rows) + 1;
    		foreach ($tag as $name => $value) {
    		    if ($name === 'Signature') {
    		        // Não traduz a assinatura do evento
    		        continue;
    		    }
    		    elseif (is_array($value)) {
    		        $rows[] = array(
    		            'ID' => $id,
    		            'PARENT' => $parent,
    		            'NAME' => (is_int($name) ? $name + 1 : (isset($this->_tagsTrad[$name]) ? $this->_tagsTrad[$name] : $name)),
    		            'VALUE' => null,
    		            '_opened' => true,
    		            'NIVEL' => $this->_tagNivel
		            );
    		        $this->translateESocialTag($value, $id, $rows);
    		        $id = count($rows) + 1;
    		    } else {
    		        $rows[] = array(
    		            'ID' => $id,
    		            'PARENT' => $parent,
    		            'NAME' => (isset($this->_tagsTrad[$name]) ? $this->_tagsTrad[$name] : $name),
    		            'VALUE' => array_key_exists($name, $this->_tagsOpc) ? $value.' - '.$this->_tagsOpc[$name][$value] : $value,
    		            '_opened' => true,
    		            'NIVEL' => $this->_tagNivel
		            );
		            $id = $id + 1;
    		    }
    		}
        } catch(\Exception $e) {
            throw $e;
        }
	}

    /**
     * @param $nrestruturam
     * @return array
     * @throws \Exception
     */
    public function setDadosTransmissao($nrestruturam, $jenkins = false) {
        try {
            $parametro = $this->getParamESocial($nrestruturam);
            $this->_tpAmb = $parametro['NRTPAMBIENTE'];
            $this->_tpInscTransmissor = $parametro['TPINSCRICAOPARC'];
            $this->_nrInscTransmissor = $parametro['NRINSCRICAOPARC'];
            $this->_nrInscEmpregador = $parametro['CDCNPJESTRUT'];
            if ($parametro) {
                if (empty($parametro['DSARQUCERT'])) {
                    throw new \Exception(Messages::HCM09000_ESO_CERTIFICADO_NOT_FOUND_ERROR, Messages::ERROR_MESSAGE_CODE);
                }
                if (empty($parametro['DSSENHACERT'])) {
                    throw new \Exception(Messages::HCM09000_ESO_SENHA_CERTIFICADO_NOT_FOUND_ERROR, Messages::ERROR_MESSAGE_CODE);
                }

                $this->_certFile = $this->_certificatesPath . $parametro['DSARQUCERT'];
                $this->_certKeyFile = $this->_tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey" . $this->_nrInscTransmissor . ".pem";
                $this->_certPass = $parametro['DSSENHACERT'];

                error_reporting(E_ALL ^ E_WARNING);
                
                // SOLUÇÃO NOVO FILESERVER	 
                // $this->_certFile = Util::buscaArquivoFileServer($this->_certFile, '.pfx');	 
                // // $this->_certFile é o conteúdo do arquivo buscado	 
                // // comentar linha "$this->_certFile = file_get_contents($this->_certFile);", pois já retornado o conteúdo do arquivo
                
                // Variável criada para realizar 5 tentativas de acessar o cetificado no FileServer antes de retornar o erro
                $openCertFile = false;
                for ($i=0;$i<5;$i++) {
                    if (fopen($this->_certFile,"r") == true) {
                        $openCertFile = true;
                        break;
                    }
                }
                if ($openCertFile) {
                    if (file_exists($this->_certKeyFile)) {
                        unlink($this->_certKeyFile);
                    }
                    if (!file_exists($this->_certKeyFile)) {
                        if (!is_dir($this->_tempCertificatesPath)) {
                            mkdir($this->_tempCertificatesPath, 0777, true);
                        }
                        // Alteração temporária para ler somente uma vez o arquivo do FileServer e armazenar na variável certFile
                        //$data = file_get_contents($this->_certFile);
                        //openssl_pkcs12_read($data, $cert, $this->_certPass);
                        $this->_certFile = file_get_contents($this->_certFile);
                        openssl_pkcs12_read($this->_certFile, $cert, $this->_certPass);
                        
                        if (is_null($cert)) {
                            throw new \Exception(Messages::HCM09000_ESO_GENERATE_CERT_KEY_ERROR, Messages::ERROR_MESSAGE_CODE);
                        } else {
                            //Dados do certificado
                            $dados = array ();
                            $dados = openssl_x509_parse( openssl_x509_read($cert['cert']) );
                            
                            //Dados mais importantes
                            //$dados['subject']['C']; //País
                            //$dados['subject']['ST']; //Estado
                            //$dados['subject']['L']; //Município
                            //$dados['subject']['CN']; //Razão Social e CNPJ / CPF
                            $validadeCert = date('d/m/Y', $dados['validTo_time_t']);//Validade	
                            $validadeCert = DateUtil::getDataDeString($validadeCert, DateUtil::FORMATO_BRASILEIRO, true);
                            //$dados['extensions']['subjectAltName'];	//Emails Cadastrados separado por ,
                            
                            $dataAtual = date('d/m/Y');
                            $dataAtual = DateUtil::getDataDeString($dataAtual, DateUtil::FORMATO_BRASILEIRO, true);
                            // Verifica se a validade é anterior à data atual
                            if (DateUtil::comparaDatas($validadeCert, $dataAtual)) {
                                throw new \Exception(Messages::HCM09000_ESO_CERTIFICADO_VALIDADE_ERROR, Messages::ERROR_MESSAGE_CODE);
                            } else {
                                file_put_contents($this->_certKeyFile, $cert["pkey"] . "\r\n" . $cert["cert"]);
                                chmod($this->_certKeyFile, 0777);
                            }
                            
                            // Aviso de proximidade do vencimento do certificado digital
                            if($jenkins){
                                $intervaloAviso = date_interval_format(date_diff($validadeCert, $dataAtual), '%a');
                                if($intervaloAviso <= 3 && $intervaloAviso >= 0){
                                    throw new \Exception('Certificado digital a expirar em '.$validadeCert->format('d/m/Y').'. Faltam '.$intervaloAviso.' dias.', Messages::ALERT_MESSAGE_CODE);
                                }
                            }
                        }
                    }
                } else {
                    if($jenkins){
                        echo 'erro open certificado ' . $this->_certFile . PHP_EOL;
                    }
                    throw new \Exception(Messages::HCM09000_ESO_CERTIFICADO_NOT_OPEN_ERROR, Messages::ERROR_MESSAGE_CODE);
                    
                }
                error_reporting(E_ALL ^ E_NOTICE);
            } else {
                throw new \Exception(Messages::HCM09000_ESO_PARAMETRO_NOT_FOUND_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch(\Exception $e) {
            if($jenkins){
                if($e->getCode() == Messages::ALERT_MESSAGE_CODE){
                    return array('ALERT' => $e->getMessage());
                }else{
                    return array('MSG' => $e->getMessage());
                }
            }
            throw $e;
        }
    }

    /**
     * @param $nrenvio
     * @param $nrevento
     * @param $nrestruturam
     * @param $nrchave
     * @param $nrtabela
     * @param $dtmescompetenc
     * @param $idoperacao
     * @param $nrtpambiente
     * @return \Teknisa\Model\Gestaopessoas\Entities\EsoEnviocadastros
     * @throws \Exception
     */
    public function novoEsoEnvioCadastro($nrenvio, $nrevento, $nrestruturam, $nrchave, $nrtabela, $dtmescompetenc, $idoperacao, $nrtpambiente, $nrenvioexcl=null, $nrvinculom = NULL, $semMovimento = null) {
        try {
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEnviocadastros $envio */
            $esoEnviocadastros = Repositories::ESO_ENVIOCADASTROS;
            
            if ($nrenvio == null) {
                $nrenvio = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_ENVIOCADASTROS', 1, 12, true);
            }
            
            $envio = new $esoEnviocadastros();
            $envio->setNrenvio($nrenvio);
            $envio->setNrevento($nrevento);
            $envio->setNrestruturam($nrestruturam);
            $envio->setNrvinculom($nrvinculom);
            $envio->setNrchave($nrchave);
            $envio->setNrtabela($nrtabela);
            $envio->setDtmescompetenc(DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true));
            $envio->setIdoperacao($idoperacao);
            $envio->setIdfechasmoviment(!empty($semMovimento) ? $semMovimento : 'N');
            $envio->setIdstatus('P');
            $envio->setNrtpambiente($nrtpambiente);
            $envio->setNrenvioexcl($nrenvioexcl);
            $envio->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $envio->setDtinclusao(DateUtil::getDataAtual());
            $envio->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $envio->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($envio);
            
            return $envio;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getEnviosParaEvento($nrEstruturam, $nrtpambiente, $nrEvento, $idStatus, $dtmescompetenc, $idGrupo, $nmEvento = null, $nrtabela = null) {
    try {
            if ($idGrupo == 'P') {
                $dtmescompetenc = DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true);
                if($nmEvento == 'S-1200' && !empty($nrtabela)){
                    return $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findBy(
                        array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrestruturam' => $nrEstruturam, 'nrtpambiente' => $nrtpambiente, 'nrevento' => $nrEvento, 'dtmescompetenc' => $dtmescompetenc, 'idstatus' => $idStatus, 'idativo' => 'S', 'nrtabela' => $nrtabela),
                        array('nrchave' => 'ASC', 'nrtabela' => 'ASC', 'dtmescompetenc' => 'ASC')
                    );
                }else{
                    return $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findBy(
                        array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrestruturam' => $nrEstruturam, 'nrtpambiente' => $nrtpambiente, 'nrevento' => $nrEvento, 'dtmescompetenc' => $dtmescompetenc, 'idstatus' => $idStatus, 'idativo' => 'S'),
                        array('nrchave' => 'ASC', 'nrtabela' => 'ASC', 'dtmescompetenc' => 'ASC')
                    );
                    
                }
            } else {
                return $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findBy(
                    array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrestruturam' => $nrEstruturam, 'nrtpambiente' => $nrtpambiente, 'nrevento' => $nrEvento, 'idstatus' => $idStatus, 'idativo' => 'S'),
                    array('nrvinculom' => 'ASC', 'dtmescompetenc' => 'ASC', 'nrchave' => 'ASC', 'nrtabela' => 'ASC')
                );
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $dsVersao
     * @param $dadosEnvio
     * @return array
     * @throws \Exception
     */
    public function getXmlEvento($dsVersao, $dadosEnvio) {
        try {
            $xmlEvento = false;
            /* Validações para evento de fechamento */
            if ($dadosEnvio['NMEVENTO'] == 'S-1299') {
                if (!$this->beforeEnvioFechamento($dadosEnvio['NRORG'], $dadosEnvio['NRESTRUTURAM'], DateUtil::getDataDeString($dadosEnvio['DTMESCOMPETENC'],DateUtil::FORMATO_BRASILEIRO,true), $dadosEnvio['NRTABELA'], $dadosEnvio['IDFECHASMOVIMENT'])) {
                    return false;
                }
            }
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEnviocadastros $envio */
            $envio = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->find(array(
                'nrenvio' => $dadosEnvio['NRENVIO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho()
            ));
            // Valida se o xml ja foi gerado no status PENDENTE
            if($envio->getDsdirxmlenv() && $envio->getIdstatus() === 'P'){
                $xmlData['IDEVENTO'] = $envio->getIdevento();
                $dsDirXml = 
                ($envio->getNrtpambiente() == 1 ? 'Producao' : 'ProducaoRestrita') . "/" .
                ($dadosEnvio['IDGRUPO'] == 'P' ? 'Periodicos' : ($dadosEnvio['IDGRUPO'] == 'N' ? 'NaoPeriodicos' : 'Cadastros')) . "/" .
                $this->_nrInscEmpregador . "/Envio/" .
                $envio->getDsdirxmlenv();
                $nmXmlEvento = $envio->getDsdirxmlenv();
                $xmlEvento = $this->getXmlFiles($dadosEnvio['NRORG'], 'envio', $envio->getIdevento(), $envio->getDsdirxmlenv(), $dsDirXml);
                // $xmlEvento = $this->getXmlOnFilesPath($this->hcmParameters->getNrorgTrabalho(), '', $xmlData['IDEVENTO'], $dsDirXml);
            }else{
                // Valida se o status do evento no banco de dados já está em processamento ou concluído caso a tela não esteja atualizada e o evento seja enviado novamente
                $idStatus = true;
                if (is_object($envio)) {
                    if (($envio->getIdstatus() == 'M' || $envio->getIdstatus() == 'C' || $envio->getIdstatus() == 'A' || !is_null($envio->getNrenvioexcl())) &&
                        ($envio->getCderro() != '301' && $envio->getCderro() != '69')) {
                        $idStatus = false;
                    }
                }
                if ($idStatus) {
                    /*LOG -*/
                    $this->saveLogESocial(' > getXmlEvento Evento:' . $dadosEnvio['NMEVENTO'] . ' Envio:' . $dadosEnvio['NRENVIO']);
                    $lap = microtime(true);
                    /*- LOG*/
                    
                    /* Monta os dados do evento para gerar o XML */
                    $xmlData = $this->eSocialFactory->buildDadosXml($dsVersao, $dadosEnvio, $this->_tpAmb, $this->_procEmi, $this->_verProc, $this->_tagsTrad);
                    /*LOG -*/
                    $this->saveLogESocial(' > > buildDadosXml elapsed ' . $this->convertSecondsToTime(microtime(true) - $lap));
                    /*- LOG*/
                } else {
                    MsgProcesso::getInstance()->addError('Envio ' . $dadosEnvio['NRENVIO'] . ' já havia sido transmitido. Favor atualizar os dados da tela.');
                }
                
                // Caso tenha erros de validação não gera o xml para envio
                if (count(MsgProcesso::getInstance()->getMessages()) > 0) {
                    return false;
                }
    
                if ($xmlData) {
                    /*LOG -*/
                    $lap = microtime(true);
                    /*- LOG*/
                    /* Utiliza os dados formatados para montar o xml conforme o template twig */
                    //$nmXmlEvento = 'envio_'.$xmlData['IDEVENTO'];
                    $nmXmlEvento = $xmlData['IDEVENTO'] . DateUtil::getDataAtual()->format('YmdHis');
                    $xmlEvento = $this->buildXml($dadosEnvio['NMEVENTO'], $dsVersao, $xmlData['XMLDATA'], $nmXmlEvento);
                    /*LOG -*/
                    $this->saveLogESocial(' > > buildXml elapsed ' . $this->convertSecondsToTime(microtime(true) - $lap));
                    /*- LOG*/
    
                    /*LOG -*/
                    $lap = microtime(true);
                    /*- LOG*/
                    /* Assina o xml do evento */
                    $xmlEvento = $this->assinaXmlEvento($xmlEvento);
                    /*LOG -*/
                    $this->saveLogESocial(' > > assinaXmlEvento elapsed ' . $this->convertSecondsToTime(microtime(true) - $lap));
                    /*- LOG*/
    
                    // Alteração temporária para salvar os arquivos no servidor AWS
                    $filePath = ($this->_tpAmb == 1 ? 'Producao' : 'ProducaoRestrita') . "/" .
                                ($dadosEnvio['IDGRUPO'] == 'P' ? 'Periodicos' : ($dadosEnvio['IDGRUPO'] == 'N' ? 'NaoPeriodicos' : 'Cadastros')) . "/" .
                                $this->_nrInscEmpregador . "/" .
                                'Envio';
                    
                    /*if (!is_null($dadosEnvio['DSDIRXMLENV'])) {
                        $this->zhFileUploader->deleteFile($dadosEnvio['DSDIRXMLENV']);
                    }*/
                    /*LOG -*/
                    $lap = microtime(true);
                    /*- LOG*/
                    $xmlPath = $this->saveXmlOnFileServer(
                        $dadosEnvio['NRESTRUTURAM'], null, $xmlEvento, $nmXmlEvento, $xmlData['IDEVENTO'], $filePath
                    );
                    /*LOG -*/
                    $this->saveLogESocial(' > > saveXmlOnFileServer elapsed ' . $this->convertSecondsToTime(microtime(true) - $lap));
                    /*- LOG*/
                    
                    /*LOG -*/
                    //$lap = microtime(true);
                    /*- LOG*/
                    /*$xmlPath = $this->saveXmlOnFilesPath(
                        $dadosEnvio['NRESTRUTURAM'], null, $xmlEvento, $nmXmlEvento, $xmlData['IDEVENTO'], $filePath
                    );*/
                    /*LOG -*/
                    //$this->saveLogESocial(' > > saveXmlOnFilesPath elapsed ' . $this->convertSecondsToTime(microtime(true) - $lap));
                    /*- LOG*/
    
                    $envio->setIdevento($xmlData['IDEVENTO']);
                    $envio->setDtenvio(DateUtil::getDataAtual());
                    $envio->setDsdirxmlenv($xmlPath);
                    $envio->setDtultatu(DateUtil::getDataAtual());
                    $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                    $this->entityManager->persist($envio);
                }
            }
            if($xmlEvento){
                return array(
                    "NRENVIO" => $dadosEnvio['NRENVIO'],
                    "IDEVENTO" => $xmlData['IDEVENTO'],
                    "NMXMLEVENTO" => $nmXmlEvento,
                    "XMLEVENTO" => $xmlEvento,
                );
            }else {
                return false;
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param $xmlEnvio
     * @throws \Exception
     */
     public function setDadosEnvio($arrayEnvio, $nrenvio){
        try{
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            
            foreach ($arrayEnvio as $itemEnvio) {
                
                $this->entityManager->beginTransaction();
                
                $novoItem = Repositories::ESO_DADOSENVIO;
                $novoItem = new $novoItem;
                $novoItem->setNrdadosenvio(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'ESO_DADOSENVIO', 1, 22, true));
                $novoItem->setNrorg($nrorg);
                $novoItem->setNrenvio($nrenvio);
                $novoItem->setNmdadosenvio($itemEnvio['NAME']);
                $novoItem->setDsdadosenvio($itemEnvio['VALUE']);
                $novoItem->setIddadosenvio($itemEnvio['ID']);
                $novoItem->setIddadosenviopai($itemEnvio['PARENT']);
                $novoItem->setIdativo('S');
                $novoItem->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $novoItem->setDtinclusao(DateUtil::getDataDeString($this->hcmParameters->getDataAtual()));
                $novoItem->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $novoItem->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                $novoItem->setDtultatu(DateUtil::getDataDeString($this->hcmParameters->getDataAtual()));
                $novoItem->setCdoperultatu($this->hcmParameters->getCdoperador());
                
                $this->entityManager->persist($novoItem);
                $this->entityManager->flush();
                $this->entityManager->commit();
                
            }
            
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
     }

    /**
     * @param $dsVersao
     * @param $dadosEnvio
     * @return array
     * @throws \Exception
     */
    public function getXmlEventoExclusaoEmpregador($dsVersao, $dadosEnvio) {
        try {
            /* Monta os dados do evento para gerar o XML */
            $xmlData = $this->eSocialFactory->buildDadosXml($dsVersao, $dadosEnvio, $this->_tpAmb, $this->_procEmi, $this->_verProc, $this->_tagsTrad);

            if ($xmlData) {
                /* Seta os dados específicos para identificar a operação de exclusão dos dados do empregador */
                $xmlData['XMLDATA']['evtInfoEmpregador']['infoEmpregador']['inclusao']['infoCadastro']['NMRAZAO'] = 'RemoverEmpregadorDaBaseDeDadosDaProducaoRestrita';
                $xmlData['XMLDATA']['evtInfoEmpregador']['infoEmpregador']['inclusao']['infoCadastro']['CLASSTRIB'] = '00';
            
                /* Utiliza os dados formatados para montar o xml conforme o template twig */
                $nmXmlEvento = 'envio_'.$xmlData['IDEVENTO'];
                $xmlEvento = $this->buildXml($dadosEnvio['NMEVENTO'], $dsVersao, $xmlData['XMLDATA'], $nmXmlEvento);

                /* Assina o xml do evento */
                $xmlEvento = $this->assinaXmlEvento($xmlEvento);

                return array(
                    "NRENVIO" => $dadosEnvio['NRENVIO'],
                    "IDEVENTO" => $xmlData['IDEVENTO'],
                    "NMXMLEVENTO" => $nmXmlEvento,
                    "XMLEVENTO" => $xmlEvento,
                );
            } else {
                return false;
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function excluirEventos($nrEstruturam, $cnpjEstrutura, $dsVersao, $nrtpambiente) {
        try {
            // Busca estrutura Matriz e Filiais e exclui os eventos de producao restrita
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $nrtipoestrutlegal = $this->hcmParameters->getTipoEstruturaLegalOrg();
            $estruturas = $this->eSocialRepository->getEstruturasESocial($nrorg, $dtmescompetenc, $nrtipoestrutlegal, '1', '');
            foreach ($estruturas as $estrutura) {
                $cnpjChars = array('.','/','-');
                if (substr(str_replace($cnpjChars, '', $cnpjEstrutura),0,8) == substr(str_replace($cnpjChars, '', $estrutura['CDCNPJEMPREGADOR']),0,8)) {
                    $return = $this->eSocialRepository->excluirEventosProducaoRestrita($nrorg, $estrutura['NRESTRUTURAM']);
                }
            }
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function beforeEnvioFechamento($nrorg, $nrestruturam, $dtmescompetenc, $nrtabela, $semMovimento = null) {
        try {
            $idFechamento = true;
            
            if ($this->validaFechamentoEstrutura($nrorg, $nrestruturam, $dtmescompetenc, $nrtabela) && !$this->validaReaberturaEstrutura($nrorg, $nrestruturam, $dtmescompetenc, $nrtabela)) {
                MsgProcesso::getInstance()->addError(Messages::evaluateMessage(Messages::HCM09000_ESO_ESTRUTURA_FECHAMENTO_EXISTS_ERROR, $nrestruturam));
                $idFechamento = false;
            }
            
            // Verifica se todos vinculos ativos do empregador possuem liquido definitivo (3035)
            if($nrtabela == 1 && ($semMovimento == null || $semMovimento != 'S')){
                $vinculos = $this->eSocialRepository->getVinculosAtivosSemLiquidoFechamento($nrorg, $this->hcmParameters->getDtcompetencia(), 1, $nrtabela, $this->hcmParameters->getDtcompetencia(), 3030, $nrestruturam);
                foreach ($vinculos as $vinculo) {
                    MsgProcesso::getInstance()->addError('O vínculo ' . $vinculo['NRVINCULOM'] . '-' . $vinculo['NMVINCULOM'] . ' possui líquido mas não possui evento de fechamento (Evento 3035)');
                    $idFechamento = false;
                }
            }
            
            return $idFechamento;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function validaFechamentoEstrutura($nrorg, $nrestruturam, $dtmescompetenc, $nrtipomoviment) {
        try {
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment);
            
            // Verifica se a estrutura possui lançamento de item de fechamento
            $lancFechamento = $this->entityManager->getRepository(Repositories::FPA_ITECALCESTRUT)->findOneBy(array(
                'nrorg' => $nrorg, 'nrestruturam' => $nrestruturam, 'dtocorrencia' => $dtmescompetenc, 'nreventom' => 3035, 'idativo' => 'S', 'nrcalculofolha' => $nrcalculofolha
            ));
            
            if (is_object($lancFechamento)) {
                return true;
            }
            
            return false;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function setEnvioFechamento($nrorg, $nrestruturam, $dsversao, $nrtpambiente, $nrtabela, $semMovimento) {
        try {
            if ($this->beforeEnvioFechamento($nrorg, $nrestruturam, DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true), $nrtabela, $semMovimento)) {
                // Busca o evento de fechamento do eSocial
                $evento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('dsversao' => $dsversao, 'nmevento' => 'S-1299'));
                
                $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true)->format('d/m/Y H:i:s');
                $this->beforeInsertEnvioCadastros($nrestruturam, $nrtpambiente, $evento->getNrevento(), $nrestruturam, $nrtabela, 'I', $dtmescompetenc);
                
                // Insere o envio de fechamento
                $nrenvio = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_ENVIOCADASTROS', 1, 12, true);
                $envio = $this->novoEsoEnvioCadastro($nrenvio, $evento->getNrevento(), $nrestruturam, $nrestruturam, $nrtabela, $dtmescompetenc, 'I', $nrtpambiente, null, null, $semMovimento);
                
                return $envio;
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function validaReaberturaEstrutura($nrorg, $nrestruturam, $dtmescompetenc, $nrtipomoviment) {
        try {
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment);
            
            // Verifica se a estrutura possui lançamento de item de reabertura
            $lancReabertura = $this->entityManager->getRepository(Repositories::FPA_ITECALCESTRUT)->findOneBy(array(
                'nrorg' => $nrorg, 'nrestruturam' => $nrestruturam, 'dtocorrencia' => $dtmescompetenc, 'nreventom' => 3036, 'idativo' => 'S', 'nrcalculofolha' => $nrcalculofolha
            ));
            
            if (is_object($lancReabertura)) {
                return true;
            }
            
            return false;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function setEnvioReabertura($nrorg, $nrestruturam, $dsversao, $nrtpambiente, $nrtabela) {
        try {
            if ($this->validaFechamentoEstrutura($nrorg, $nrestruturam, DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true), $nrtabela)) {
                if (!$this->validaReaberturaEstrutura($nrorg, $nrestruturam, DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true), $nrtabela)) {
                    // Busca o evento de reabertura do eSocial
                    $evento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('dsversao' => $dsversao, 'nmevento' => 'S-1298'));
                    
                    $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true)->format('d/m/Y H:i:s');
                    $this->beforeInsertEnvioCadastros($nrestruturam, $nrtpambiente, $evento->getNrevento(), $nrestruturam, $nrtabela, 'I', $dtmescompetenc);
                    
                    // Insere o envio de reabertura
                    $nrenvio = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_ENVIOCADASTROS', 1, 12, true);
                    $envio = $this->novoEsoEnvioCadastro($nrenvio, $evento->getNrevento(), $nrestruturam, $nrestruturam, $nrtabela, $dtmescompetenc, 'I', $nrtpambiente);
                    
                    return $envio;
                } else {
                    MsgProcesso::getInstance()->addError('Já existe reabertura dos eventos periódicos para a estrutura ' . $nrestruturam);
                }
            } else {
                MsgProcesso::getInstance()->addError('Não foi encontrado o evento de fechamento para a estrutura ' . $nrestruturam);
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function setEventoFechamentoEstrutura($nrorg, $nrestruturam, $nrtipomoviment) {
        try {
            $nrcalculo = $this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment);
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\FpaItecalcestrut $envio */
            $fpaItecalcestrut = Repositories::FPA_ITECALCESTRUT;
            
            $item = new $fpaItecalcestrut();
            $item->setNrorg($nrorg);
            $item->setNrtipoestrutura($this->hcmParameters->getTipoEstruturaLegalOrg());
            $item->setNrestruturam($nrestruturam);
            $item->setNreventom(3035);
            $item->setNrcalculofolha($nrcalculo);
            $item->setDtocorrencia(DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
            $item->setVrlancestrutura(0);
            $item->setIdativo('S');
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setNritecalcestrut(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCESTRUT', 1, 12, true));
            
            $this->entityManager->persist($item);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function setEventoReaberturaEstrutura($nrorg, $nrestruturam, $nrtipomoviment) {
        try {
            $nrcalculo = $this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment);
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\FpaItecalcestrut $envio */
            $fpaItecalcestrut = Repositories::FPA_ITECALCESTRUT;
            
            $item = new $fpaItecalcestrut();
            $item->setNrorg($nrorg);
            $item->setNrtipoestrutura($this->hcmParameters->getTipoEstruturaLegalOrg());
            $item->setNrestruturam($nrestruturam);
            $item->setNreventom(3036);
            $item->setNrcalculofolha($nrcalculo);
            $item->setDtocorrencia(DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
            $item->setVrlancestrutura(0);
            $item->setIdativo('S');
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setNritecalcestrut(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCESTRUT', 1, 12, true));
            
            $this->entityManager->persist($item);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function unsetEventoEstrutura($nrorg, $nrestruturam, $dtmescompetenc, $nrtipomoviment, $nreventom) {
        try {
            $nrcalculo = $this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment);
            
            // Verifica se a estrutura possui lançamento do evento
            $lancEstrutura = $this->entityManager->getRepository(Repositories::FPA_ITECALCESTRUT)->findOneBy(array(
                'nrorg' => $nrorg, 'nrestruturam' => $nrestruturam, 'dtocorrencia' => $dtmescompetenc, 'nreventom' => $nreventom, 'idativo' => 'S', 'nrcalculofolha' => $nrcalculo
            ));
            
            if (is_object($lancEstrutura)) {
                $lancEstrutura->setIdativo('N');
                $this->entityManager->persist($lancEstrutura);
                $this->entityManager->flush();
                return true;
            }
            
            return false;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * Método temporário para salvar arquivos no servidor AWS para evitar tráfego com o FileServer.
     * 
     * @param $nrparcnegocio
     * @param $nranexo
     * @param $fileContent
     * @param $fileName
     * @param $dsreferencia
     * @return string
     * @throws \Exception
     */
    public function saveXmlOnFilesPath($nrparcnegocio, $nranexo, $fileContent, $fileName, $dsreferencia, $path) {
        try {
            if (!is_dir($this->filesDir . $this->hcmParameters->getNrorgTrabalho() . "/" . $this->filesDirESocial . $path)) {
                mkdir($this->filesDir . $this->hcmParameters->getNrorgTrabalho() . "/" . $this->filesDirESocial . $path, 0777, true);
            }
            $filePath = $this->filesDir . $this->hcmParameters->getNrorgTrabalho() . "/" . $this->filesDirESocial . $path . $fileName . ".xml";
            if (is_file($filePath)) {
                unlink($filePath);
            }
            $xmlFile = fopen($filePath, "w") or die("Unable to open file $fileName.xml!");
            fwrite($xmlFile, $fileContent);
            fclose($xmlFile);
            chmod($filePath, 0777);  //changed to add the zero
            return $fileName;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrparcnegocio
     * @param $nranexo
     * @param $fileContent
     * @param $fileName
     * @param $dsreferencia
     * @return string
     * @throws \Exception
     */
    public function saveXmlOnFileServer($nrparcnegocio, $nranexo, $fileContent, $fileName, $dsreferencia, $customFolder='') {
        try {
            $row = array(
                "DSPATH" => array(
                    array(
                        "name"               => $fileName.".xml",
                        "lastModified"       => 1507122512763,
                        "lastModifiedDate"   => DateUtil::getDataAtual()->format(DateUtil::FORMATO_AMERICANO_DATAHORA),
                        "webkitRelativePath" => "",
                        "size"               => strlen($fileContent),
                        "type"               => "application/xml",
                        "b64File"            => "data:application/xml;base64," . (string)base64_encode($fileContent),
                    ),
                ),
                "NRORG"          => $this->hcmParameters->getNrorgTrabalho(),
                "NRPARAMETRO"    => "",
                "NRPARCNEGOCIO"  => $nrparcnegocio,
                "NRANEXO"        => $nranexo,
                "CDOPERINCLUSAO" => $this->hcmParameters->getCdoperador(),
                "CDOPERADOR_LOG" => $this->hcmParameters->getCdoperador(),
                "DSREFERENCIA"   => $dsreferencia,
                "IDFOTO"         => 'N',
            );
            $fileFieldName = "DSPATH";
            $this->zhFileUploader->uploadFilesInRow($row, $fileFieldName, $customFolder);
            //$upload = $this->uploadsFileService->setGpeAnexos($row); @TODO Save on GPE_ANEXOS
            $filePath = $row['DSPATH'][0]['path'];
            return $filePath;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nmEvento
     * @param $versao
     * @param $data
     * @param $save
     * @return string   return $xml string if save is false | $fileName id save is true
     * @throws \Exception
     */
    private function buildXml($nmEvento, $dsVersao, $data, $nameToSave, $saveXml=false) {
        try {
            $templateDir = $this->_xmlTemplatesPath . "/v_$dsVersao";
            $templateBuilder = new Template($data, $templateDir);
            $xml = $templateBuilder->build("{$nmEvento}.xml.twig");

            if (!is_null($nameToSave) && $saveXml) {
                $fileName = $nameToSave . ".xml";
                $fileDir = $this->_xmlFilesPath;
                if (!is_dir($fileDir)) {
                    mkdir($fileDir, 0777, true);
                }
                if (is_file($fileDir . DIRECTORY_SEPARATOR . $fileName)) {
                    unlink($fileDir . DIRECTORY_SEPARATOR . $fileName);
                }
                file_put_contents($fileDir . DIRECTORY_SEPARATOR . $fileName, $this->formatXML($xml));
                chmod($fileDir . DIRECTORY_SEPARATOR . $fileName, 0777);
            }

            return $xml;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $xml
     * @return mixed|\SimpleXMLElement|string
     */
    private function formatXML($xml)
    {
        // Substitui caracteres especias que devem ser evitados ao enviar o xml para o eSocial
        $xml = str_replace($this->_arrXmlEscape, $this->_arrXmlCaracter, $xml);

        // Este bloco transforma o texto gerado, para que as tags do twig sejam ocultadas, a fim de possibilitar a
        // formatacao do documentos, que somente e disponivel para um texto no formato .xml
        $xml = str_replace("{{", "aaa{{", $xml);
        $xml = str_replace("}}", "}}aaa", $xml);
        $xml = str_replace("{%", "<TEMPORARY_TAG>aaa{%", $xml);
        $xml = str_replace("%}", "%}aaa</TEMPORARY_TAG>", $xml);

        $xml = str_replace("&lt;", "<", $xml);
        $xml = str_replace("&gt;", ">", $xml);
        $xml = str_replace("&quot;", '"', $xml);
        $xml = str_replace("&", "e", $xml);     // Retirado carater '&' pois estava dando erro quando existia em algum campo de descrição

        $xml    = new \SimpleXMLElement($xml);
        $domxml = new \DOMDocument();
        $domxml->preserveWhiteSpace = false;
        $domxml->formatOutput = true;
        $domxml->loadXML($xml->asXML());

        // Remocao do texto temporario gerado.
        //$xml =  $domxml->saveHTML();
        $xml =  $domxml->saveXML();
        $xml = str_replace("aaa{{", "{{", $xml);
        $xml = str_replace("}}aaa", "}}", $xml);
        $xml = str_replace("<TEMPORARY_TAG>aaa{%", "{%", $xml);
        $xml = str_replace("%}aaa</TEMPORARY_TAG>", "%}", $xml);

        // Remove a tag XML "<?xml version='1.0'>" pois não é utilizada ao montar o SOAP para envio
        $xml = preg_replace('!^[^>]+>(\r\n|\n)!','',$xml);

        return $xml;
    }

    public function removerTagsVazias(\DOMDocument $xml) {
        $xpath = new \DOMXPath($xml);
        foreach($xpath->query('//*[not(*) and not(@*) and not(text()[normalize-space()])]') as $node) {
            if (!$node->hasAttributes()) {
                $node->parentNode->removeChild($node);
            }
        }
        return $xml;
    }

    /**
     * @param $xmlEvento
     * @return string
     * @throws \Exception
     */
    public function assinaXmlEvento($xmlEvento) {
        try {
            /* Formata o xml*/
            $domXml = new \DOMDocument("1.0", "UTF-8");
            $domXml->loadXML($xmlEvento, LIBXML_NOBLANKS);
            //$domXml = $this->removerTagsVazias($domXml); //opcional
            $domXml->formatOutput = true;
            $xmlTemp = $domXml->saveXML($domXml->documentElement);

            $domXml = new \DOMDocument("1.0", "UTF-8");
            $domXml->loadXML($xmlTemp);
            $domXml->encoding = "UTF-8";

            /* Busca qual tag será assinada */
            $tagAssinatura = "";
            $id = "";
            /** @var \DOMNode $nodeEvento */
            $nodeEvento = $domXml->getElementsByTagName('eSocial')->item(0)->childNodes[1]; //Pega o nome da tag principal do evento
            //$id = $nodeEvento->getAttribute("Id");
            $tagAssinatura = $nodeEvento->nodeName;
            /** @var \DOMNode $node */
            /*foreach ($nodeEvento->childNodes as $node) {
                if ($node->nodeType == 1 && ($node->nodeName != 'ideEvento' && $node->nodeName != 'ideEmpregador')) {
                    $tagAssinatura = $node->nodeName;
                    $node->setAttribute("Id", $id);
                }
            }*/

            $domXmlAssinado = $this->assinaXml($domXml, array($tagAssinatura));
            $xmlAssinado = $domXmlAssinado->saveXML($domXmlAssinado->documentElement);

            return $xmlAssinado;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param \DOMDocument $domXml
     * @param array $tags
     * @return \DOMDocument
     * @throws \Exception
     */
    private function assinaXml(\DOMDocument $domXml, array $tags) {
        try {
            error_reporting(E_ALL ^ E_WARNING);
            // Alteração temporária para ler somente uma vez o arquivo do FileServer e armazenar na variável certFile
//            if (fopen($this->_certFile,"r") == true) {
                // Get file name without extension
                //$data = file_get_contents($this->_certFile);
                //openssl_pkcs12_read($data, $cert, $this->_certPass);
                openssl_pkcs12_read($this->_certFile, $cert, $this->_certPass);

                if (is_null($cert)) {
                    throw new \Exception(Messages::HCM09000_ESO_GENERATE_CERT_KEY_ERROR, Messages::ERROR_MESSAGE_CODE);
                } else {
                    if (!file_exists($this->_certKeyFile)) {
                        file_put_contents($this->_certKeyFile, $cert["pkey"] . "\r\n" . $cert["cert"]);
                        chmod($this->_certKeyFile, 0777);
                    }

                    $chavePublica = $this->getPublicKey($cert["cert"]);
                    $chavePrivada = openssl_get_privatekey($cert["pkey"]);

                    foreach ($tags as $tag) {
                        $domXml = $this->assinaXmlAux($domXml, $tag, $chavePublica, $chavePrivada);
                    }

                    openssl_free_key($chavePrivada);
                    return $domXml;
                }
//            } else {
//                throw new \Exception(Messages::HCM09000_ESO_CERTIFICADO_NOT_FOUND_ERROR, Messages::ERROR_MESSAGE_CODE);
//            }
            error_reporting(E_ALL ^ E_NOTICE);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param \DOMDocument $domXml
     * @param $tag
     * @param $chavePublica
     * @param $chavePrivada
     * @return \DOMDocument
     * @throws \Exception
     */
    private function assinaXmlAux(\DOMDocument $domXml, $tag, $chavePublica, $chavePrivada) {
        try {
            $nodes = $domXml->getElementsByTagName($tag);
            foreach ($nodes as $node) {
                $root = $node->parentNode;
                $id = trim($node->getAttribute("Id"));
                //$dados = $node->C14N(false, false, null, null);
                $dados = $root->C14N(false, false, null, null);
                //$hashValue = hash("sha1", $dados, true);
                $hashValue = hash("sha256", $dados, true);
                $digValue = base64_encode($hashValue);

                $signature = $domXml->createElement("Signature");
                $root->appendChild($signature);
                $signature->setAttribute("xmlns", "http://www.w3.org/2000/09/xmldsig#");
                //$signature->setAttribute("Id", "Ass_" . $id);

                $signedInfo = $domXml->createElement("SignedInfo");
                $signature->appendChild($signedInfo);

                $novaTag = $domXml->createElement("CanonicalizationMethod");
                $signedInfo->appendChild($novaTag);
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

                $novaTag = $domXml->createElement("SignatureMethod");
                $signedInfo->appendChild($novaTag);
                //$novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256");

                $reference = $domXml->createElement("Reference");
                $signedInfo->appendChild($reference);
                //$reference->setAttribute("URI", "#" . $id);
                $reference->setAttribute("URI", "");

                $transforms = $domXml->createElement("Transforms");
                $reference->appendChild($transforms);

                $novaTag = $domXml->createElement("Transform");
                $transforms->appendChild($novaTag);
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#enveloped-signature");

                $novaTag = $domXml->createElement("Transform");
                $transforms->appendChild($novaTag);
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

                $novaTag = $domXml->createElement("DigestMethod");
                $reference->appendChild($novaTag);
                //$novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#sha1");
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/2001/04/xmlenc#sha256");

                $novaTag = $domXml->createElement("DigestValue", $digValue);
                $reference->appendChild($novaTag);

                $dados = $signedInfo->C14N(false, false, null, null);
                $assinatura = "";
                openssl_sign($dados, $assinatura, $chavePrivada, "SHA256");
                $signatureValue = base64_encode($assinatura);


                $novaTag = $domXml->createElement("SignatureValue", $signatureValue);
                $signature->appendChild($novaTag);

                $keyInfo = $domXml->createElement("KeyInfo");
                $signature->appendChild($keyInfo);

                $X509Data = $domXml->createElement("X509Data");
                $keyInfo->appendChild($X509Data);

                $novaTag = $domXml->createElement("X509Certificate", $chavePublica);
                $X509Data->appendChild($novaTag);
            }
            return $domXml;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    private function getPublicKey($key) {
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }

    /**
     * @param $idGrupo
     * @param $empregador
     * @param $dsVersao
     * @param $eventosLote
     * @return string
     * @throws \Exception
     */
    public function getXmlLote($idGrupo, $empregador, $dsVersao, $eventosLote) {
        try {
            $xmlData = array(
                "IDGRUPO" => ($idGrupo == 'P' ? '3' : ($idGrupo == 'N' ? '2' : '1')),
                "EMPTPINSCRICAO" => $empregador['TPINSCRICAOEMP'],
                "EMPNRINSCRICAO" => $empregador['NRINSCRICAOEMP'],
                "TRANSMTPINSCRICAO" => $this->_tpInscTransmissor,
                "TRANSMNRINSCRICAO" => $this->_nrInscTransmissor,
                "EVENTOS" => $eventosLote,
            );

            $xmlLote = $this->buildXml("envioLoteEventos", $dsVersao, $xmlData, "envioLoteEventos", true);
            /*$idGrupo = ($idGrupo == 'P' ? '3' : ($idGrupo == 'N' ? '2' : '1'));
            $xmlLote = $this->buildXmlLote($idGrupo, $empregador['TPINSCRICAOEMP'], $empregador['NRINSCRICAOEMP'], $this->_tpInscTransmissor, $this->_nrInscTransmissor, $eventosLote);
            foreach ($eventosLote as $evento) {
                $xmlLote = str_replace('XMLEVENTO', $evento['XMLEVENTO'], $xmlLote);
            }*/

            //return "envioLoteEventos.xml";
            return $xmlLote;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function buildXmlLote($idGrupo, $empTpInscricao, $empNrInscricao, $transmTpInscricao, $transmNrInscricao, $arrEventos) {
        try {
            $domXml = new \DOMDocument("1.0", "UTF-8");

            $enviarLoteEventos = $domXml->createElement("EnviarLoteEventos");
            $domXml->appendChild($enviarLoteEventos);

            $loteEventos = $domXml->createElement("loteEventos");
            $enviarLoteEventos->appendChild($loteEventos);

            $eSocial = $domXml->createElement("eSocial");
            $eSocial->setAttribute("xmlns", "http://www.esocial.gov.br/schema/lote/eventos/envio/v1_1_1");
            $loteEventos->appendChild($eSocial);

            $envioLoteEventos = $domXml->createElement("envioLoteEventos");
            $envioLoteEventos->setAttribute("grupo", $idGrupo);
            $eSocial->appendChild($envioLoteEventos);

            $ideEmpregador = $domXml->createElement("ideEmpregador");
            $envioLoteEventos->appendChild($ideEmpregador);

            $tpInsc = $domXml->createElement("tpInsc", $empTpInscricao);
            $nrInsc = $domXml->createElement("nrInsc", $empNrInscricao);
            $ideEmpregador->appendChild($tpInsc);
            $ideEmpregador->appendChild($nrInsc);

            $ideTransmissor = $domXml->createElement("ideTransmissor");
            $envioLoteEventos->appendChild($ideTransmissor);

            $tpInsc = $domXml->createElement("tpInsc", $transmTpInscricao);
            $nrInsc = $domXml->createElement("nrInsc", $transmNrInscricao);
            $ideTransmissor->appendChild($tpInsc);
            $ideTransmissor->appendChild($nrInsc);

            $eventos = $domXml->createElement("eventos");
            $envioLoteEventos->appendChild($eventos);

            foreach ($arrEventos as $dadosEvento) {
                $evento = $domXml->createElement("evento", "XMLEVENTO");
                $evento->setAttribute("Id", $dadosEvento['IDEVENTO']);
                //$evento->appendChild($domXml->importNode($dadosEvento['XMLEVENTO']->documentElement, true));
                $eventos->appendChild($evento);
            }

            return $domXml->saveXML($domXml->documentElement);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrRecibo
     * @param $dsVersao
     * @return string
     * @throws \Exception
     */
    public function getXmlConsulta($nrRecibo, $dsVersao) {
        try {
            $xmlData = array(
                "NRRECIBO" => $nrRecibo,
            );
            $xmlConsulta = $this->buildXml("consultaLoteEventos", $dsVersao, $xmlData, "consultaLoteEventos", true);
            return "consultaLoteEventos.xml";
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $fileName
     * @return string
     * @throws \Exception
     */
    public function enviarLote($eventosLote, $xmlLote) {
        try {
            /*if (!is_file($this->_xmlFilesPath . DIRECTORY_SEPARATOR . $xmlLote)) {
                throw new \Exception(Messages::HCM09000_ESO_FILE_LOTE_NOT_FOUND_ERROR, Messages::ERROR_MESSAGE_CODE);
            }

            $conteudo = file_get_contents($this->_xmlFilesPath . DIRECTORY_SEPARATOR . $xmlLote);
            $soapVar = new \SoapVar($conteudo, XSD_ANYXML);*/
            $soapVar = new \SoapVar($xmlLote, XSD_ANYXML);
            
            if ($this->_tpAmb == self::TPAMB_PROD) {
                $wsLocation = self::URL_ENVIO_PRODUCAO . self::LOCATION_ENVIO;
                $wsUri = self::URL_ENVIO_PRODUCAO;
                $wsWsdl = self::URL_ENVIO_PRODUCAO . self::LOCATION_ENVIO . '?wsdl';
                $wsFuncion = self::FUNCTION_ENVIO;
            } else {
                $wsLocation = self::URL_PRODREST . self::LOCATION_ENVIO;
                $wsUri = self::URL_PRODREST;
                $wsWsdl = self::URL_PRODREST . self::LOCATION_ENVIO . '?wsdl';
                $wsFuncion = self::FUNCTION_ENVIO;
            }

            $resultado = $this->buildSoapCall(
                $this->_certKeyFile, $wsLocation, $wsUri, $wsWsdl, $wsFuncion, $soapVar
            );

            if ($resultado->EnviarLoteEventosResult) {
                $xmlResult = $resultado->EnviarLoteEventosResult->any;
                $retorno = $this->trataRetornoEnvioLote($eventosLote, $xmlResult);
                return $retorno;
            }  else {
                throw new \Exception($resultado->__toString());
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function trataRetornoEnvioLote($eventosLote, $xmlResult) {
        try {
            $codRetorno = null;
            $idStatus = null;
            $nrRecibo = null;
            $cdErro = null;
            $msg = null;

            $xmlObj = simplexml_load_string($xmlResult);

            $codRetorno = $xmlObj->retornoEnvioLoteEventos->status->cdResposta;
            switch($xmlObj->retornoEnvioLoteEventos->status->cdResposta) {
                case self::CD_RET_LOTE_201:
                    $idStatus = self::IDSTATUS_PROCESSAMENTO;
                    $cdErro = null;
                    $msg = (string) $xmlObj->retornoEnvioLoteEventos->status->descResposta;
                    $dtRecepcao = (string) $xmlObj->retornoEnvioLoteEventos->dadosRecepcaoLote->dhRecepcao;
                    $nrRecibo = (string) $xmlObj->retornoEnvioLoteEventos->dadosRecepcaoLote->protocoloEnvio;
                    break;
                case self::CD_RET_LOTE_202:
                    $idStatus = self::IDSTATUS_PROCESSAMENTO;
                    $cdErro = null;
                    $msg = $this->buildMsgErroLote($xmlResult);
                    $dtRecepcao = (string) $xmlObj->retornoEnvioLoteEventos->dadosRecepcaoLote->dhRecepcao;
                    $nrRecibo = (string) $xmlObj->retornoEnvioLoteEventos->dadosRecepcaoLote->protocoloEnvio;
                    break;
                case self::CD_RET_LOTE_301:
                    $idStatus = self::IDSTATUS_ERRO_ENVIO;
                    $cdErro = (string) $xmlObj->retornoEnvioLoteEventos->status->cdResposta;
                    $msg = null;
                    $dtRecepcao = null;
                    $nrRecibo = null;
                    break;
                case self::CD_RET_LOTE_401:
                case self::CD_RET_LOTE_402:
                case self::CD_RET_LOTE_403:
                case self::CD_RET_LOTE_404:
                case self::CD_RET_LOTE_405:
                default:
                    $idStatus = self::IDSTATUS_ERRO_ENVIO;
                    $cdErro = (string) $xmlObj->retornoEnvioLoteEventos->status->cdResposta;
                    $msg = $this->buildMsgErroLote($xmlResult);
                    $dtRecepcao = null;
                    $nrRecibo = null;
                    break;
            }

            if (!is_null($dtRecepcao)) {
                $dtRecepcao = DateUtil::getDataDeString(substr($dtRecepcao,0,10).' '.substr($dtRecepcao,11,8), 'Y-m-d H:i:s');
            }

            foreach ($eventosLote as $evento) {
                // @TODO Atualizar atributos dos envios inclusive datas de envio e resposta
                //$evento['IDSTATUS'] = $idStatus;
                //$evento['CDERRO'] = $cdErro;
                //$evento['DSOBSERVACAO'] = $msg;
                //$evento['DTRESPOSTA'] = $dtRecepcao;
                //$evento['NRRECIBO'] = $nrRecibo;

                //Salva o xml de retorno do envio
                //$fileName = 'retorno_' . $evento['IDEVENTO'] . ".xml";
                //$fileName = 'retornoEnvioLote' . DateUtil::getDataAtual()->format('YmdHis');
                $fileName = 'lote' . DateUtil::getDataAtual()->format('YmdHis');
                /*$fileDir = $this->_xmlFilesPath;
                if (!is_dir($fileDir)) {
                    mkdir($fileDir, 0777, true);
                }
                if (is_file($fileDir . DIRECTORY_SEPARATOR . $fileName)) {
                    unlink($fileDir . DIRECTORY_SEPARATOR . $fileName);
                }
                file_put_contents($fileDir . DIRECTORY_SEPARATOR . $fileName, $this->formatXML($xmlObj->asXML()));
                chmod($fileDir . DIRECTORY_SEPARATOR . $fileName, 0777);*/

                /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEnviocadastros $envio */
                $envio = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->find(array(
                    'nrenvio' => $evento['NRENVIO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho()
                ));
                /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEvento $evento */
                $evento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array(
                    'nrevento' => $envio->getNrevento()
                ));

                // Alteração temporária para salvar os arquivos no servidor AWS
                /*if (!is_null($envio->getDsdirxmlretenv())) {
                    $this->zhFileUploader->deleteFile($envio->getDsdirxmlretenv());
                }*/
                $filePath = ($this->_tpAmb == 1 ? 'Producao' : 'ProducaoRestrita') . "/" .
                            ($evento->getIdgrupo() == 'P' ? 'Periodicos' : ($evento->getIdgrupo() == 'N' ? 'NaoPeriodicos' : 'Cadastros')) . "/" .
                            $this->_nrInscEmpregador . "/" .
                            'RetornoEnvio';
                $xmlPath = $this->saveXmlOnFileServer(
                    $envio->getNrestruturam(), null, $xmlObj->asXML(), $fileName, $envio->getIdevento(), $filePath
                );
                /*$xmlPath = $this->saveXmlOnFilesPath(
                    $envio->getNrestruturam(), null, $xmlObj->asXML(), $fileName, $envio->getIdevento(), $filePath
                );*/

                $envio->setIdstatus($idStatus);
                $envio->setCderro($cdErro);
                $envio->setDsobservacao(substr($msg,0,4000));
                $envio->setDsobservacao2(null);
                $envio->setDtresposta($dtRecepcao);
                $envio->setNrrecibo($nrRecibo);
                $envio->setDsdirxmlretenv($xmlPath);
                $envio->setDtultatu(DateUtil::getDataAtual());
                $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                $this->entityManager->persist($envio);
                $this->entityManager->flush();
                
                // Salva histórico de envio
                $this->salvaHistoricoEnvio($envio);
            }

            return array(
                'COD' => $codRetorno,
                'REC' => $nrRecibo,
                'MSG' => $msg,
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $xmlResponse
     * @return string
     * @throws \Exception
     */
    public function buildMsgErroLote($xmlResponse) {
        try {
            $xmlObj = simplexml_load_string($xmlResponse);

            $msg = '';
            $msg .= $xmlObj->retornoEnvioLoteEventos->status->cdResposta . '-' . $xmlObj->retornoEnvioLoteEventos->status->descResposta;

            foreach ($xmlObj->retornoEnvioLoteEventos->status->ocorrencias->ocorrencia as $ocorrencia) {
                $msg .= "\n[" . (isset($this->_tipoOcorRetorno[(string)$ocorrencia->tipo]) ? $this->_tipoOcorRetorno[(string)$ocorrencia->tipo] : (string)$ocorrencia->tipo) . '] ' . $ocorrencia->codigo . '-' . $ocorrencia->descricao;
                if ($ocorrencia->localizacao) {
                    $msg .= " (" . $ocorrencia->localizacao . ")";
                }
            }

            return $msg;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param $xmlResponse
     * @return string
     * @throws \Exception
     */
    public function salvaHistoricoEnvio($envio) {
        try {
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoHistoricoenvio $envio */
            $esoHitoricoenvio = Repositories::ESO_HISTORICOENVIO;
            
            $historico = new $esoHitoricoenvio();
            $historico->setNrorg($envio->getNrorg());
            $historico->setNrenvio($envio->getNrenvio());
            $historico->setNrevento($envio->getNrevento());
            $historico->setIdevento($envio->getIdevento());
            $historico->setNrtpambiente($envio->getNrtpambiente());
            $historico->setCdprotocolo($envio->getNrrecibo());
            $historico->setDtenvio($envio->getDtenvio());
            $historico->setDtresposta($envio->getDtresposta());
            $historico->setDsobservacao($envio->getDsobservacao());
            $historico->setDsdirxmlenv($envio->getDsdirxmlenv());
            $historico->setDsdirxmlretenv($envio->getDsdirxmlretenv());
            $historico->setDtinclusao(DateUtil::getDataAtual());
            $historico->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $historico->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $historico->setNrhistoricoenvio(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_HISTORICOENVIO', 1, 12, true));
            
            $this->entityManager->persist($historico);

            return $historico;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $xmlConsulta
     * @return string
     * @throws \Exception
     */
    public function consultarEvento($nrRecibo, $xmlConsulta, $idExclusao=false, $jenkins = false) {
        try {
            if (!is_file($this->_xmlFilesPath . DIRECTORY_SEPARATOR . $xmlConsulta)) {
                throw new \Exception(Messages::HCM09000_ESO_FILE_CONSULTA_NOT_FOUND_ERROR, Messages::ERROR_MESSAGE_CODE);
            }

            $conteudo = file_get_contents($this->_xmlFilesPath . DIRECTORY_SEPARATOR . $xmlConsulta);
            $soapVar = new \SoapVar($conteudo, XSD_ANYXML);
            
            if ($this->_tpAmb == self::TPAMB_PROD) {
                $wsLocation = self::URL_CONSULTA_PRODUCAO . self::LOCATION_CONSULTA;
                $wsUri = self::URL_CONSULTA_PRODUCAO;
                $wsWsdl = self::URL_CONSULTA_PRODUCAO . self::LOCATION_CONSULTA . '?wsdl';
                $wsFuncion = self::FUNCTION_CONSULTA;
            } else {
                $wsLocation = self::URL_PRODREST . self::LOCATION_CONSULTA;
                $wsUri = self::URL_PRODREST;
                $wsWsdl = self::URL_PRODREST . self::LOCATION_CONSULTA . '?wsdl';
                $wsFuncion = self::FUNCTION_CONSULTA;
            }

            $resultado = $this->buildSoapCall(
                $this->_certKeyFile, $wsLocation, $wsUri, $wsWsdl, $wsFuncion, $soapVar
            );

            if ($resultado->ConsultarLoteEventosResult) {
                $xmlResult = $resultado->ConsultarLoteEventosResult->any;
                $retorno = $this->trataRetornoConsultaLote($nrRecibo, $xmlResult, $idExclusao);
                return $retorno;
            }  else {
                throw new \Exception($resultado->__toString());
            }
        } catch(\Exception $e) {
            if($jenkins){
                return array("MSG" => $e->getMessage());
            }
            throw $e;
        }
    }

    public function trataRetornoConsultaLote($nrRecibo, $xmlResult, $idExclusao=false) {
        try {
            $tmpEspera = 0;
            $codRetorno = null;
            $idEvento = null;
            $msg = null;
            $msgRetornoEvento = null;
            $nrRecibo2 = null;
            $retornoEvento = null;

            $xmlObj = simplexml_load_string($xmlResult);

            $codRetorno = (string)$xmlObj->retornoProcessamentoLoteEventos->status->cdResposta;
            switch($xmlObj->retornoProcessamentoLoteEventos->status->cdResposta) {
                case self::CD_RET_CONS_101:
                    $msg = (string) $xmlObj->retornoProcessamentoLoteEventos->status->descResposta;
                    if ($xmlObj->retornoProcessamentoLoteEventos->status->tempoEstimadoConclusao) {
                        $msg .= ' - Tempo Estimado Conclusão: '. (string) $xmlObj->retornoProcessamentoLoteEventos->status->tempoEstimadoConclusao;
                        $tmpEspera = (string) $xmlObj->retornoProcessamentoLoteEventos->status->tempoEstimadoConclusao;
                    }
                    break;
                case self::CD_RET_CONS_201:
                case self::CD_RET_CONS_202:
                    //$msg = (string) $xmlObj->retornoProcessamentoLoteEventos->status->descResposta;
                    $msg = "";
                    foreach ($xmlObj->retornoProcessamentoLoteEventos->retornoEventos->evento as $evento) {
                        $msgRetornoEvento = '';
                        //Busca o Id do evento
                        foreach ($evento[0]->attributes() as $attName => $attValue) {
                            if ($attName == 'Id') {
                                $idEvento = (string) $attValue;
                            }
                        }

                        //Caso seja o evento de exclusão dos dados do empregador retorna as ocorrências como mensagem pois não existe registro de envio para salvar os dados
                        if ($idExclusao) {
                            $codRetorno = (string) $evento->retornoEvento->eSocial->retornoEvento->processamento->cdResposta;
                            $msg = (string) $evento->retornoEvento->eSocial->retornoEvento->processamento->descResposta;
                            $nrRecibo2 = (string) $evento->retornoEvento->eSocial->retornoEvento->recibo->nrRecibo;
                            foreach ($evento->retornoEvento->eSocial->retornoEvento->processamento->ocorrencias->ocorrencia as $ocorrencia) {
                                $codRetorno = (string) $ocorrencia->codigo;
                                $msg = (string) $ocorrencia->descricao;
                            }
                        } else {
                            //Trata o retono de processamento do evento e salva os xmls de retorno e totalização
                            if (isset($evento->tot[1])) {
                                // Retorno para os eventos S-1299 e S-1295 em que existem dois totalizadores (S-5011 e S-5012)
                                $eventoTot = clone($evento);
                                unset($eventoTot->retornoEvento);
                                
                                $msgRetornoEvento = $this->trataRetornoProcessamentoEvento($idEvento, $evento->retornoEvento, $eventoTot, $nrRecibo);
                            } else {
                                // Retorno para eventos com apenas um totalizador
                                $msgRetornoEvento = $this->trataRetornoProcessamentoEvento($idEvento, $evento->retornoEvento, $evento->tot, $nrRecibo);
                            }
                            if (!empty($msgRetornoEvento) && strpos($msg, $msgRetornoEvento) === false) {
                                $msg .= " " . $msgRetornoEvento;
                            }
                        }
                    }
                    if (empty($msg)) {
                        $msg = (string) $xmlObj->retornoProcessamentoLoteEventos->status->descResposta;
                    } else {
                        $msg = "Lote Processado. " . $msg;
                    }
                    break;
                case self::CD_RET_CONS_301:
                case self::CD_RET_CONS_401:
                case self::CD_RET_CONS_402:
                case self::CD_RET_CONS_403:
                case self::CD_RET_CONS_404:
                case self::CD_RET_CONS_405:
                case self::CD_RET_CONS_501:
                case self::CD_RET_CONS_502:
                case self::CD_RET_CONS_503:
                case self::CD_RET_CONS_504:
                case self::CD_RET_CONS_505:
                default:
                    $msg = $this->buildMsgErroConsulta($xmlResult);
                    
                    /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEnviocadastros $envio */
                    $envios = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findBy(array(
                        'nrrecibo' => $nrRecibo, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()
                    ));
                    
                    //$fileName = 'retornoConsultaEvento' . DateUtil::getDataAtual()->format('YmdHis');
                    $fileName = 'recibo' . $nrRecibo . DateUtil::getDataAtual()->format('YmdHis');
                    foreach ($envios as $envio) {
                        // Alteração temporária para salvar os arquivos no servidor AWS
                        /*if (!is_null($envio->getDsdirxmlretpro())) {
                            $this->zhFileUploader->deleteFile($envio->getDsdirxmlretpro());
                        }*/
                        /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEvento $evento */
                        $evento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array(
                            'nrevento' => $envio->getNrevento()
                        ));
                        $filePath = ($this->_tpAmb == 1 ? 'Producao' : 'ProducaoRestrita') . "/" .
                                    ($evento->getIdgrupo() == 'P' ? 'Periodicos' : ($evento->getIdgrupo() == 'N' ? 'NaoPeriodicos' : 'Cadastros')) . "/" .
                                    $this->_nrInscEmpregador . "/" .
                                    'RetornoProcessamento';
                        $xmlPath = $this->saveXmlOnFileServer(
                            $envio->getNrestruturam(), null, $xmlResult, $fileName, $envio->getIdevento(), $filePath
                        );
                        /*$xmlPath = $this->saveXmlOnFilesPath(
                            $envio->getNrestruturam(), null, $xmlResult, $fileName, $envio->getIdevento(), $filePath
                        );*/
                        
                        $envio->setCderro($codRetorno);
                        //$envio->setDtprocessamento($dtProcessamento);
                        $envio->setDsobservacao(substr($msg,0,4000));
                        //$envio->setDsobservacao2(substr($dsObservacao2,0,1000));
                        $envio->setDsdirxmlretpro($xmlPath);
                        $envio->setDtultatu(DateUtil::getDataAtual());
                        $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $this->entityManager->persist($envio);
                        $this->entityManager->flush();
                    }
                    break;
            }

            return array(
                'COD' => $codRetorno,
                'MSG' => $msg,
                'TMP' => $tmpEspera,
                'REC' => $nrRecibo2,
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $xmlResponse
     * @return string
     * @throws \Exception
     */
    public function buildMsgErroConsulta($xmlResponse) {
        try {
            $xmlObj = simplexml_load_string($xmlResponse);

            $msg = '';
            
            if ($xmlObj->retornoEnvioLoteEventos) {
                $msg .= $xmlObj->retornoEnvioLoteEventos->status->cdResposta . '-' . $xmlObj->retornoEnvioLoteEventos->status->descResposta;
    
                foreach ($xmlObj->retornoEnvioLoteEventos->status->ocorrencias->ocorrencia as $ocorrencia) {
                    $msg .= "\n[" . (isset($this->_tipoOcorRetorno[(string)$ocorrencia->tipo]) ? $this->_tipoOcorRetorno[(string)$ocorrencia->tipo] : (string)$ocorrencia->tipo) . '] ' . $ocorrencia->codigo . '-' . $ocorrencia->descricao;
                    if ($ocorrencia->localizacao) {
                        $msg .= " (" . $ocorrencia->localizacao . ")";
                    }
                }
            }
            
            if ($xmlObj->retornoProcessamentoLoteEventos) {
                $msg .= $xmlObj->retornoProcessamentoLoteEventos->status->cdResposta . '-' . $xmlObj->retornoProcessamentoLoteEventos->status->descResposta;
    
                if ($xmlObj->retornoProcessamentoLoteEventos->status->ocorrencias) {
                    foreach ($xmlObj->retornoProcessamentoLoteEventos->status->ocorrencias->ocorrencia as $ocorrencia) {
                        $msg .= "\n[" . (isset($this->_tipoOcorRetorno[(string)$ocorrencia->tipo]) ? $this->_tipoOcorRetorno[(string)$ocorrencia->tipo] : (string)$ocorrencia->tipo) . '] ' . $ocorrencia->codigo . '-' . $ocorrencia->descricao;
                        if ($ocorrencia->localizacao) {
                            $msg .= " (" . $ocorrencia->localizacao . ")";
                        }
                    }
                }
            }

            return $msg;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaPossibilidadeAlteracao($nrchave, $situacaofunc, $editarexcluir) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        if ($situacaofunc === "Gozo de Ferias") {
            $tipoTabela = 2;
            $resultado = $this->eSocialRepository->getEnvioEventos($tipoTabela, $nrchave, 'S-2230', $nrorg, $nrorgpadrao);
        } else if ($situacaofunc === "Afastamento" ) {
            $tipoTabela = null;
            $resultado = $this->eSocialRepository->getEnvioEventos($tipoTabela, $nrchave, 'S-2230', $nrorg, $nrorgpadrao);
        }
        
        //separação dos dois eventos obtidos
        $itens = $resultado[0];
        if (isset($resultado[1]))
            $itens2 = $resultado[1];
        
        $resposta['EDICAO'] = true; $resposta['EXCLUSAO'] = true;
        
        //verifica se existe o evento S-3000 concluído para permitir a exclusão
        if (isset($itens2)) {
            foreach ($itens2 as $item2) {
                if ($item2['IDSTATUS'] === 'E' || $item2['IDSTATUS'] === 'R' || $item2['IDSTATUS'] === 'P' || $item2['IDSTATUS'] === 'M') {
                    $resposta['EDICAO'] = false;
                    $resposta['EXCLUSAO'] = false;
                } else if ($item2['IDSTATUS'] === 'C' || $item2['IDSTATUS'] === 'A') {
                    $resposta['EDICAO'] = true;
                    $resposta['EXCLUSAO'] = true;
                }
                if($resposta['EXCLUSAO'] === false || $resposta['EDICAO'] === false)
                    break;
            }
        } else if (count($itens) > 0) {
            foreach($itens as $item){
                if ($item['IDSTATUS'] === 'E' || $item['IDSTATUS'] === 'R' || $item['IDSTATUS'] === 'P') {
                    $resposta['EDICAO'] = true;
                    $resposta['EXCLUSAO'] = false;
                } else if ($item['IDSTATUS'] === 'C' || $item['IDSTATUS'] === 'A' || $item['IDSTATUS'] === 'M') {
                    $resposta['EDICAO'] = false;
                    $resposta['EXCLUSAO'] = false;
                }
                if($resposta['EXCLUSAO'] === false || $resposta['EDICAO'] === false)
                    break;
            }
        }

        if($editarexcluir == 'editar'){
            return $resposta['EDICAO'];
        } else if ($editarexcluir == 'excluir'){
            return $resposta['EXCLUSAO'];
        } 
        
    }

    public function trataRetornoProcessamentoEvento($idEvento, \SimpleXMLElement $xmlRetornoEvento, \SimpleXMLElement $xmlRetornoTotEvento, $nrRecibo = null) {
        try {
            $msgRetorno = '';
            $dtProcessamento = null;
            $nrRecibo2 = null;
            $dsObservacao = null;
            $dsObservacao2 = null;
            $dsInfComplem = null;
            $xmlTotPath = null;

            //Salva o xml de retorno do evento
            //$fileName = 'consulta_' . $idEvento . ".xml";
            //$fileName = 'retornoConsulta' . $idEvento . DateUtil::getDataAtual()->format('YmdHis');
            $fileName = 'evento' . $idEvento . DateUtil::getDataAtual()->format('YmdHis');
            $fileNameTot = 'total' . $idEvento . DateUtil::getDataAtual()->format('YmdHis');
            /*$fileDir = $this->_xmlFilesPath;
            if (!is_dir($fileDir)) {
                mkdir($fileDir, 0777, true);
            }
            if (is_file($fileDir . DIRECTORY_SEPARATOR . $fileName)) {
                unlink($fileDir . DIRECTORY_SEPARATOR . $fileName);
            }
            file_put_contents($fileDir . DIRECTORY_SEPARATOR . $fileName, $this->formatXML($xmlRetornoEvento->asXML()));
            chmod($fileDir . DIRECTORY_SEPARATOR . $fileName, 0777);*/

            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEnviocadastros $envio */
            if(!empty($nrRecibo)){
                $envio = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findOneBy(array(
                    'idevento' => $idEvento, 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrrecibo' => $nrRecibo
                ));
            }else{
                $envio = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findOneBy(array(
                    'idevento' => $idEvento, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()
                ));
            }
            if (is_object($envio)) {
                if ($envio->getIdstatus() != self::IDSTATUS_CONCLUIDO_ADVERTENCIA && $envio->getIdstatus() != self::IDSTATUS_CONCLUIDO) {
                    // Alteração temporária para salvar os arquivos no servidor AWS
                    /*if (!is_null($envio->getDsdirxmlretpro())) {
                        $this->zhFileUploader->deleteFile($envio->getDsdirxmlretpro());
                    }*/
                    /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEvento $evento */
                    $evento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array(
                        'nrevento' => $envio->getNrevento()
                    ));
                    $filePath = ($this->_tpAmb == 1 ? 'Producao' : 'ProducaoRestrita') . "/" .
                                ($evento->getIdgrupo() == 'P' ? 'Periodicos' : ($evento->getIdgrupo() == 'N' ? 'NaoPeriodicos' : 'Cadastros')) . "/" .
                                $this->_nrInscEmpregador . "/" .
                                'RetornoProcessamento';
                    $xmlPath = $this->saveXmlOnFileServer(
                        $envio->getNrestruturam(), null, $xmlRetornoEvento->asXML(), $fileName, $envio->getIdevento(), $filePath
                    );
                    /*$xmlPath = $this->saveXmlOnFilesPath(
                        $envio->getNrestruturam(), null, $xmlRetornoEvento->asXML(), $fileName, $envio->getIdevento(), $filePath
                    );*/
                    
                    if ($xmlRetornoTotEvento->asXML()) {
                        $filePath = ($this->_tpAmb == 1 ? 'Producao' : 'ProducaoRestrita') . "/" .
                                ($evento->getIdgrupo() == 'P' ? 'Periodicos' : ($evento->getIdgrupo() == 'N' ? 'NaoPeriodicos' : 'Cadastros')) . "/" .
                                $this->_nrInscEmpregador . "/" .
                                'Totalizador';
                        $xmlTotPath = $this->saveXmlOnFileServer(
                            $envio->getNrestruturam(), null, $xmlRetornoTotEvento->asXML(), $fileNameTot, $envio->getIdevento(), $filePath
                        );
                        /*$xmlTotPath = $this->saveXmlOnFilesPath(
                            $envio->getNrestruturam(), null, $xmlRetornoTotEvento->asXML(), $fileNameTot, $envio->getIdevento(), $filePath
                        );*/
                        $this->trataRetornoTotalizador($envio, $xmlRetornoTotEvento);
                    }
        
                    $retornoEvento = $xmlRetornoEvento->eSocial->retornoEvento;
        
                    switch($retornoEvento->processamento->cdResposta) {
                        case self::CD_RET_EVEN_201:
                        case self::CD_RET_EVEN_202:
                            if ($retornoEvento->processamento->cdResposta == self::CD_RET_EVEN_202) {
                                $envio->setIdstatus(self::IDSTATUS_CONCLUIDO_ADVERTENCIA);
                                $envio->setCderro((string) $retornoEvento->processamento->cdResposta);
                                $msgErroEvento = $this->buildMsgErroEvento($retornoEvento->processamento);
                                $dsObservacao = $msgErroEvento['msg'];
                                $dsInfComplem = $msgErroEvento['inf'];
                            } else {
                                $envio->setIdstatus(self::IDSTATUS_CONCLUIDO);
                                $dsObservacao = $retornoEvento->processamento->descResposta;
                            }
                            
                            $dtProcessamento = (string) $retornoEvento->processamento->dhProcessamento;
                            $nrRecibo2 = (string) $retornoEvento->recibo->nrRecibo;
                            
                            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEvento $evento */
                            $evento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nrevento' => $envio->getNrevento()));
                            
                            // Caso seja consulta do evento de fechamento dos periódicos com sucesso de processamento, gera um item de fechamento para a estrutura
                            if ($evento->getNmevento() == 'S-1299') {
                                $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
                                $this->unsetEventoEstrutura($envio->getNrorg(), $envio->getNrestruturam(), $dtmescompetenc, $envio->getNrtabela(), 3035);
                                $this->unsetEventoEstrutura($envio->getNrorg(), $envio->getNrestruturam(), $dtmescompetenc, $envio->getNrtabela(), 3036);
                                $this->setEventoFechamentoEstrutura($envio->getNrorg(), $envio->getNrestruturam(), $envio->getNrtabela());
                            }
                            
                            // Caso seja consulta do evento de reabertura dos periódicos com sucesso de processamento, gera um item de reabertura para a estrutura
                            if ($evento->getNmevento() == 'S-1298') {
                                $this->setEventoReaberturaEstrutura($envio->getNrorg(), $envio->getNrestruturam(), $envio->getNrtabela());
                            }
                            
                            // Caso seja consulta do evento de admissão preliminiar processado com sucesso, atualiza o número do recibo na pessoa
                            if ($evento->getNmevento() == 'S-2190') {
                                $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
                                $gpePessoah = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaHistoricoAtualPorPessoa(
                                    $envio->getNrorg(), $envio->getNrchave(), $dtmescompetenc
                                );
                                   if (is_object($gpePessoah)) {
                                    $gpePessoah->setCdreciboadmpre($nrRecibo2);
                                    $this->entityManager->persist($gpePessoah);
                                }
                            }
                            
                            // Caso seja consulta do evento de reintegração processado com sucesso, inativa o envio de desligamento informando o envio da reintegração como envio de exclusão
                            if ($evento->getNmevento() == 'S-2298') {
                                $envioRescisao = current($this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosCadastrosEvento(
                                    $envio->getNrorg(), 'S-2299', $envio->getNrestruturam(), $envio->getNrtpambiente(), 'S', $envio->getNrchave(), null, array(), 'N'
                                ));
                                if (is_object($envioRescisao)) {
                                    $envioRescisao->setNrenvioexcl($envio->getNrenvio());
                                    $this->entityManager->persist($envioRescisao);
                                }
                            }
                            
                            break;
                        case self::CD_RET_EVEN_301:
                        case self::CD_RET_EVEN_401:
                        case self::CD_RET_EVEN_402:
                        case self::CD_RET_EVEN_403:
                        case self::CD_RET_EVEN_404:
                        case self::CD_RET_EVEN_405:
                        case self::CD_RET_EVEN_406:
                        case self::CD_RET_EVEN_411:
                        default:
                            $msgRetorno = 'Existem erros/avisos em eventos do lote. Favor verificar. ';
                            $envio->setIdstatus(self::IDSTATUS_ERRO_PROCESSAMENTO);
                            $envio->setCderro((string) $retornoEvento->processamento->cdResposta);
                            $dtProcessamento = (string) $retornoEvento->processamento->dhProcessamento;
                            $msgErroEvento = $this->buildMsgErroEvento($retornoEvento->processamento);
                            $dsObservacao = $msgErroEvento['msg'];
                            $dsInfComplem = $msgErroEvento['inf'];
                            // Alteração temporária para ler os arquivos no servidor AWS para evitar tráfego com o FileServer.
                            //$xmlEnv = $this->hcmParameters->getFileServerDirectory() . $envio->getDsdirxmlenv();
                            //$xmlEnv = $this->filesDir . $envio->getDsdirxmlenv() . ".xml";
                            // ----
                            //error_reporting(E_ALL ^ E_WARNING);
                            //if (fopen($xmlEnv, 'r')) {
                            //  /** \Teknisa\Model\Gestaopessoas\Entities\EsoEvento $evento */
                            //  $evento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array(
                            //      'nrevento' => $envio->getNrevento()
                            //  ));
                            //  /** \Teknisa\Model\Gestaopessoas\Entities\EsoResumo $eventoResumo */
                            //  $eventoResumo = $this->entityManager->getRepository(Repositories::ESO_RESUMO)->findOneBy(array(
                            //      'nrevento' => $envio->getNrevento(), 'nmregisropai' => 'eSocial'
                            //  ));
                            //  $dsObservacao2 = $this->validaSchemaXml($evento->getDsversao(), $eventoResumo->getNmregistro(), file_get_contents($xmlEnv));
                            //} else {
                            //  $dsObservacao2 = '';
                            //}
                            //error_reporting(E_ALL ^ E_NOTICE);
                            break;
                    }
        
                    if (!is_null($dtProcessamento)) {
                        $dtProcessamento = DateUtil::getDataDeString(substr($dtProcessamento,0,10).' '.substr($dtProcessamento,11,8), 'Y-m-d H:i:s');
                    }
        
                    $envio->setDtprocessamento($dtProcessamento);
                    $envio->setNrrecibo2($nrRecibo2);
                    $envio->setDsobservacao(substr($dsObservacao,0,4000));
                    $envio->setDsobservacao2(substr((is_null($dsObservacao2) ? $dsInfComplem : $dsObservacao2),0,4000));
                    $envio->setDsdirxmlretpro($xmlPath);
                    $envio->setDsdirxmlrettot($xmlTotPath);
                    $envio->setDtultatu(DateUtil::getDataAtual());
                    $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                    $this->entityManager->persist($envio);
                    $this->entityManager->flush();
                }
            } else {
                $msgRetorno .= 'Evento não encontrado (ID:'.$idEvento.').';
            }

            return $msgRetorno;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function trataRetornoTotalizador($envio, \SimpleXMLElement $xmlRetornoTotEvento) {
        try {
            //Trata retorno dos eventos totalizadores
            foreach ($xmlRetornoTotEvento->tot as $totalizador) {
                switch ((string)$totalizador['tipo']) {
                    case 'S5001':
                        $this->trataRetornoTotalizadorS5001($envio, $totalizador);
                        break;
                    case 'S5011':
                        $this->trataRetornoTotalizadorS5011($envio, $totalizador);
                        break;
                }
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function trataRetornoTotalizadorS5001($envio, \SimpleXMLElement $xmlEventoS5001) {
        try {
            $vrCalculado = null;
            $vrDescontado = null;
            $nrEstrutLotacao = null;
            $vrBaseMensal = null;
            $vrBase13 = null;
            $vrValorMensal = null;
            $vrValor13 = null;
            $vrSalFamiliaMensal = null;
            $vrSalFamilia13 = null;
            $vrMaternidadeMensal = null;
            $vrMaternidade13 = null;
            
            if($xmlEventoS5001->eSocial->evtBasesTrab->infoCpCalc){
                $vrCalculado = (float)$xmlEventoS5001->eSocial->evtBasesTrab->infoCpCalc->vrCpSeg;
                $vrDescontado = (float)$xmlEventoS5001->eSocial->evtBasesTrab->infoCpCalc->vrDescSeg;
            }
            if($xmlEventoS5001->eSocial->evtBasesTrab->infoCp){
                $nrEstrutLotacao = (string)$xmlEventoS5001->eSocial->evtBasesTrab->infoCp->ideEstabLot->codLotacao;
                if ($xmlEventoS5001->eSocial->evtBasesTrab->infoCp->ideEstabLot->infoCategIncid->infoBaseCS) {
                    foreach ($xmlEventoS5001->eSocial->evtBasesTrab->infoCp->ideEstabLot->infoCategIncid->infoBaseCS as $infoBaseCS) {
                        //valores mensais
                        if ((string)$infoBaseCS->ind13 == '0') {
                            switch ((string)$infoBaseCS->tpValor) {
                                case '11':
                                case '12':
                                case '13':
                                case '14':
                                case '15':
                                case '16':
                                case '17':
                                case '18':
                                case '19':
                                    $vrBaseMensal = (is_null($vrBaseMensal) ? 0 : $vrBaseMensal) + (float)$infoBaseCS->valor;
                                    break;
                                case '21':
                                case '22':
                                case '23':
                                    $vrValorMensal = (is_null($vrValorMensal) ? 0 : $vrValorMensal) + (float)$infoBaseCS->valor;
                                    break;
                                case '31':
                                    $vrSalFamiliaMensal = (is_null($vrSalFamiliaMensal) ? 0 : $vrSalFamiliaMensal) + (float)$infoBaseCS->valor;
                                    break;
                                case '32':
                                    $vrMaternidadeMensal = (is_null($vrMaternidadeMensal) ? 0 : $vrMaternidadeMensal) + (float)$infoBaseCS->valor;
                                    break;
                            }
                        }
                        
                        //valores 13
                        if ((string)$infoBaseCS->ind13 == '1') {
                            switch ((string)$infoBaseCS->tpValor) {
                                case '11':
                                case '12':
                                case '13':
                                case '14':
                                case '15':
                                case '16':
                                case '17':
                                case '18':
                                case '19':
                                    $vrBase13 = (is_null($vrBase13) ? 0 : $vrBase13) + (float)$infoBaseCS->valor;
                                    break;
                                case '21':
                                case '22':
                                case '23':
                                    $vrValor13 = (is_null($vrValor13) ? 0 : $vrValor13) + (float)$infoBaseCS->valor;
                                    break;
                                case '31':
                                    $vrSalFamilia13 = (is_null($vrSalFamilia13) ? 0 : $vrSalFamilia13) + (float)$infoBaseCS->valor;
                                    break;
                                case '32':
                                    $vrMaternidade13 = (is_null($vrMaternidade13) ? 0 : $vrMaternidade13) + (float)$infoBaseCS->valor;
                                    break;
                            }
                        }
                    }
                }
            }
            
            $envio->setVrcalculado($vrCalculado);
            $envio->setVrdescontado($vrDescontado);
            $envio->setNrestrutlotacao($nrEstrutLotacao);
            $envio->setVrbasemensal($vrBaseMensal);
            $envio->setVrbase13($vrBase13);
            $envio->setVrvalormensal($vrValorMensal);
            $envio->setVrvalor13($vrValor13);
            $envio->setVrsalfamilia($vrSalFamiliaMensal);
            $envio->setVrsalfamilia13($vrSalFamilia13);
            $envio->setVrvalormatermensal($vrMaternidadeMensal);
            $envio->setVrvalormater13($vrMaternidade13);
            $this->entityManager->persist($envio);
            $this->entityManager->flush();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function trataRetornoTotalizadorS5011($envio, \SimpleXMLElement $xmlEventoS5011) {
        try {
            $vrCalculado = null;
            $vrDescontado = null;
            $nrEstrutLotacao = null;
            $vrBaseMensal = null;
            $vrBase13 = null;
            $vrValorMensal = null;
            $vrValor13 = null;
            $vrSalFamiliaMensal = null;
            $vrSalFamilia13 = null;
            $vrMaternidadeMensal = null;
            $vrMaternidade13 = null;
            
            $vrCalculado = (float)$xmlEventoS5011->eSocial->evtCS->infoCS->infoCPSeg->vrCpSeg;
            $vrDescontado = (float)$xmlEventoS5011->eSocial->evtCS->infoCS->infoCPSeg->vrDescCP;
            /*$nrEstrutLotacao = (string)$xmlEventoS5011->eSocial->evtCS->infoCS->ideEstab->codLotacao;
            if ($xmlEventoS5011->eSocial->evtCS->infoCS->ideEstab->infoCategIncid->infoBaseCS) {
                foreach ($xmlEventoS5011->eSocial->evtCS->infoCS->ideEstab->infoCategIncid->infoBaseCS as $infoBaseCS) {
                    //valores mensais
                    if ((string)$infoBaseCS->ind13 == '0') {
                        switch ((string)$infoBaseCS->tpValor) {
                            case '11':
                                $vrBaseMensal = (float)$infoBaseCS->valor;
                                break;
                            case '21':
                                $vrValorMensal = (float)$infoBaseCS->valor;
                                break;
                            case '31':
                                $vrSalFamiliaMensal = (float)$infoBaseCS->valor;
                                break;
                            case '32':
                                $vrMaternidadeMensal = (float)$infoBaseCS->valor;
                                break;
                        }
                    }
                    
                    //valores 13
                    if ((string)$infoBaseCS->ind13 == '1') {
                        switch ((string)$infoBaseCS->tpValor) {
                            case '11':
                                $vrBase13 = (float)$infoBaseCS->valor;
                                break;
                            case '21':
                                $vrValor13 = (float)$infoBaseCS->valor;
                                break;
                            case '31':
                                $vrSalFamilia13 = (float)$infoBaseCS->valor;
                                break;
                            case '32':
                                $vrMaternidade13 = (float)$infoBaseCS->valor;
                                break;
                        }
                    }
                }
            }*/
            
            $envio->setVrcalculado($vrCalculado);
            $envio->setVrdescontado($vrDescontado);
            /*$envio->setNrestrutlotacao($nrEstrutLotacao);
            $envio->setVrbasemensal($vrBaseMensal);
            $envio->setVrbase13($vrBase13);
            $envio->setVrvalormensal($vrValorMensal);
            $envio->setVrvalor13($vrValor13);
            $envio->setVrsalfamilia($vrSalFamiliaMensal);
            $envio->setVrsalfamilia13($vrSalFamilia13);
            $envio->setVrvalormatermensal($vrMaternidadeMensal);
            $envio->setVrvalormater13($vrMaternidade13);*/
            $this->entityManager->persist($envio);
            $this->entityManager->flush();
        } catch(\Exception $e) {
            throw $e;
        }
    }


    /**
     * @param $processamento
     * @return string
     * @throws \Exception
     */
    public function buildMsgErroEvento(\SimpleXMLElement $processamento) {
        try {
            $msg = '';
            $msg .= $processamento->cdResposta . '-' . $processamento->descResposta;
            $inf = '';

            if ($processamento->ocorrencias->ocorrencia) {
                foreach ($processamento->ocorrencias->ocorrencia as $ocorrencia) {
                    $msg .= "\n[" . (isset($this->_tipoOcorRetorno[(string)$ocorrencia->tipo]) ? $this->_tipoOcorRetorno[(string)$ocorrencia->tipo] : (string)$ocorrencia->tipo) . '] ' . $ocorrencia->codigo . '-' . $ocorrencia->descricao;
                    if ($ocorrencia->localizacao) {
                        $msg .= " (" . $ocorrencia->localizacao . ")";
                    }
                    $inf .= $this->verificaInfComplementar((string)$ocorrencia->codigo, (string)$ocorrencia->descricao);
                }
            }

            return array(
                'msg' => $msg,
                'inf' => ($inf == '' ? null : $inf),
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaInfComplementar($codigo, $descricao) {
        try {
            $infComplementar = $this->entityManager->getRepository(Repositories::ESO_INFCOMPLEMENTAR)->findOneBy(array('cderro' => str_pad($codigo, 4, "0", STR_PAD_LEFT)));

            if (is_object($infComplementar)) {
                if ($infComplementar->getDsinfcomplementar()) {
                    return $infComplementar->getCderro() . "-" . $infComplementar->getDsinfcomplementar() . "\n";
                } else {
                    return '';
                }
            } else {
                return '';
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function validaSchemaXml($dsVersao, $schemaName, $xmlContent) {
        try {
            $validation = '';
            $dsVersao = str_replace('.','-',$dsVersao);
            $schema = $this->_xsdTemplatesPath . DIRECTORY_SEPARATOR . "v_$dsVersao" . DIRECTORY_SEPARATOR . $schemaName . '.xsd';

            if (file_exists($schema)) {
                $xml = new \DOMDocument("1.0", "UTF-8");
                $xml->loadXML($xmlContent, LIBXML_NOBLANKS);

                // Enable user error handling
                libxml_use_internal_errors(true);

                if (!$xml->schemaValidate($schema)) {
                    $validation = 'Erros encontrados:';

                    $errors = libxml_get_errors();
                    foreach ($errors as $error) {
                        $errorMsg = "\n";
                        /*switch ($error->level) {
                            case LIBXML_ERR_WARNING:
                                $errorMsg .= "Warning $error->code: ";
                                break;
                            case LIBXML_ERR_ERROR:
                                $errorMsg .= "Error $error->code: ";
                                break;
                            case LIBXML_ERR_FATAL:
                                $errorMsg .= "Fatal Error $error->code: ";
                                break;
                        }*/

                        // Retira a descrição do schema da mensagem de erro (primeira ocorrencia entre chaves)
                        $posIni = strpos($error->message, '{');
                        $posFim = strpos($error->message, '}');
                        $errorMsgSchema = substr($error->message, $posIni, $posFim-$posIni+1);
                        $errorMsg .= str_replace($errorMsgSchema, '', trim($error->message));

                        /*if ($error->file) {
                            $errorMsg .= " in $error->file";
                        }*/
                        //$errorMsg .= " on line $error->line";
                        $validation .= $errorMsg;
                    }
                    libxml_clear_errors();
                } else {
                    $validation = 'Não foram encontrados erros';
                }
            }
            return $validation;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function ativaEvento($nrenvio, $nrorg, $nrestruturam){
        $this->eSocialRepository->ativaEvento($nrenvio, $nrorg, $nrestruturam);
    }

    /*------------------------------------------------------------------------------------*/

    /**
     * @return \SoapClient
     * @throws \Exception
     */
    public function buildSoapCall($cert, $location, $uri, $wsdl, $functionName, $soapVar) {
        try {
            error_reporting(E_ALL ^ E_WARNING ^ E_ERROR);
            $options = array(
                'local_cert' => $cert,
                'trace' => 1,
                'location' => $location,
                'uri' => $uri,
                //'crypto_method' =>  STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT,
                'soap_version'   => SOAP_1_1,
                'stream_context' => stream_context_create(array(
                    'http' => array(
                        'user_agent' => 'PHP/SOAP',
                        'header' => "Accept: application/xml"
                    ),
                    'ssl' => array(
                        'verify_peer' => false,
                        'verify_peer_name' => false,
                        'allow_self_signed' => true,
                    )
                )),
                'exceptions'     => true,
                //'proxy_host'     => "192.168.122.3",
                //'proxy_port'     => 8080,
                //'proxy_login'    => "login",
                //'proxy_password' => "password",
                'connection_timeout' => 120,
            );

            $client = new \SoapClient($wsdl, $options);

            ini_set('default_socket_timeout', 5000);
            ini_set('soap.wsdl_cache_enabled',0);
            ini_set('soap.wsdl_cache_ttl',0);
            $resultado = $client->__soapCall($functionName, array($soapVar));

            if (get_class($resultado) == 'SoapFault') {
                /** @var \SoapFault $resultado */
                throw new \Exception($resultado->getMessage());
            }

            error_reporting(E_ALL ^ E_NOTICE);
            unset($client);
            return $resultado;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Método temporário para salvar log de execução do eSocial
     * 
     * @param $content
     * @param $closeFile
     */
    public function saveLogESocial($content, $closeFile=false) {
        try {
            if ($this->gravaLog == '1') {
                $logDir = $this->filesDir . $this->hcmParameters->getNrorgTrabalho() . "/log/";
                $this->logContent = $this->logContent . "[" . $this->logId . " " . DateUtil::getDataAtual()->format('H:i:s') . "] " . $content . "\n";
                
                if ($closeFile) {
                    if (!is_dir($logDir)) {
                        mkdir($logDir, 0777, true);
                    }
                    // Nome para criar arquivo de log por dia
                    $fileName = 'logESocial' . DateUtil::getDataAtual()->format('Ymd');
                    $filePath = $logDir . $fileName . ".txt";
                    if ($filePath === false) {
                        // Caso não consiga criar um novo arquivo utiliza o arquivo de log antigo
                        $filePath = $this->filesDir . "log/" . "logESocial" . ".txt";
                    }
                    $logFile = fopen($filePath, "a");
                    fwrite($logFile, $this->logContent . "=========================================\n");
                    fclose($logFile);
                    chmod($filePath, 0777);  //changed to add the zero
                    $this->logContent = '';
                }
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function convertSecondsToTime($seconds) {
        $t = round($seconds);
        return sprintf('%02d:%02d:%02d', ($t/3600),($t/60%60), $t%60);
    }
    
    
    /**
     * 
     * Rotina que verifica a existência de envios de periódicos pendentes antes do envio de 1200 ou 1210
     * 
     */
    
    public function getPendenciasEnvio ($nmevento, $nrchave, $nrorg){
        // $nrchave  = $row['NRCHAVE']; 
        // $nrorg    = $row['NRORG'];
        // $nmevento = $row['NMEVENTOM'];
        
        if ($nmevento == "S-1200" || $nmevento == "S-1210"){
            // retorna eventos pendentes ou false
            $eventosPendentes = $this->eSocialRepository->getEnviosNaoPeriodicosPendentes($nrchave, $nrorg, array(self::IDSTATUS_ERRO_ENVIO, self::IDSTATUS_ERRO_PROCESSAMENTO, self::IDSTATUS_PENDENTE));
            return !empty($eventosPendentes) ? $eventosPendentes : false;
            
        } else {
            return false;
        }
        
    }
    
    public function verificaEnvioEsocialRescisaoComplem ($dataSet, $response){
        try{
            Inicializa::Container($this->entityManager, $this->hcmParameters, $dataSet['DTPAGAMENTO']);
            Inicializa::carregaVariaveis();
            
            $this->_vinculom = TkDI::getContainer()->vinculom;
            
            $vinculos = $this->_vinculom->retornaVinculosPorTipoCalculo($dataSet['TP_CALCULO'],$dataSet['NRORG'], $this->hcmParameters->getDtcompetencia(), $dataSet['DSINICIO'], $dataSet['DSFIM'],
                                                                        DateUtil::getDataDeString(substr($dataSet['DTRESCISAOINI'],0, 10)), DateUtil::getDataDeString(substr($dataSet['DTRESCISAOFIM'], 0, 10)), 
                                                                        $dataSet['DSORDEM'], $dataSet['NRESTRUTURAM']);
            
            foreach($vinculos as $index => $vinculo){
                if($index == 0){
                    $dataSet['NRVINCULOM'] = $vinculo->getNrvinculom();
                }else{
                    $dataSet['NRVINCULOM'] .= ','.$vinculo->getNrvinculom();
                }
            }
            
            $enviosRetific = $this->eSocialRepository->retornaVinculosEnvioEsocialRescisaoComplem($dataSet);

            if($enviosRetific){
                $this->entityManager->beginTransaction();
                $nrestruturam = null;
                $paramESocial = null;
                $eventosLote = array();
                $idGrupo = null;
                $dsVersao = null;
                foreach($enviosRetific as $row){
                    if($nrestruturam != $row['NRESTRUTURAM']){
                        $paramESocial = $this->getParamESocial($row['NRESTRUTURAM']);
                        $this->setDadosTransmissao($row['NRESTRUTURAM']);
                        $nrestruturam = $row['NRESTRUTURAM'];
                    }
                    $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $row['NRENVIO']);
                    
                    $evento = $this->getEventoPorNomeVersao($row['NMEVENTO'], $row['DSVERSAO']);
                    
                    $dsVersao = str_replace('.','-',$row['DSVERSAO']);
                    $idGrupo = $evento->getIdgrupo();
                    
                    // Incluí o evento de exclusão/retificação referente ao envio que será excluído/retificado
                    $idOperacao = 'R';
                    $envio = $this->getEnvioCadastro($row['NRORG'],$row['NRENVIO']);
                    $nrenvioexcl = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_ENVIOCADASTROS', 1, 12, true);
                    $envioexcl = $this->novoEsoEnvioCadastro(
                        $nrenvioexcl, $evento->getNrevento(), $row['NRESTRUTURAM'], $row['NRCHAVE'], $row['NRTABELA'], $row['DTMESCOMPETENC'], $idOperacao, $row['NRTPAMBIENTE'], null, $row['NRVINCULOM']
                    );
                    
                    $arrEnvioExcl = $envioexcl->toArray();
                    $arrEnvioExcl['NMEVENTO'] = $evento->getNmevento();
                    $arrEnvioExcl['IDGRUPO'] = $evento->getIdgrupo();
                    $arrEnvioExcl['NRENVIOORIG'] = $row['NRENVIO'];
                    $arrEnvioExcl['NMEVENTOORIG'] = $row['NMEVENTO'];
                    $arrEnvioExcl['NRTABELAORIG'] = $row['NRTABELA'];
                    $arrEnvioExcl['DTMESCOMPORIG'] = \HCM\Util\DateUtil::getDataDeString($row['DTMESCOMPETENC'], \HCM\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
                    $arrEnvioExcl['NRRECIBO2ORIG'] = $row['NRRECIBO2'];
                    $arrEnvioExcl['DTRESPOSTAORIG'] = \HCM\Util\DateUtil::getDataDeString($row['DTRESPOSTA'], \HCM\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA, true)->format('d/m/Y');
                    
                    $arrEnvioExcl['DTMESCOMPETENC'] = is_object($arrEnvioExcl['DTMESCOMPETENC']) ? $arrEnvioExcl['DTMESCOMPETENC']->format('d/m/Y H:i:s') : $arrEnvioExcl['DTMESCOMPETENC'];
                    $xmlEvento = $this->getXmlEvento($dsVersao, $arrEnvioExcl);
                    
                    if ($xmlEvento) {
                        $eventosLote[] = $xmlEvento;
                    }
                    
                    foreach ($msgProcesso->getMessages() as $msg) {
                        $msgValidacao[] = array(
                            'NMEVENTO' => $row['NMEVENTO'],
                            'NRCHAVE' => $row['NRCHAVE'],
                            'MSG' => $msg['MSG'],
                            'TYPE' => $msg['TYPE'],
                        );
                    }
                    $msgProcesso->clear();
                    unset($msgProcesso);
                    
                    // Atualiza o evento a ser excluído com o número do envio de exclusão
                    $envio = $this->setEventoExclusaoEventoOrigem($this->hcmParameters->getNrorgTrabalho(), $envioexcl->getNrenvio(), $row['NRENVIO'], 'N', $row['NREVENTO']);
                }
                if (empty($msgValidacao) && empty($eventosLote)) {
                    throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
                }
    
                $xmlLoteEventos = $this->getXmlLote($idGrupo, $paramESocial, $dsVersao, $eventosLote);
    
                $retorno = $this->enviarLote($eventosLote, $xmlLoteEventos);
                
                $codRetorno = $retorno['COD'];
                $nrRecibo = $retorno['REC'];
                $msgRetorno =  'Envio da Retificação: '. $retorno['MSG'];
                
                // Caso o evento de exclusão/retificação seja enviado com sucesso realiza a consulta automática
                if ($codRetorno == '201') {
                    $this->entityManager->flush();
                    $this->entityManager->commit();
                    $this->entityManager->beginTransaction();
                    $xmlConsulta = $this->getXmlConsulta($nrRecibo, $dsVersao);
                    $retornoConsulta = $this->consultarEvento($nrRecibo, $xmlConsulta);
                    $codRetornoConsulta = $retornoConsulta['COD'];
                    $msgRetornoConsulta = $retornoConsulta['MSG'];
                    $tmpEsperaConsulta = $retornoConsulta['TMP'];
                    if ($codRetornoConsulta == '101') {
                        // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                        sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                        $xmlConsulta = $this->getXmlConsulta($nrRecibo, $dsVersao);
                        $retornoConsulta = $this->consultarEvento($nrRecibo, $xmlConsulta);
                        $codRetornoConsulta = $retornoConsulta['COD'];
                        $msgRetornoConsulta = $retornoConsulta['MSG'];
                        $tmpEsperaConsulta = $retornoConsulta['TMP'];
                    }
                }
                $msgRetorno = $msgRetorno .' / Consulta da Retificação: '. $codRetornoConsulta . '-' . $msgRetornoConsulta . ($tmpEsperaConsulta ? '(Tempo de Espera:'.$tmpEsperaConsulta.')' : '');
            
                 if (count($msgValidacao) > 0) {
                    $this->entityManager->flush();
                    $this->entityManager->commit();
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', $msgValidacao));
                } elseif ($msgRetorno) {
                    $this->entityManager->flush();
                    $this->entityManager->commit();
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', [$msgRetorno]));
                } else {
                    $this->entityManager->rollback();
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', ["Serviço de reintegração não disponível"]));
                }
            }
        }catch(\Exception $e) {
            $this->entityManager->rollback();
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function envioEsocialDescentralizado ($dataSet, $response){
        try{
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = array_key_exists('DTMESCOMPETENC', $dataSet) ? $dataSet['DTMESCOMPETENC'] : $this->hcmParameters->getDtCompetencia();
            $nrtipoestrutlegal = $this->hcmParameters->getTipoEstruturaLegalOrg();
            $paramESocial = null;
            $eventosLote = array();
            $msgValidacao = array();
            $idGrupo = null;
            $dsVersao = null;
            
            if(array_key_exists('NRVINCULOM', $dataSet) && !empty($dataSet['NRVINCULOM'])){
                $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($nrorg, $dataSet['NRVINCULOM'], $dtmescompetenc);
                $nrestrutLegal = null;
                if($vinculo){
                    $nrestrutLegal = $vinculo->getNrestrutlegal();
                }
                $dataSet['NRESTRUTURAM'] = array_key_exists('NRESTRUTURAM', $dataSet)? $dataSet['NRESTRUTURAM'] : $nrestrutLegal;
                $paramESocial = $this->getParamESocial($dataSet['NRESTRUTURAM']);
                $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $dataSet['NRESTRUTURAM']);
    
                // Verifica se a estrutura tem parâmetro do eSocial
                if (empty($paramESocial)) {
                   MsgProcesso::getInstance()->addError($this->arrayMensagens['HCM09000_ESO_PARAMETRO_NOT_FOUND_ERROR']);
                }else{
                    $this->setDadosTransmissao($dataSet['NRESTRUTURAM']);
                    $evento = $this->getEventoPorNomeVersao($dataSet['NMEVENTO'], $paramESocial['DSVERSAO']);
                    $dsVersao = str_replace('.','-',$paramESocial['DSVERSAO']);
                    $idGrupo = $evento->getIdgrupo();
                    
                    $estrutura = $this->getEstruturasESocial($nrorg, $dtmescompetenc, $nrtipoestrutlegal, $idGrupo, 'AND NRESTRUTURAM = '.$dataSet['NRESTRUTURAM'].'')[0];
                    $dataSet['CDCNPJEMPREGADOR'] = $estrutura['CDCNPJESTRUT'];
                    $dataSet['NRTPAMBIENTE'] = $paramESocial['NRTPAMBIENTE'];
                    
                    $arrItens = $this->getItensParaEnvioEvento($dataSet['NMEVENTO'], $idGrupo, $dataSet)[0];
                    if (empty($arrItens)) {
                        MsgProcesso::getInstance()->addError($this->arrayMensagens['HCM09000_ESO_ENVIOS_NOT_FOUND_ERROR']);
                    }else{
                        if($dataSet['NMEVENTO'] == 'S-2205' || $dataSet['NMEVENTO'] == 'S-2206'){
                            $idOperacao = 'A';
                        }else{
                            $idOperacao = 'I';
                        }
                        
                        try {
                            $this->beforeInsertEnvioCadastros($dataSet['NRESTRUTURAM'], $dataSet['NRTPAMBIENTE'], $evento->getNrevento(), $dataSet['NRCHAVE'], $dataSet['NRTABELA'], $idOperacao, $arrItens['DTMESCOMPETENC']);
                        } catch(\Exception $e) {
                            throw $e;
                        }
                        
                        //Insere o envio para posteriormente fazer o envio
                        try {
                            $this->entityManager->beginTransaction();
                            
                            $nrenvio = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_ENVIOCADASTROS', 1, 12, true);
                            $envio = $this->novoEsoEnvioCadastro(
                                $nrenvio, $evento->getNrevento(), $dataSet['NRESTRUTURAM'], $dataSet['NRCHAVE'], $dataSet['NRTABELA'], DateUtil::getDataDeString($arrItens['DTMESCOMPETENC'], DateUtil::FORMATO_BRASILEIRO_DATAHORA, true), $idOperacao, $paramESocial['NRTPAMBIENTE'], null, $dataSet['NRVINCULOM']
                            );
                            
                            $this->entityManager->flush();
                            $this->entityManager->commit();
                        } catch (\Exception $e) {
                            $this->entityManager->rollback();
                            throw $e;
                        }
                        
                        $itemEnvio = $envio->toArray();
                        $itemEnvio['NMEVENTO'] = $evento->getNmevento();
                        $itemEnvio['IDGRUPO'] = $evento->getIdgrupo();
                        $itemEnvio['DTMESCOMPETENC'] = is_object($arrItens['DTMESCOMPETENC']) ? $arrItens['DTMESCOMPETENC']->format('d/m/Y H:i:s') : $arrItens['DTMESCOMPETENC'];
                        $itemEnvio['DSVERSAO'] = $dsVersao;
                        $itemEnvio['NRINSCRICAOPARC'] = $estrutura['CDCNPJESTRUT'];
                        $itemEnvio['NRTPAMBIENTE'] = $paramESocial['NRTPAMBIENTE'];
                        $itemEnvio['DTINCLUSAO'] = $this->hcmParameters->getDataAtual();
                        $itemEnvio['DTULTATU'] = $this->hcmParameters->getDataAtual();
                        $itemEnvio['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                        $itemEnvio['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
                        $itemEnvio['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                        $itemEnvio['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
                        
                        //Realiza o envio e a consulta
                        try {
                            $this->entityManager->beginTransaction();
                            
                            $xmlEvento = $this->getXmlEvento($dsVersao, $itemEnvio);
                        
                            $this->beforeEnvioEvento($nrorg, $itemEnvio['NRESTRUTURAM'], $itemEnvio['NRTPAMBIENTE'], $paramESocial, $evento->getNrevento(), $itemEnvio['NRCHAVE'], $itemEnvio['NRTABELA'], $itemEnvio['IDOPERACAO'], $itemEnvio['DTMESCOMPETENC']);
                            
                            $xmlLoteEventos = $this->getXmlLote($idGrupo, $paramESocial, $dsVersao, [$xmlEvento]);
                
                            $retorno = $this->enviarLote(array($xmlEvento), $xmlLoteEventos);
                            
                            if($retorno){
                                $codRetorno = $retorno['COD'];
                                $nrRecibo = $retorno['REC'];
                                $msgRetorno =  'Envio : '. $retorno['MSG'];
                                $envio->setNrrecibo($nrRecibo);
                                $envio->setDtresposta(DateUtil::getDataDeString($this->hcmParameters->getDataAtual()));
                                
                                // Caso o evento de exclusão/retificação seja enviado com sucesso realiza a consulta automática
                                if ($codRetorno == '201') {
                                    $this->entityManager->flush();
                                    $this->entityManager->commit();
                                    $this->entityManager->beginTransaction();
                                    $xmlConsulta = $this->getXmlConsulta($nrRecibo, $dsVersao);
                                    $retornoConsulta = $this->consultarEvento($nrRecibo, $xmlConsulta);
                                    $codRetornoConsulta = $retornoConsulta['COD'];
                                    $msgRetornoConsulta = $retornoConsulta['MSG'];
                                    $tmpEsperaConsulta = $retornoConsulta['TMP'];
                                    if ($codRetornoConsulta == '101') {
                                        // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                                        sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                                        $xmlConsulta = $this->getXmlConsulta($nrRecibo, $dsVersao);
                                        $retornoConsulta = $this->consultarEvento($nrRecibo, $xmlConsulta);
                                        $codRetornoConsulta = $retornoConsulta['COD'];
                                        $msgRetornoConsulta = $retornoConsulta['MSG'];
                                        $tmpEsperaConsulta = $retornoConsulta['TMP'];
                                    }
                                }
                                $msgRetorno = $msgRetorno .' / Consulta do envio: '. $codRetornoConsulta . '-' . $msgRetornoConsulta . ($tmpEsperaConsulta ? '(Tempo de Espera:'.$tmpEsperaConsulta.')' : '');
                            }
                            
                            $this->entityManager->flush();
                            $this->entityManager->commit();
                        } catch (\Exception $e) {
                            $this->entityManager->rollback();
                            throw $e;
                        }
                    }
                }
                
                foreach ($msgProcesso->getMessages() as $msg) {
                    $msgValidacao[] = array(
                        'NMEVENTO' => $dataSet['NMEVENTO'],
                        'NRCHAVE' => $dataSet['NRCHAVE'],
                        'MSG' => $msg['MSG'],
                        'TYPE' => $msg['TYPE'],
                    );
                }
                $msgProcesso->clear();
                unset($msgProcesso);
                if (count($msgValidacao) > 0) {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', $msgValidacao));
                } elseif ($msgRetorno) {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', [['MSG' => $msgRetorno, 'TYPE' => 'S']]));
                } else {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', [['MSG' => "Serviço de envio não disponível", 'TYPE' => 'E']]));
                }
            }else{
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', [['MSG' => "Vínculo não identificado.", 'TYPE' => 'E']]));
            }
            
        }catch(\Exception $e) {
            if(array_key_exists($e->getMessage(), $this->arrayMensagens)){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', [['MSG' => $this->arrayMensagens[$e->getMessage()], 'TYPE' => 'S']]));
            }else{
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function envioEsocialDescentralizadoCalculo ($dataSet, $response){
        try{
            Inicializa::Container($this->entityManager, $this->hcmParameters, $dataSet['DTPAGAMENTO']);
            Inicializa::carregaVariaveis();
            
            $this->_vinculom = TkDI::getContainer()->vinculom;
            
            $vinculos = $this->_vinculom->retornaVinculosPorTipoCalculo($dataSet['TP_CALCULO'],$dataSet['NRORG'], $this->hcmParameters->getDtcompetencia(), $dataSet['DSINICIO'], $dataSet['DSFIM'],
                                                                        DateUtil::getDataDeString(substr($dataSet['DTRESCISAOINI'],0, 10)), DateUtil::getDataDeString(substr($dataSet['DTRESCISAOFIM'], 0, 10)), 
                                                                        $dataSet['DSORDEM'], $dataSet['NRESTRUTURAM']);
            if($vinculos){
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $dtmescompetenc = array_key_exists('DTMESCOMPETENC', $dataSet) ? $dataSet['DTMESCOMPETENC'] : $this->hcmParameters->getDtCompetencia();
                $nrtipoestrutlegal = $this->hcmParameters->getTipoEstruturaLegalOrg();
                $paramESocial = null;
                $eventosLote = array();
                $msgValidacao = array();
                $idGrupo = null;
                $dsVersao = null;
                $idestrutura = false;
                
                foreach($vinculos as $index => $vinculo){
                    $dataSet['NRVINCULOM'] = $vinculo->getNrvinculom();
                    $dataSet['NRESTRUTURAM'] = $vinculo->historico->getNrestrutlegal();
                    if($dataSet['TP_CALCULO'] == 6 && $vinculo->getNrtipovinculom() == 1){
                        $dataSet['NMEVENTO'] = 'S-2299';
                        $dataSet['NRCHAVE'] = $vinculo->getNrvinculom();
                        $dataSet['NRTABELA'] = null;
                    }else if($dataSet['TP_CALCULO'] == 6){
                        $dataSet['NMEVENTO'] = 'S-2399';
                        $dataSet['NRCHAVE'] = $vinculo->getNrvinculom();
                        $dataSet['NRTABELA'] = null;
                    }else if($dataSet['TP_CALCULO'] == 2){
                        $dataSet['NMEVENTO'] = 'S-2230';
                        $gozoFerias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->retornaGozosFeriasEntreDatas($dataSet['NRORG'], $dataSet['NRVINCULOM'], DateUtil::getDatadeString($dataSet['DTINIGOZOFERIAS']), DateUtil::getDatadeString($dataSet['DTFIMGOZOFERIAS']))[0];
                        $dataSet['NRGOZOFERIAS'] = $gozoFerias->getNrgozoferias();
                        $dataSet['NRCHAVE'] = $dataSet['NRGOZOFERIAS'];
                        $dataSet['NRTABELA'] = 2;
                        $dataSet['DTMESCOMPETENC'] = substr($dataSet['DTMESCOMPETENC'], 0, 10);
                    }
                    
                    if($nrestruturam != $dataSet['NRESTRUTURAM']){
                        $paramESocial = $this->getParamESocial($dataSet['NRESTRUTURAM']);
                        $this->setDadosTransmissao($dataSet['NRESTRUTURAM']);
                        $nrestruturam = $dataSet['NRESTRUTURAM'];
                        $idestrutura = true;
                    }else{
                        $idestrutura = false;
                    }
                    
                    $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $dataSet['NRENVIO']);
                   
                    // Verifica se a estrutura tem parâmetro do eSocial
                    if (empty($paramESocial)) {
                       MsgProcesso::getInstance()->addError($this->arrayMensagens['HCM09000_ESO_PARAMETRO_NOT_FOUND_ERROR']);
                    }else{
                        if(($dataSet['NMEVENTO'] == 'S-2230' && $paramESocial['IDAPROVACALCFERIAS'] == 'S') || (($dataSet['NMEVENTO'] == 'S-2399'|| $dataSet['NMEVENTO'] == 'S-2299') && $paramESocial['IDAPROVACALCRESC'] == 'S')){
                            if($idestrutura){
                               MsgProcesso::getInstance()->addError('Estrutura exige aprovação do cálculo para enviar ao ESocial. Favor verificar os Parametros do ESocial ou aprovar os cálculos da estrutura '.$dataSet['NRESTRUTURAM'].'.');
                            }
                        }else{
                            
                            $this->setDadosTransmissao($dataSet['NRESTRUTURAM']);
                            $evento = $this->getEventoPorNomeVersao($dataSet['NMEVENTO'], $paramESocial['DSVERSAO']);
                            $dsVersao = str_replace('.','-',$paramESocial['DSVERSAO']);
                            $idGrupo = $evento->getIdgrupo();
                            
                            $estrutura = $this->getEstruturasESocial($nrorg, $dtmescompetenc, $nrtipoestrutlegal, $idGrupo, 'AND NRESTRUTURAM = '.$dataSet['NRESTRUTURAM'].'')[0];
                            $dataSet['CDCNPJEMPREGADOR'] = $estrutura['CDCNPJESTRUT'];
                            $dataSet['NRTPAMBIENTE'] = $paramESocial['NRTPAMBIENTE'];
                            
                            $arrItens = $this->getItensParaEnvioEvento($dataSet['NMEVENTO'], $idGrupo, $dataSet)[0];
                            if (empty($arrItens)) {
                                MsgProcesso::getInstance()->addError($this->arrayMensagens['HCM09000_ESO_ENVIOS_NOT_FOUND_ERROR']);
                            }else{
                                if($dataSet['NMEVENTO'] == 'S-2205' || $dataSet['NMEVENTO'] == 'S-2206'){
                                    $idOperacao = 'A';
                                }else{
                                    $idOperacao = 'I';
                                }
                        
                                try {
                                    $this->beforeInsertEnvioCadastros($dataSet['NRESTRUTURAM'], $dataSet['NRTPAMBIENTE'], $evento->getNrevento(), $dataSet['NRCHAVE'], $dataSet['NRTABELA'], $idOperacao, $arrItens['DTMESCOMPETENC']);
                                } catch(\Exception $e) {
                                    throw $e;
                                }
                                
                                //Insere o envio para posteriormente fazer o envio
                                try {
                                    $this->entityManager->beginTransaction();
                                
                                    $nrenvio = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_ENVIOCADASTROS', 1, 12, true);
                                    $envio = $this->novoEsoEnvioCadastro(
                                        $nrenvio, $evento->getNrevento(), $dataSet['NRESTRUTURAM'], $dataSet['NRCHAVE'], $dataSet['NRTABELA'], DateUtil::getDataDeString($arrItens['DTMESCOMPETENC'], DateUtil::FORMATO_BRASILEIRO_DATAHORA, true), $idOperacao, $paramESocial['NRTPAMBIENTE'], null, $dataSet['NRVINCULOM']
                                    );
                            
                                    $this->entityManager->flush();
                                    $this->entityManager->commit();
                                } catch (\Exception $e) {
                                    $this->entityManager->rollback();
                                    throw $e;
                                }
                                
                                $itemEnvio = $envio->toArray();
                                $itemEnvio['NMEVENTO'] = $evento->getNmevento();
                                $itemEnvio['IDGRUPO'] = $evento->getIdgrupo();
                                $itemEnvio['DTMESCOMPETENC'] = is_object($arrItens['DTMESCOMPETENC']) ? $arrItens['DTMESCOMPETENC']->format('d/m/Y H:i:s') : $arrItens['DTMESCOMPETENC'];
                                $itemEnvio['DSVERSAO'] = $dsVersao;
                                $itemEnvio['NRINSCRICAOPARC'] = $estrutura['CDCNPJESTRUT'];
                                $itemEnvio['NRTPAMBIENTE'] = $paramESocial['NRTPAMBIENTE'];
                                $itemEnvio['DTINCLUSAO'] = $this->hcmParameters->getDataAtual();
                                $itemEnvio['DTULTATU'] = $this->hcmParameters->getDataAtual();
                                $itemEnvio['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                                $itemEnvio['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
                                $itemEnvio['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                                $itemEnvio['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
                            
                                //Realiza o envio e a consulta
                                try {
                                    $this->entityManager->beginTransaction();
                                
                                    $xmlEvento = $this->getXmlEvento($dsVersao, $itemEnvio);
                                    if ($xmlEvento) {
                                        $eventosLote[] = $xmlEvento;
                                    }
                                
                                    $this->beforeEnvioEvento($nrorg, $itemEnvio['NRESTRUTURAM'], $itemEnvio['NRTPAMBIENTE'], $paramESocial, $evento->getNrevento(), $itemEnvio['NRCHAVE'], $itemEnvio['NRTABELA'], $itemEnvio['IDOPERACAO'], $itemEnvio['DTMESCOMPETENC']);
                             
                                    if (empty($eventosLote)) {
                                        MsgProcesso::getInstance()->addError($this->arrayMensagens['HCM09000_ESO_LOTE_VAZIO_ERROR']);
                                    }else{
                                        $xmlLoteEventos = $this->getXmlLote($idGrupo, $paramESocial, $dsVersao, $eventosLote);
                            
                                        $retorno = $this->enviarLote($eventosLote, $xmlLoteEventos);
                                        
                                        if($retorno){
                                            $codRetorno = $retorno['COD'];
                                            $nrRecibo = $retorno['REC'];
                                            $msgRetorno =  'Envio : '. $retorno['MSG'];
                                            $envio->setNrrecibo($nrRecibo);
                                            $envio->setDtresposta(DateUtil::getDataDeString($this->hcmParameters->getDataAtual()));
                                            
                                            // Caso o evento de exclusão/retificação seja enviado com sucesso realiza a consulta automática
                                            if ($codRetorno == '201') {
                                                $this->entityManager->flush();
                                                $this->entityManager->commit();
                                                $this->entityManager->beginTransaction();
                                                $xmlConsulta = $this->getXmlConsulta($nrRecibo, $dsVersao);
                                                $retornoConsulta = $this->consultarEvento($nrRecibo, $xmlConsulta);
                                                $codRetornoConsulta = $retornoConsulta['COD'];
                                                $msgRetornoConsulta = $retornoConsulta['MSG'];
                                                $tmpEsperaConsulta = $retornoConsulta['TMP'];
                                                if ($codRetornoConsulta == '101') {
                                                    // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                                                    sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                                                    $xmlConsulta = $this->getXmlConsulta($nrRecibo, $dsVersao);
                                                    $retornoConsulta = $this->consultarEvento($nrRecibo, $xmlConsulta);
                                                    $codRetornoConsulta = $retornoConsulta['COD'];
                                                    $msgRetornoConsulta = $retornoConsulta['MSG'];
                                                    $tmpEsperaConsulta = $retornoConsulta['TMP'];
                                                }
                                            }
                                            $msgRetorno = $msgRetorno .' / Consulta do envio: '. $codRetornoConsulta . '-' . $msgRetornoConsulta . ($tmpEsperaConsulta ? '(Tempo de Espera:'.$tmpEsperaConsulta.')' : '');
                                        }
                                    }
                                
                                    $this->entityManager->flush();
                                    $this->entityManager->commit();
                                } catch (\Exception $e) {
                                    $this->entityManager->rollback();
                                    throw $e;
                                }
                            }
                        }
                    }
                }
                
                
                foreach ($msgProcesso->getMessages() as $msg) {
                    $msgValidacao[] = array(
                        'NMEVENTO' => $dataSet['NMEVENTO'],
                        'NRCHAVE' => $dataSet['NRCHAVE'],
                        'MSG' => $msg['MSG'],
                        'TYPE' => $msg['TYPE'],
                    );
                }
                
                $msgProcesso->clear();
                unset($msgProcesso);
                
                if (count($msgValidacao) > 0) {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', $msgValidacao));
                } elseif ($msgRetorno) {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', [['MSG' => $msgRetorno, 'TYPE' => 'S']]));
                } else {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', [['MSG' => "Serviço de envio não disponível", 'TYPE' => 'E']]));
                }
            }
        }catch(\Exception $e) {
            if(array_key_exists($e->getMessage(), $this->arrayMensagens)){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', [['MSG' => $this->arrayMensagens[$e->getMessage()], 'TYPE' => 'S']]));
            }else{
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function setEnvio2200ManualAdmissao($arrayEnvio){
        try{
            $this->entityManager->beginTransaction();
            
            $evento2200 = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nmevento' => 'S-2200', 'dsversao' => $arrayEnvio[0]['DSVERSAO']));
            $evento2300 = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nmevento' => 'S-2300', 'dsversao' => $arrayEnvio[0]['DSVERSAO']));
            
            foreach($arrayEnvio as $row){
                if($row['NREVENTO'] == 'S-2200') {
                    $envio = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosCadastrosEvento($this->hcmParameters->getNrorgTrabalho(), 'S-2200', $row['NRESTRUTURAM'], $row['NRTPAMBIENTE'], 'S', $row['NRCHAVE']);
                } else {
                    $envio = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosCadastrosEvento($this->hcmParameters->getNrorgTrabalho(), 'S-2300', $row['NRESTRUTURAM'], $row['NRTPAMBIENTE'], 'S', $row['NRCHAVE']);
                }
                if(empty($envio)){
                    $esoEnviocadastros = Repositories::ESO_ENVIOCADASTROS;
                    
                    $envio = new $esoEnviocadastros;
                    
                    if($row['NREVENTO'] == 'S-2200') {
                        $envio->setNrevento($evento2200->getNrevento());
                    } else {
                        $envio->setNrevento($evento2300->getNrevento());
                    }
                    $envio->setNrestruturam($row['NRESTRUTURAM']);
                    $envio->setNrvinculom($row['NRVINCULOM']);
                    $envio->setNrchave($row['NRCHAVE']);
                    $envio->setNrtabela($row['NRTABELA']);
                    $envio->setDtmescompetenc(DateUtil::getDataDeString($row['DTMESCOMPETENC'], DateUtil::FORMATO_BRASILEIRO, true));
                    $envio->setIdoperacao($row['IDOPERACAO']);
                    $envio->setIdfechasmoviment('N');
                    $envio->setIdstatus($row['IDSTATUS']);
                    $envio->setNrtpambiente($row['NRTPAMBIENTE']);
                    $envio->setNrenvioexcl(null);
                    $envio->setDsobservacao($row['DSOBSERVACAO']);
                    $envio->setIdativo($row['IDATIVO']);
                    $envio->setNrorg($this->hcmParameters->getNrorgTrabalho());
                    $envio->setDtinclusao(DateUtil::getDataAtual());
                    $envio->setCdoperinclusao($this->hcmParameters->getCdoperador());
                    $envio->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                 
                    $envio->setNrenvio(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_ENVIOCADASTROS', 1, 12, true));

                    $this->entityManager->persist($envio);
                    $this->entityManager->flush();
                }
            }
            
            $this->entityManager->commit();
        }catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));	 
        }
    }

}