<?php

namespace HCM\PTO\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\Util\Repositories;
use HCM\Geral\Repository\VinculoRepository;

class OcorrenciaFrequenciaController extends Crud {

    protected $parameterBagColumns = array('NRORG', 'DTMESCOMPETENC', 'NRORGPADRAO', 'NRPERIODOAPURACAO', 'NRHORDIAM', "P_DTMESCOMPETENC", 'NRESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST');
    protected $periodoApuracao;
    protected $marcacaoPontoService;
    protected $codigoapuracao;
    
    protected const COLLUMNS_TO_BE_ALTERED_BEFORE_APPLY_FILTER = ['NRESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST'];
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, EntityManager $entityManager, HCMParameters $hcmParameters,\HCM\PTO\Services\OcorrenciaFrequenciaService $ocorrenciaFrequenciaService, \HCM\PTO\Services\PeriodoApuracaoService $periodoApuracao,\HCM\PTO\Services\MarcacaoPontoService $marcacaoPontoService, \HCM\Geral\Services\SituacaoFuncionalService $situacaoFuncionalService, VinculoRepository $vinculoRepository){
        $this->periodoApuracao = $periodoApuracao;
        $this->parameterBag = $parameterBag;
        $this->entityManager = $entityManager;
        $this->hcmParameters   = $hcmParameters;
        $this->ocorrenciaFrequenciaService = $ocorrenciaFrequenciaService;
        $this->marcacaoPontoService = $marcacaoPontoService;
        $this->situacaoFuncionalService = $situacaoFuncionalService;
        $this->vinculoRepository = $vinculoRepository;
        parent::__construct($dataSourceManager);
    }

    public function getOcorrencia (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($conditions as $filter) {
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }
            $results = $this->periodoApuracao->getPeriodoApuracaoAtual($parameters, $conditions);
            $this->codigoapuracao = array();
            foreach($results[0] as $index => $result){
                $this->codigoapuracao[$index] = $result["NRPERIODOAPURACAO"];
            }
            $filterData = $request->getFilterCriteria();
            $filterData->addCondition('NRPERIODOAPURACAO', $this->codigoapuracao, '=');
            $filterData->setDataSourceName($this->dataSourceName = "getOcorrenciaFrequencia");
            $filterData = Util::createNewFilterColumnConditionsToCommaImploded($filterData, self::COLLUMNS_TO_BE_ALTERED_BEFORE_APPLY_FILTER);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getMarcacoesPonto($request, $response) {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }
            $marcacaoPontos = $this->marcacaoPontoService->getMarcacoesPonto($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getMarcacoesPonto', $marcacaoPontos));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getHoradiah (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName = "gpeHordiah_getHoradiah");
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getSaida (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName = "gpeIntervalosHorario_getIntervaloHorarioSaida");
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {
        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $this->parameterBag->set('NRPERIODOAPURACAO', $this->codigoapuracao);
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    public function setOcorrencia(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "ptoProgocorrencia_setProgocorrencia";
            $ocorrenciaData = array();
            foreach($request->getDataSet()->getRows() as $key => &$row) {
                if(array_key_exists('NRPROGOCORRENCIA', $row)){
                    if(empty($row['NRPROGOCORRENCIA'])){
                        $row['NRPROGOCORRENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_PROGOCORRENCIA', 1, 12, true);
                    }
                } else {
                    $row['NRPROGOCORRENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_PROGOCORRENCIA', 1, 12, true);
                }
                if(!array_key_exists('NRMOTIVOOCORFREQ', $row)){
                    $row['NRMOTIVOOCORFREQ'] = null;
                }
                if(!array_key_exists('NRHORDIAM', $row)){
                    $row['NRHORDIAM'] = null;
                }
                if(!array_key_exists('NRINTERVALOHORARIO', $row)){
                    $row['NRINTERVALOHORARIO'] = null;
                }
                 if(!array_key_exists('NRMARCACAOPONTO', $row)){
                    $row['NRMARCACAOPONTO'] = null;
                }
                if(!array_key_exists('NRSITUFREQUENCIA', $row)){
                    $row['NRSITUFREQUENCIA'] = null;
                }
                if(!array_key_exists('NRSITUFREQGATILHO', $row)){
                    $row['NRSITUFREQGATILHO'] = null;
                }
                
                // Tratativas para coletar dados dos dados anteriores da ocorrencia para manipulacao
                $ocorrenciaData[$key]['OCORRENCIA_DATA'] = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->findOneBy(
                    array(
                        'nrorg'             => $row['NRORG'],
                        'nrprogocorrencia'  => $row['NRPROGOCORRENCIA']
                    )
                );
                
                $ocorrenciaData[$key]['ROW'] = $row;
                $ocorrenciaData[$key]['OPERATION'] = $row['__is_new'] ? Util::DATA_OPERATION_INSERT : Util::DATA_OPERATION_UPDATE;
                
            }
            
            parent::save($request, $response);
            
            // After save
            $this->afterSaveOcorrenciaFrequencia($ocorrenciaData);
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     *  Rotina invocada no momento posterior a sinronizacao da ocorrencia com a base de dados
     * 
     *  @param  Array $ocorrenciaData   [['ROW'] => [], ['OCORRENCIA_DATA'] => [], ['OPERATION'] => string]
     * 
     */
    private function afterSaveOcorrenciaFrequencia($ocorrenciaData) {
        
        // Tratativas para cadastro automatico de situacoes funcionais
        foreach($ocorrenciaData as $data) {
            $this->situacaoFuncionalService->afterAlterOcorrenciaSetAlteSituFunc(
                $data['ROW']['NRVINCULOM'],
                $data['ROW'],
                $data['OCORRENCIA_DATA'],
                $data['OPERATION']
            );
        }
    }
    
     public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "ptoProgocorrencia_setProgocorrencia";
            $previousOcorrenciaData = array();
            $ocorrenciaData = array();
            
            // Tratativas para coletar dados dos dados anteriores da ocorrencia para manipulacao
            foreach($request->getDataSet()->getRows() as $key => $row) {
                
                $ocorrenciaData[$key]['OCORRENCIA_DATA'] = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->findOneBy(
                    array(
                        'nrorg'             => $row['NRORG'],
                        'nrprogocorrencia'  => $row['NRPROGOCORRENCIA']
                    )
                );
                
                $ocorrenciaData[$key]['ROW'] = $row;
            }
            
            parent::delete($request, $response);
            
            // After delete
            foreach($ocorrenciaData as $data) {
                
                // Tratativa para manipular as situacoes funcionais anteriormente criadas automaticamente
                $this->situacaoFuncionalService->afterAlterOcorrenciaSetAlteSituFunc(
                    $data['ROW']['NRVINCULOM'],
                    [],
                    $data['OCORRENCIA_DATA'],
                    Util::DATA_OPERATION_DELETE
                );
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function alteraStatusOcorrencia(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            $nrprogocorrencia       = $row['NRPROGOCORRENCIA'];
            $idprogocorconfirmada   = $row['IDPROGOCORCONFIRMADA'];
            $dsobservacaorecusa     = $row['DSOBSERVACAORECUSA'];
            
            $statusAlteradoComSucesso = $this->ocorrenciaFrequenciaService->alteraStatusOcorrencia($nrprogocorrencia, $idprogocorconfirmada, $dsobservacaorecusa);
            
            $dataSet = array('success' => $statusAlteradoComSucesso);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('alteraStatusOcorrencia', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function downloadArquivosOcorrencia(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            $nrprogocorrencia = $row['NRPROGOCORRENCIA'];
            
            $arquivoOcorrenciaNome = $this->ocorrenciaFrequenciaService->downloadArquivosOcorrencia($nrprogocorrencia);
            
            $dataSet = array('FILENAME' => $arquivoOcorrenciaNome);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('downloadArquivosOcorrencia', $dataSet));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function geraOcorrenciaAlmoco(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            
            $result = $this->ocorrenciaFrequenciaService->geraOcorrenciasAlmoco($rows);
            
            $dataSet = array('success' => $result);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('data', $dataSet));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function setCadastroAutomaticoOcorrencia(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach($request->getDataSet()->getRows() as $key => &$row) {
                $erro = false;
                
                if( count($row['NRVINCULOM']) > 0 ) {
                    foreach ($row['NRVINCULOM'] as $vinculo) {
                        $row['NRPROGOCORRENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_PROGOCORRENCIA', 1, 12, true);
                        $params = array( 'ROW'=>$row, 'VINCULO'=>$vinculo );
                        
                        $this->ocorrenciaFrequenciaService->setOcorrencia($params);
                    }
                } else if( count($row['NRESTRUTLEGAL_LIST']) > 0 ) {
                    $vinculos = $this->vinculoRepository->getVinculosByEstruturas($row['NRORG'], $this->hcmParameters->getDtCompetencia(), $row['NRESTRUTLEGAL_LIST'], $row['NRESTRUTGEREN_LIST'], null, $row['NRTIPOVINCULOM']);
                    foreach ($vinculos as $vinculo) {
                        $row['NRPROGOCORRENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_PROGOCORRENCIA', 1, 12, true);
                        $params = array( 'ROW'=>$row, 'VINCULO'=>$vinculo['NRVINCULOM'] );
                        
                        $this->ocorrenciaFrequenciaService->setOcorrencia($params);
                    }
                } else if( count($row['NRESTRUTGEREN_LIST']) > 0 ) {
                    $vinculos = $this->vinculoRepository->getVinculosByEstruturas($row['NRORG'], $this->hcmParameters->getDtCompetencia(), $row['NRESTRUTLEGAL_LIST'], $row['NRESTRUTGEREN_LIST'], null, $row['NRTIPOVINCULOM']);
                    foreach ($vinculos as $vinculo) {
                        $row['NRPROGOCORRENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_PROGOCORRENCIA', 1, 12, true);
                        $params = array( 'ROW'=>$row, 'VINCULO'=>$vinculo['NRVINCULOM'] );
                        
                        $this->ocorrenciaFrequenciaService->setOcorrencia($params);
                    }
                } else {
                    $erro = true;
                }
            }
            
            if ($erro == false) {
                $response->addNotification(new Notification("Cadastro Automático Realizado Com Sucesso!", Notification::TYPE_SUCCESS));
            } else {
                $response->addNotification(new Notification("Erro no cadastro automático!", Notification::TYPE_ERROR));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function atualizaFeriado(DTO\Request\Row $request, DTO\Response $response){
        try{
            $rows = $request->getRow();
            
            $nrVinculom     = $row['NRVINCULOM']    ?? null;
            $nrEstrutLegal  = $row['NRESTRUTLEGAL'] ?? null;
            $nrCalendario   = $row['NRCALENDARIO']  ?? null;
            
            $result = $this->ocorrenciaFrequenciaService->atualizaFeriado(
                $nrVinculom,
                $nrEstrutLegal,
                $nrCalendario
            );
            
            $dataSet = array('success' => $result);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('data', $dataSet));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
}