<?php

namespace HCM\PTO\Repository;

use Doctrine\ORM\EntityManager;

class OcorrenciaFrequenciaRepository {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    /**
     *  Rotina que aplica um script SQL responsavel por cadastrar ocorrencias do tipo "Abonar falta" para determinados vinculos em dias cadastrados no calendario da estrutura legal.
     * Por definicao, apenas os feriados apos a competencia informada serao cadastradas, com o objetivo de nao afetar periodos anteriores.
     * 
     *  @param  $nrOrg
     *  @param  $dtMesCompetenc
     *  @param  $nrVinculom
     *  @param  $nrEstrutLegal
     *  @param  $nrCalendario
     * 
     *  @return bool
     * 
     */
    public function atualizaFeriado(
        $nrOrg,
        $dtMesCompetenc,
        $nrVinculom         = null,
        $nrEstrutLegal      = null,
        $nrCalendario       = null
    ) {
        
        $params = [
            'P_NRORG'           => $nrOrg,
            'P_DTMESCOMPETENC'  => $dtMesCompetenc,
            'P_NRVINCULOM'      => $nrVinculom,
            'P_NRESTRUTLEGAL'   => $nrEstrutLegal,
            'P_NRCALENDARIO'    => $nrCalendario,
        ];
        
        $query = $this->entityManager->getConnection()->prepare("
            DECLARE
                V_NRORG                         NUMBER  := :P_NRORG;
                V_DTMESCOMPETENC                DATE    := :P_DTMESCOMPETENC;
                V_NRTIPOPROGOCORRENCIA_ABONO    NUMBER  := 5;
                V_NRVINCULOM                    NUMBER  := :P_NRVINCULOM;
                V_NRESTRUTLEGAL                 NUMBER  := :P_NRESTRUTLEGAL;
                V_NRCALENDARIO                  NUMBER  := :P_NRCALENDARIO;
            BEGIN
               FOR C_PRINC IN
                  (
                    SELECT
                        VM.NRVINCULOM,
                        VM.NMVINCULOM,
                        VH.DTMESCOMPETENC AS DTMESCOMPETENC_VINC,
                        VH.NRESTRUTGEREN,
                        LEGAL.NMESTRUTURAH,
                        LEGAL.DTMESCOMPETENC AS DTMESCOMPETENC_LEGAL,
                        FER.DTCALENDARIO,
                        FER.DSOBSCALEND
                    FROM GPE_VINCULOM VM
                    JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                        SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH WHERE NRORG = VH.NRORG AND NRVINCULOM = VH.NRVINCULOM AND DTMESCOMPETENC <= V_DTMESCOMPETENC
                    )
                    JOIN ESTRUTURAH LEGAL ON LEGAL.NRORG = VM.NRORG AND LEGAL.NRESTRUTURAM = VH.NRESTRUTLEGAL AND LEGAL.DTMESCOMPETENC = (
                        SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH WHERE NRORG = LEGAL.NRORG AND NRESTRUTURAM = LEGAL.NRESTRUTURAM AND DTMESCOMPETENC <= V_DTMESCOMPETENC
                    )
                    LEFT JOIN OCORCALENDARIO FER ON FER.NRORG = VM.NRORG AND FER.NRCALENDARIO = LEGAL.NRCALENDARIO
                        /* GARANTE QUE APENAS OS FERIADOS EXISTENTES APOS A ADMISSAO DO VINCULO SEJAM CONSIDERADOS*/
                        AND FER.DTCALENDARIO >= VM.DTADMISSAOVINC
                        /* CONSIDERA APENAS FERIADOS A PARTIR DA COMPETENCIA ANTERIOR */
                        AND FER.DTCALENDARIO >= ADD_MONTHS(V_DTMESCOMPETENC, -1)
                    WHERE VM.NRORG = V_NRORG
                        AND (VM.NRVINCULOM      = V_NRVINCULOM      OR V_NRVINCULOM IS NULL)
                        AND (VH.NRESTRUTLEGAL   = V_NRESTRUTLEGAL   OR V_NRESTRUTLEGAL IS NULL)
                        AND (FER.NRCALENDARIO   = V_NRCALENDARIO    OR V_NRCALENDARIO IS NULL)
                        /* GARANTE QUE NAO EXISTE OCORRENCIA DE ABONO JA CADASTRADA */
                        AND NOT EXISTS (
                            SELECT 1
                            FROM PTO_PROGOCORRENCIA PO
                            WHERE PO.NRORG = VM.NRORG
                                AND PO.NRVINCULOM = VM.NRVINCULOM
                                AND TRUNC(PO.DTINICIOPROGOCOR) = TRUNC(PO.DTFIMPROGOCOR)
                                AND TRUNC(PO.DTINICIOPROGOCOR) = TRUNC(FER.DTCALENDARIO)
                                AND PO.NRTIPOPROGOCORRENCIA = 5
                                AND PO.IDTIPODIACALENDARIO = 'FERIADO'
                        )
                        /* VALIDACAO DE VINCULOS ATIVOS */
                        AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC > V_DTMESCOMPETENC)
                  )
               LOOP
                  INSERT INTO PTO_PROGOCORRENCIA (
                        NRORG, NRPROGOCORRENCIA, NRVINCULOM, DTINICIOPROGOCOR, DTFIMPROGOCOR, IDDIASSEMANA, IDTIPODIACALENDARIO, IDTIPOHORARIO,
                        NRMOTIVOOCORFREQ, NRHORDIAM, NRSITUFREQUENCIA, NRSITUFREQGATILHO, QTMINGATILHOPROGOCOR, QTMAXGATILHOPROGOCOR, VRCOEFICIENTEACRESCIMO, IDPROGOCORRENCIA,
                        QTMINIMAPROGOCOR, QTMAXIMAPROGOCOR, DSOBSERVACAO, IDATIVO, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO, NRORGULTATU,
                        DTULTATU, CDOPERULTATU, NRTIPOPROGOCORRENCIA, NRMARCACAOPONTO, NRINTERVALOHORARIO, IDPROGOCORCONFIRMADA, CDOPERVERIFICADO, DSOBSERVACAORECUSA,
                        HRPROJETARINTERVALONPROG, DSPATH, CDTABECDI, CDDIAGNOST, NRPESSOA
                    ) VALUES (
                        V_NRORG, (SELECT MAX(NRPROGOCORRENCIA) + 1 FROM PTO_PROGOCORRENCIA WHERE NRORG = V_NRORG), C_PRINC.NRVINCULOM, C_PRINC.DTCALENDARIO, C_PRINC.DTCALENDARIO, '1111111', 'FERIADO', 'N',
                        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                        NULL, NULL, C_PRINC.DSOBSCALEND, 'S', 1, SYSDATE, 1, NULL,
                        NULL, NULL, V_NRTIPOPROGOCORRENCIA_ABONO, NULL, NULL, 'S', NULL, NULL,
                        NULL, NULL, NULL, NULL, NULL
                    );
               END LOOP;
            END;"
        );

        return $query->execute($params);
    }
    
}