<?php

namespace HCM\SST\Controllers;

use HCM\SST\Services\CidService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class CidController extends Crud {

    protected $dataSourceName = 'sst_cid';
    protected $cidService;

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, CidService $cidService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->cidService = $cidService;
        parent::__construct($dataSourceManager);
    }

    public function getCid(DTO\Request\Filter $request, DTO\Response $response)
    {
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            
            $cdcid = '';
            $dscid = '';
            $idnivelcid = '';
            $idcidrestritosexo = '';
            foreach ($filters as $filter) {
                if($filter['columnName'] === 'CDCID'){
                    $cdcid = $filter['value'];
                }
                if($filter['columnName'] === 'DSCID'){
                    $dscid = $filter['value'];
                }
                if($filter['columnName'] === 'IDNIVELCID'){
                    $idnivelcid = $filter['value'];
                }
                if($filter['columnName'] === 'IDCIDRESTRITOSEXO'){
                    $idcidrestritosexo = $filter['value'];
                }
            }
            $parameters = array($cdcid,$dscid,$idnivelcid,$idcidrestritosexo);
            $result = $this->cidService->getCid($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getCid', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    //Usar em fields de select "faz paginação".
    public function getSelectCid(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
} 